library ieee;
library work;
use work.Latches_Flipflops.all;
use work.CFEBJTAG;
use work.Command_Module;
use ieee.std_logic_1164.all;
Library unisim;
use UNISIM.vcomponents.all;
--use UNISIM.vpck.all;
use UNISIM.all;


entity DCFEB_V6_FF_EMU is
port
	(

    CSP_DCFEB_LA_CTRL: inout std_logic_vector(35 downto 0);      -- bgb
	 VME_LA_TRG: in std_logic;
    FASTCLK : in std_logic;
    adc_mask: INOUT STD_LOGIC_VECTOR(11 downto 0);
	 
	 rx_dat : IN STD_LOGIC;
	 rx_clk : IN STD_LOGIC;
	 reset : IN STD_LOGIC;
	 tx_dat : OUT STD_LOGIC;
	 tp_tck: OUT STD_LOGIC;
	 tp_tms: OUT STD_LOGIC;
	 tp_tdi: OUT STD_LOGIC;
	 tp_tdo: OUT STD_LOGIC);

end DCFEB_V6_FF_EMU ;


ARCHITECTURE DCFEB_V6_FF_EMU_arch of DCFEB_V6_FF_EMU  is

-- bgb 
-- bgb  Added Chip Scope Pro Logic analyzer for CFEB JTAG signals
-- bgb 

component csp_dcfeb_la
  PORT (
    CONTROL : INOUT STD_LOGIC_VECTOR(35 DOWNTO 0);
    CLK : IN STD_LOGIC;
    DATA : IN STD_LOGIC_VECTOR(119 DOWNTO 0);
    TRIG0 : IN STD_LOGIC_VECTOR(7 DOWNTO 0)
	 );

end component;

-- Synplicity black box declaration
attribute syn_black_box : boolean;
attribute syn_black_box of csp_dcfeb_la: component is true;

-- bgb 
-- bgb End of Logic Analyzer declaration
-- bgb 

COMPONENT ff_emu IS 
	PORT
	(
		flf_p1_in :  IN  STD_LOGIC;
		flf_p2_in :  IN  STD_LOGIC;
		flf_e1_in :  IN  STD_LOGIC;
		flf_e2_in :  IN  STD_LOGIC;
		vlf_e1_in :  IN  STD_LOGIC;
		vlf_e2_in :  IN  STD_LOGIC;
		vlf_e3_in :  IN  STD_LOGIC;
		vlf_ta1_in :  IN  STD_LOGIC;
		vlf_ta2_in :  IN  STD_LOGIC;
		vlf_ta3_in :  IN  STD_LOGIC;
		vlf_tb1_in :  IN  STD_LOGIC;
		vlf_tb2_in :  IN  STD_LOGIC;
		flf_p1_out :  OUT  STD_LOGIC;
		flf_p2_out :  OUT  STD_LOGIC;
		flf_e1_out :  OUT  STD_LOGIC;
		flf_e2_out :  OUT  STD_LOGIC;
		vlf_e1_out :  OUT  STD_LOGIC;
		vlf_e2_out :  OUT  STD_LOGIC;
		vlf_e3_out :  OUT  STD_LOGIC;
		vlf_ta1_out :  OUT  STD_LOGIC;
		vlf_ta2_out :  OUT  STD_LOGIC;
		vlf_ta3_out :  OUT  STD_LOGIC;
		vlf_tb1_out :  OUT  STD_LOGIC;
		vlf_tb2_out :  OUT  STD_LOGIC;
		clk_ref :  IN  STD_LOGIC;
		rx_clk :  IN  STD_LOGIC;
		rx_dat :  IN  STD_LOGIC;
		tx_dat :  OUT  STD_LOGIC;
		msel0 :  IN  STD_LOGIC;
		msel1 :  IN  STD_LOGIC;
		msel2 :  IN  STD_LOGIC;
		reset :  IN  STD_LOGIC;
		token :  OUT  STD_LOGIC;
		clk_cmos :  OUT  STD_LOGIC
	);
END COMPONENT;


COMPONENT tdo_mux
  port(
  
    TCK : in std_ulogic;
    TDO_0C : in std_ulogic;
    TDO_17 : in std_ulogic;
    FSEL : in std_logic_vector(31 downto 0);
    TDO : out std_ulogic);

end COMPONENT;

COMPONENT BGB_BSCAN_emulator is

  port(

		CAPTURE1 : out std_ulogic;
		DRCK1    : out std_ulogic;
		RESET1   : out std_ulogic;
		SEL1     : out std_ulogic;
		SHIFT1   : out std_ulogic;
		UPDATE1  : out std_ulogic;
		RUNTEST1  : out std_ulogic;
		TDO1		: in  std_ulogic;
		
		CAPTURE2 : out std_ulogic;
		DRCK2    : out std_ulogic;
		RESET2   : out std_ulogic;
		SEL2     : out std_ulogic;
		SHIFT2   : out std_ulogic;
		UPDATE2  : out std_ulogic;
		RUNTEST2  : out std_ulogic;
		TDO2		: in  std_ulogic;

		TDO3		: in  std_ulogic;
		TDO4		: in  std_ulogic;

      TDO	: out std_ulogic;

      TCK	: in  std_ulogic;
      TDI	: in  std_ulogic;
      TMS	: in  std_ulogic;
      TRST	: in  std_ulogic
    );

end COMPONENT;

COMPONENT instr_dcd is

  port(
    TCK	: in  std_ulogic;
    DRCK	: in  std_ulogic;
    SEL	: in  std_ulogic; 
    TDI	: in  std_ulogic;
    UPDATE	: in  std_ulogic;
    SHIFT	: in  std_ulogic;
    RST	: in  std_ulogic;
    CLR	: in  std_ulogic;
    F	: out  std_logic_vector (31 downto 0);
    TDO	: out  std_ulogic
  );

end COMPONENT;

COMPONENT user_wr_reg is
  generic (
     width : integer := 12;
     def_value : std_ulogic_vector (11 downto 0) := "111111111111"
  );
  port (
    TCK	: in  std_ulogic;
    DRCK	: in  std_ulogic;
    FSEL	: in  std_ulogic;
    SEL	: in  std_ulogic;
    TDI	: in  std_ulogic;
    DSY_IN	: in  std_ulogic;
    SHIFT	: in  std_ulogic;
    UPDATE	: in  std_ulogic;
    RST	: in  std_ulogic;
    DSY_CHAIN	: in  std_ulogic;
    PO	: out  std_logic_vector (width-1 downto 0);
    TDO	: out  std_ulogic;
    DSY_OUT	: out  std_ulogic
  );    
 
end COMPONENT;

COMPONENT user_cap_reg is
  generic (
     width : integer := 16
  );
  port  (
    DRCK	: in  std_ulogic;
    FSH	: in  std_ulogic;
    FCAP	: in  std_ulogic;
    SEL	: in  std_ulogic;
    TDI	: in  std_ulogic;
    SHIFT	: in  std_ulogic;
    CAPTURE : in  std_ulogic;
    RST	: in  std_ulogic;
		PI : in std_logic_vector (width-1 downto 0);
    TDO	: out  std_ulogic
    );      
		
end COMPONENT;
 
-------------------------------------------------------------------------------


signal fsel : std_logic_vector(31 downto 0);
-- signal adc_mask : std_ulogic_vector(11 downto 0);
signal bpi_status : std_logic_vector(15 downto 0);
signal drck1, sel1, reset1, shift1, capture1, update1 : std_ulogic;
signal drck2, sel2, reset2, shift2, capture2, update2 : std_ulogic;
signal tdo_f0c, tdo_f17 : std_ulogic;
signal b_tms1, b_tdi1, b_tck1 : std_ulogic;
signal b_tms2, b_tdi2, b_tck2 : std_ulogic;
signal tdo1 : std_ulogic;
signal tdo2 : std_ulogic;
signal tdo3 : std_ulogic := '0';
signal tdo4 : std_ulogic := '0';

signal tck, tms, tdi, tdo : std_ulogic;
signal trst : std_ulogic;

-- FF_EMU signals

signal ff_emu_flf_p1_in, ff_emu_flf_p2_in: std_logic;
signal ff_emu_flf_e1_in, ff_emu_flf_e2_in: std_logic;
signal ff_emu_vlf_e1_in, ff_emu_vlf_e2_in, ff_emu_vlf_e3_in: std_logic;
signal ff_emu_vlf_ta1_in, ff_emu_vlf_ta2_in, ff_emu_vlf_ta3_in: std_logic;
signal ff_emu_vlf_tb1_in, ff_emu_vlf_tb2_in: std_logic;
signal ff_emu_flf_p1_out, ff_emu_flf_p2_out: std_logic;
signal ff_emu_flf_e1_out, ff_emu_flf_e2_out: std_logic;
signal ff_emu_vlf_e1_out, ff_emu_vlf_e2_out, ff_emu_vlf_e3_out: std_logic;
signal ff_emu_vlf_ta1_out, ff_emu_vlf_ta2_out, ff_emu_vlf_ta3_out: std_logic;
signal ff_emu_vlf_tb1_out, ff_emu_vlf_tb2_out: std_logic;
signal token, clk_cmos, clk_ref: std_logic;
signal msel0: std_logic := '0';
signal msel1: std_logic := '0';
signal msel2: std_logic := '0';

  --Declaring Logic Analyzer signals -- bgb
  signal dcfeb_la_data : std_logic_vector(119 downto 0);
  signal dcfeb_la_trig : std_logic_vector(7 downto 0);
  
-------------------------------------------------------------------------------

begin

-- JTAG test points

tp_tck <= tck;
tp_tms <= tms;
tp_tdi <= tdi;
tp_tdo <= tdo;

-- bgb 
-- bgb  Added Chip Scope Pro Logic analyzer for DCFEB emulation signals
-- bgb
 
csp_dcfeb_la_i : csp_dcfeb_la
  port map (
    CONTROL => CSP_DCFEB_LA_CTRL,
    CLK => FASTCLK,
    DATA => dcfeb_la_data,
    TRIG0 => dcfeb_la_trig
	 );
	 
-- trigger assignments (8 bits)
  dcfeb_la_trig(0) <= tck;
  dcfeb_la_trig(1) <= tms;
  dcfeb_la_trig(2) <= tdi;
  dcfeb_la_trig(3) <= tdo;
  dcfeb_la_trig(4) <= fsel(12);
  dcfeb_la_trig(5) <= fsel(23);
  dcfeb_la_trig(6) <= '0';
  dcfeb_la_trig(7) <= VME_LA_TRG;
	 
-- data assignments (120 bits)
  dcfeb_la_data(31 downto 0)   <= fsel;
  dcfeb_la_data(43 downto 32)  <= adc_mask;
  dcfeb_la_data(47 downto 44)  <= x"0";
  dcfeb_la_data(63 downto 48) <= bpi_status;
  dcfeb_la_data(64) <= tck;
  dcfeb_la_data(65) <= tms;
  dcfeb_la_data(66) <= tdi;
  dcfeb_la_data(67) <= tdo;
  dcfeb_la_data(68) <= drck1;
  dcfeb_la_data(69) <= sel1;
  dcfeb_la_data(70) <= reset1;
  dcfeb_la_data(71) <= shift1;
  dcfeb_la_data(72) <= capture1;
  dcfeb_la_data(73) <= update1;
  dcfeb_la_data(74) <= tdo1;
  dcfeb_la_data(75) <= drck2;
  dcfeb_la_data(76) <= sel2;
  dcfeb_la_data(77) <= reset2;
  dcfeb_la_data(78) <= shift2;
  dcfeb_la_data(79) <= capture2;
  dcfeb_la_data(80) <= update2;
  dcfeb_la_data(81) <= tdo2;
  dcfeb_la_data(82) <= tdo_f0c;
  dcfeb_la_data(83) <= tdo_f17;
--  dcfeb_la_data(95 downto 84) <= d_wr_reg;
--  dcfeb_la_data(111 downto 96) <= q_cap_reg;
--  dcfeb_la_data(119 downto 112) <= d_instr_reg;
  dcfeb_la_data(119 downto 84) <= x"000000000";
 
-- bgb 
-- bgb End of Logic Analyzer instantiation
-- bgb 

-- Reference clock

clk_ref <= clk_cmos; -- from QPLL

-- FF_EMU TX inputs

ff_emu_flf_p1_in <= '0';
ff_emu_flf_p2_in <= '0';
ff_emu_flf_e1_in <= shift2;
ff_emu_flf_e2_in <= tdo;
ff_emu_vlf_e1_in <= '0';

ff_emu_vlf_e2_in <= tck;
--ff_emu_vlf_e2_in <= '0';

ff_emu_vlf_e3_in <= '0';
ff_emu_vlf_ta1_in <= tck;
ff_emu_vlf_ta2_in <= '0';
ff_emu_vlf_ta3_in <= '0';
ff_emu_vlf_tb1_in <= '0';
ff_emu_vlf_tb2_in <= '0';

-- dl_jtag_tdo(1) <= flf_e2_rx;

PMAP_FF_EMU: ff_emu
  port map (
		flf_p1_in => ff_emu_flf_p1_in,
		flf_p2_in => ff_emu_flf_p2_in,
		flf_e1_in => ff_emu_flf_e1_in,
		flf_e2_in => ff_emu_flf_e2_in,
		vlf_e1_in => ff_emu_vlf_e1_in,
		vlf_e2_in => ff_emu_vlf_e2_in,
		vlf_e3_in => ff_emu_vlf_e3_in,
		vlf_ta1_in => ff_emu_vlf_ta1_in,
		vlf_ta2_in => ff_emu_vlf_ta2_in,
		vlf_ta3_in => ff_emu_vlf_ta3_in,
		vlf_tb1_in => ff_emu_vlf_tb1_in,
		vlf_tb2_in => ff_emu_vlf_tb2_in,
		flf_p1_out => ff_emu_flf_p1_out,
		flf_p2_out => ff_emu_flf_p2_out,
		flf_e1_out => ff_emu_flf_e1_out,
		flf_e2_out => ff_emu_flf_e2_out,
		vlf_e1_out => ff_emu_vlf_e1_out,
		vlf_e2_out => ff_emu_vlf_e2_out,
		vlf_e3_out => ff_emu_vlf_e3_out,
		vlf_ta1_out => ff_emu_vlf_ta1_out,
		vlf_ta2_out => ff_emu_vlf_ta2_out,
		vlf_ta3_out => ff_emu_vlf_ta3_out,
		vlf_tb1_out => ff_emu_vlf_tb1_out,
		vlf_tb2_out => ff_emu_vlf_tb2_out,
		clk_ref => clk_ref,
		rx_clk => rx_clk,
		rx_dat => rx_dat,
		tx_dat => tx_dat,
		msel0 => msel0,
		msel1 => msel1,
		msel2 => msel2,
		reset => reset,
		token => token,
		clk_cmos => clk_cmos
    );

-- FF_EMU RX outputs

tck <= ff_emu_vlf_ta1_out;
tms <= ff_emu_vlf_ta2_out;
tdi <= ff_emu_vlf_ta3_out;
trst <= reset;

PMAP_BSCAN : BGB_BSCAN_emulator 
  port map (

		CAPTURE1 => capture1,
		DRCK1 => drck1,
		RESET1 => reset1,
		SEL1 => sel1,
		SHIFT1 => shift1,
		UPDATE1 => update1,
		RUNTEST1 => open,
		TDO1 => tdo1,
		
		CAPTURE2 => capture2,
		DRCK2 => drck2,
		RESET2 => reset2,
		SEL2 => sel2,
		SHIFT2 => shift2,
		UPDATE2 => update2,
		RUNTEST2 => open,
		TDO2 => tdo2,

		TDO3 => tdo3,
		TDO4 => tdo4,

		TCK => tck,
		TDI => tdi,
		TMS => tms,
		TDO => tdo,
		TRST => trst
    );

PMAP_INSTR_DECODER : instr_dcd
  port map (
    TCK => tck,      -- in
    DRCK => drck1,       -- in 
    SEL => sel1,         -- in 
    TDI => tdi,         -- in 
    UPDATE => update1,   -- in 
    SHIFT => shift1,     -- in
    RST => reset1,       -- in  
    CLR => '0',         -- in
    F => fsel,          -- out
    TDO => tdo1     -- out
  );

PMAP_TDO_MUX : tdo_mux
  port map(
  
    TCK => tck,         -- in
    TDO_0C => tdo_f0c,         -- in
    TDO_17 => tdo_f17,         -- in
    FSEL => fsel,         
    TDO => tdo2
  );


PMAP_ADC_MASK : user_wr_reg -- #(.width(12), .def_value(12'hFFF))
--  generic map (
--     width => 12,
--     def_value => "111111111111"
--  );
  port map (
    TCK => tck,         -- in
    DRCK => drck2,       -- in
    FSEL => fsel(12),   -- in
    SEL => sel2,         -- in
    TDI => tdi,         -- in
    DSY_IN => '0',      -- in (not used)
    SHIFT => shift2,     -- in
    UPDATE => update2,   -- in
    RST => reset2,       -- in
    DSY_CHAIN => '0',   -- in (not used)
    PO => adc_mask,     -- out
    TDO => tdo_f0c,      -- out
    DSY_OUT => open     -- out (not used)
  );    

-- bgb
-- bgb put the value of adc_mask into bpi_status register for read back
-- bgb

   bpi_status <= x"B" & adc_mask;
 
 
PMAP_BPI_STATUS : user_cap_reg -- #(.width(16))
--  generic map (
--     width => 16
--  );
  port map (
    DRCK => drck2,       -- in
    FSH => '0',         -- in (not used)
    FCAP => fsel(23),   -- in
    SEL => sel2,         -- in
    TDI => tdi,         -- in
    SHIFT => shift2,     -- in
    CAPTURE => capture2, -- in
    RST => reset2,       -- in
	 PI => bpi_status,  -- in
    TDO => tdo_f17      -- in
    );      
		
 
end DCFEB_V6_FF_EMU_arch;
