`timescale 1ns / 1ps
//`uselib dir=C:/Xilinx/12.3/ISE_DS/ISE/verilog/src/unisims 

//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    12:02:31 02/24/2011 
// Design Name: 
// Module Name:    jtag_access 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
//    JTAG functionality, instantiate shift registers for capture and settings,
//    also contains serial interfaces to off chip devices
//
// Function  Description
// ---------------------------------------
//   0     | No Op 
//   1     | SCAM Reset (not needed in DCFEB)
//   2     | DCFEB status reg shift only
//   3     | DCFEB status reg capture and shift
//   4     | Program Comparator DAC
//   5     | Set Extra L1a Delay
//   6     | Read FIFO 1 -- ADC data (16 channels x 12 bits = 192 bits) wide X (6 chips x 8 sample)/event deep
//   7     | Set F5, F8, and F9 in one serial loop (daisy chained)
//   8     | Set Pre Block End (not needed in DCFEB)
//   9     | Set Comparator Mode and Timing bits
//  10     | Set Buckeye Mask for shifting (default 6'b111111)
//  11     | Shift data to/from Buckeye chips
//  12     | Set ADC configuration MASK
//  13     | Command to initialize ADC
//  14     | Shift data and write to ADC configuration memory
//  15     | Command to restart pipeline
//  16     | Set pipeline depth
//  17     | Perform a bit slip operation for odd deserializers
//  18     | Perform a bit slip operation for even deserializers
//  19     | Set Chip selection register.
//  20     | Set number of samples to readout.
//  21     | Write word to BPI interface write FIFO.
//  22     | Read word from BPI readback FIFO.
//  23     | Read status word from BPI interface.
//  24     | Read BPI timer (32 bits).
//  25     | Reset BPI interface.
//  26     | Disable BPI processing.
//  27     | Enable BPI processing.
//  28     | Read SEU 021 error count.
//  29     | Read SEU 120 error count.
//  30     | Clear SEU error counters.
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module jtag_bscan1 (
  CAPTURE,
  DRCK,
  RESET,
  RUNTEST,
  SEL,
  SHIFT,
  TCK,
  TDI,
  TMS,
  UPDATE,
  TDO
);

  output CAPTURE;
  output DRCK;
  output RESET;
  output RUNTEST;
  output SEL;
  output SHIFT;
  output TCK;
  output TDI;
  output TMS;
  output UPDATE;

  input TDO;

	
   BSCAN_VIRTEX6 #(.DISABLE_JTAG(0),.JTAG_CHAIN(1))  // User 1 for instruction decodes
   BSCAN_user1 (
      .CAPTURE(CAPTURE), // 1-bit output CAPTURE output from TAP controller
      .DRCK(DRCK),       // 1-bit output Data register output for USER functions
      .RESET(RESET),    // 1-bit output Reset output for TAP controller
      .RUNTEST(RUNTEST), // 1-bit output State output asserted when TAP controller is in Run Test Idle state.
      .SEL(SEL),        // 1-bit output USER active output
      .SHIFT(SHIFT),    // 1-bit output SHIFT output from TAP controller
      .TCK(TCK),         // 1-bit output Scan Clock output. Fabric connection to TAP Clock pin.
      .TDI(TDI),         // 1-bit output TDI output from TAP controller
      .TMS(TMS),         // 1-bit output Test Mode Select input. Fabric connection to TAP.
      .UPDATE(UPDATE),   // 1-bit output UPDATE output from TAP controller
      .TDO(TDO)          // 1-bit input Data input for USER function
   );
  
endmodule
