-- Hamming (16,11) decoder (SECDED)
--
--

LIBRARY IEEE;
USE IEEE.std_logic_1164.all;
use IEEE.std_logic_misc.all;

entity hamming_16_11_decoder is
  port(
    word_in  : in std_logic_vector(15 downto 0);
    word_dec : out std_logic_vector(10 downto 0);
    s_err : out std_logic; -- signals a single-bit error
    d_err : out std_logic  -- signals a double-bit error
  );
end hamming_16_11_decoder;


architecture h_16_11_dec_arch1 of hamming_16_11_decoder is 

	signal x : std_logic_vector(1 to 11); -- message vector
	signal p : std_logic_vector(1 to 5);  -- parity bits vector
	signal s : std_logic_vector(1 to 5);  -- vector of 4 syndrome bits plus the overall parity bit s(5)
	signal e : std_logic_vector(1 to 11); -- error vector

	signal d_err_int : std_logic;
    
begin

	x(1 to 11) <= word_in(15 downto 5); -- message bits
	p(1 to 5)  <= word_in(4 downto 0); -- parity bits
	
	-- syndrome calculation
	s(1) <= x(1) XOR x(2) XOR x(4) XOR x(5) XOR x(7) XOR x(9) XOR x(11) XOR p(1);	
	s(2) <= x(1) XOR x(3) XOR x(4) XOR x(6) XOR x(7) XOR x(10) XOR x(11) XOR p(2);
	s(3) <= x(2) XOR x(3) XOR x(4) XOR x(8) XOR x(9) XOR x(10) XOR x(11) XOR p(3);
	s(4) <= x(5) XOR x(6) XOR x(7) XOR x(8) XOR x(9) XOR x(10) XOR x(11) XOR p(4);
	                                                             
	-- overall parity bit calculation
	s(5) <= x(1) XOR x(2) XOR x(3) XOR x(4) XOR x(5) XOR x(6) XOR x(7) XOR x(8) XOR x(9) XOR x(10) XOR x(11)
      XOR p(1) XOR p(2) XOR p(3) XOR p(4) XOR p(5);
	
	-- error vector calculation			 
	e(1)  <= '1' when s(1 to 4) = "1100" else '0';
	e(2)  <= '1' when s(1 to 4) = "1010" else '0';
	e(3)  <= '1' when s(1 to 4) = "0110" else '0';
	e(4)  <= '1' when s(1 to 4) = "1110" else '0';
	e(5)  <= '1' when s(1 to 4) = "1001" else '0';
	e(6)  <= '1' when s(1 to 4) = "0101" else '0';
	e(7)  <= '1' when s(1 to 4) = "1101" else '0'; 
	e(8)  <= '1' when s(1 to 4) = "0011" else '0';
	e(9)  <= '1' when s(1 to 4) = "1011" else '0';
	e(10) <= '1' when s(1 to 4) = "0111" else '0';
	e(11) <= '1' when s(1 to 4) = "1111" else '0';
		
    s_err <= OR_REDUCE(e) and not(d_err_int);
    
	-- correction logic
	word_dec(10 downto 0) <=  x(1 to 11) XOR e(1 to 11);
	d_err_int <= '1' when (s(1 to 4) /= "0000") AND (s(5) = '0') else '0';
	d_err <= d_err_int;

end h_16_11_dec_arch1;
