-- Hamming (9,5) decoder (SEC)
--
--

LIBRARY IEEE;
USE IEEE.std_logic_1164.all;
use IEEE.std_logic_misc.all;

entity hamming_9_5_decoder is
  port(
    word_in  : in std_logic_vector(8 downto 0);
    word_dec : out std_logic_vector(4 downto 0);
    s_err : out std_logic -- signals a single-bit error
  );
end hamming_9_5_decoder;


architecture arch1 of hamming_9_5_decoder is 

	signal x : std_logic_vector(1 to 5); -- message vector
	signal p : std_logic_vector(1 to 4);  -- parity bits vector
	signal s : std_logic_vector(1 to 4);  -- vector of syndrome bits
	signal e : std_logic_vector(1 to 5); -- error vector
	
begin

	x(1 to 5) <= word_in(8 downto 4); -- message bits
	p(1 to 4) <= word_in(3 downto 0); -- parity bits
	
	-- syndrome calculation
	s(1) <= x(1) XOR x(2) XOR x(4) XOR x(5) XOR p(1);	
	s(2) <= x(1) XOR x(3) XOR x(4) XOR p(2);
	s(3) <= x(2) XOR x(3) XOR x(4) XOR p(3);
	s(4) <= x(5) XOR p(4);
	
	-- error vector calculation			 
	e(1)  <= '1' when s(1 to 4) = "1100" else '0';
	e(2)  <= '1' when s(1 to 4) = "1010" else '0';
	e(3)  <= '1' when s(1 to 4) = "0110" else '0';
	e(4)  <= '1' when s(1 to 4) = "1110" else '0';
	e(5)  <= '1' when s(1 to 4) = "1001" else '0';
		
    s_err <= OR_REDUCE(s);
    
	-- correction logic
	word_dec(4 downto 0) <=  x(1 to 5) XOR e(1 to 5);

end arch1;
