-- NAME: reg_n_SECDECSET
--
-- DESCRIPTION: n-bit register with Single Error Correction & Double Error Correction & SET handling

LIBRARY ieee;
USE IEEE.STD_LOGIC_UNSIGNED.all;
USE IEEE.std_logic_arith.all;
USE ieee.std_logic_1164.all;


ENTITY reg_1_SECDECSET_ml IS
PORT (
  data_1_in : IN STD_LOGIC;
  data_2_in : IN STD_LOGIC;
  data_3_in : IN STD_LOGIC;
  data_4_in : IN STD_LOGIC;
  data_5_in : IN STD_LOGIC;
  data_out : OUT STD_LOGIC
  
);

end reg_1_SECDECSET_ml;


ARCHITECTURE reg_1_SECDECSET_ml_architecture OF reg_1_SECDECSET_ml IS

signal data_in : STD_LOGIC_VECTOR(4 downto 0);

begin

majority_logic: process (data_1_in, data_2_in, data_3_in, data_4_in, data_5_in, data_in)

begin
  
	data_in(0) <= data_1_in;
	data_in(1) <= data_2_in;
	data_in(2) <= data_3_in;
	data_in(3) <= data_4_in;
	data_in(4) <= data_5_in;
	
	case (data_in) is

		-- data = 0 - NO SEU/DEU
		when "00000" =>
		data_out <= '0';
	
		-- data = 0 - SEU
		when "10000" | "01000" | "00100" | "00010" | "00001"=> 
		data_out <= '0';

		-- data = 0 - DEU
		when "11000" | "10100" | "10010" | "10001" | "01100" | "01010" | "01001" | "00110" | "00101" | "00011" => 
		data_out <= '0';

		-- data = 1 - NO SEU/DEU     
		when "11111" => 
		data_out <= '1';
                         
		-- data = 1 - SEU
		when "01111" | "10111" | "11011" | "11101" | "11110" => 
		data_out <= '1';
   
		-- data = 1 - DEU
		when "00111" | "01011" | "01101" | "01110" | "10011" | "10101" | "10110" | "11001" | "11010" | "11100" => 
		data_out <= '1';
    
		-- Others
		when others => 
		data_out <= '0';
        
	end case;                                     
  
end process;
  
END reg_1_SECDECSET_ml_architecture;
