-- NAME: reg_n_SECDECSET
--
-- DESCRIPTION: n-bit register with Single Error Correction & Double Error Detection & SET handling

LIBRARY ieee;
USE IEEE.STD_LOGIC_UNSIGNED.all;
USE IEEE.std_logic_arith.all;
USE ieee.std_logic_1164.all;


ENTITY reg_1r_SECDECSET_regs IS

PORT (
  clk : IN STD_LOGIC;
  reset : IN STD_LOGIC_VECTOR(2 downto 0);
  enable : IN STD_LOGIC_VECTOR(2 downto 0);
  data_1_in : IN STD_LOGIC;
  data_2_in : IN STD_LOGIC;
  data_3_in : IN STD_LOGIC;
  data_1_out : OUT STD_LOGIC;
  data_2_out : OUT STD_LOGIC;
  data_3_out : OUT STD_LOGIC;
  data_4_out : OUT STD_LOGIC;
  data_5_out : OUT STD_LOGIC
  
);

end reg_1r_SECDECSET_regs;


ARCHITECTURE reg_1r_SECDECSET_regs_architecture OF reg_1r_SECDECSET_regs IS


begin

regs: process (clk, reset, enable, data_1_in, data_2_in, data_3_in)

begin
  
		if rising_edge(clk) then
			if (reset(0) = '1') then
				data_1_out <= '0';
				data_2_out <= '0';
			elsif (enable(0) = '1') then
				data_1_out <= data_1_in;
				data_2_out <= data_1_in;
			end if;
		end if;	
  
		if rising_edge(clk) then
			if (reset(1) = '1') then
				data_3_out <= '0';
				data_4_out <= '0';
			elsif (enable(1) = '1') then
				data_3_out <= data_2_in;
				data_4_out <= data_2_in;
			end if;
		end if;	
  
 		if rising_edge(clk) then
			if (reset(2) = '1') then
				data_5_out <= '0';
			elsif (enable(2) = '1') then
				data_5_out <= data_3_in;
			end if;
		end if;	
 	
end process;
	      
END reg_1r_SECDECSET_regs_architecture;
