-- NAME: reg_n_SECDECSET
--
-- DESCRIPTION: n-bit register with Single Error Correction & Double Error Correction & SET handling

LIBRARY ieee;
USE IEEE.STD_LOGIC_UNSIGNED.all;
USE IEEE.std_logic_arith.all;
USE ieee.std_logic_1164.all;


ENTITY reg_n_SECDECSET_ml IS
GENERIC (
  n: integer range 1 to 64 := 1 -- number of bits
);
PORT (
  data_1_in : IN STD_LOGIC_VECTOR(n-1 downto 0);
  data_2_in : IN STD_LOGIC_VECTOR(n-1 downto 0);
  data_3_in : IN STD_LOGIC_VECTOR(n-1 downto 0);
  data_4_in : IN STD_LOGIC_VECTOR(n-1 downto 0);
  data_5_in : IN STD_LOGIC_VECTOR(n-1 downto 0);
  data_out : OUT STD_LOGIC_VECTOR(n-1 downto 0)
  
);

end reg_n_SECDECSET_ml;


ARCHITECTURE reg_n_SECDECSET_ml_architecture OF reg_n_SECDECSET_ml IS

type array_nx5 is array (n-1 downto 0) of std_logic_vector(4 downto 0);
signal data_in : array_nx5;

begin

majority_logic: process (data_1_in, data_2_in, data_3_in, data_4_in, data_5_in, data_in)

variable i : integer;

begin
  
	for i in 0 to n-1 loop

		data_in(i)(0) <= data_1_in(i);
		data_in(i)(1) <= data_2_in(i);
		data_in(i)(2) <= data_3_in(i);
		data_in(i)(3) <= data_4_in(i);
		data_in(i)(4) <= data_5_in(i);
		
		case (data_in(i)) is
  
		-- data = 0 - NO SEU/DEU
		when "00000" =>
		data_out(i) <= '0';
		
		-- data = 0 - SEU
		when "10000" | "01000" | "00100" | "00010" | "00001"=> 
		data_out(i) <= '0';
  
		-- data = 0 - DEU
		when "11000" | "10100" | "10010" | "10001" | "01100" | "01010" | "01001" | "00110" | "00101" | "00011" => 
		data_out(i) <= '0';

		-- data = 1 - NO SEU/DEU     
		when "11111" => 
		data_out(i) <= '1';
                             
		-- data = 1 - SEU
		when "01111" | "10111" | "11011" | "11101" | "11110" => 
		data_out(i) <= '1';
       
		-- data = 1 - DEU
		when "00111" | "01011" | "01101" | "01110" | "10011" | "10101" | "10110" | "11001" | "11010" | "11100" => 
		data_out(i) <= '1';
        
		-- Others
		when others => 
		data_out(i) <= '0';
        
		end case;                                     
  
	end loop;
		
end process;
  
END reg_n_SECDECSET_ml_architecture;
