-- NAME: reg_n_SECDECSET
--
-- DESCRIPTION: n-bit register with Single Error Correction & Double Error Detection & SET handling

LIBRARY ieee;
USE IEEE.STD_LOGIC_UNSIGNED.all;
USE IEEE.std_logic_arith.all;
USE ieee.std_logic_1164.all;


ENTITY reg_n_SECDECSET_regs_init IS
GENERIC (
  n: integer range 1 to 64 := 1 -- number of bits
);
PORT (
  clk : IN STD_LOGIC;
  reset : IN STD_LOGIC_VECTOR(2 downto 0);
  enable : IN STD_LOGIC_VECTOR(2 downto 0);
  init : IN STD_LOGIC_VECTOR(2 downto 0);
  data_1_in : IN STD_LOGIC_VECTOR(n-1 downto 0);
  data_2_in : IN STD_LOGIC_VECTOR(n-1 downto 0);
  data_3_in : IN STD_LOGIC_VECTOR(n-1 downto 0);
  data_1_init : IN STD_LOGIC_VECTOR(n-1 downto 0);
  data_2_init : IN STD_LOGIC_VECTOR(n-1 downto 0);
  data_3_init : IN STD_LOGIC_VECTOR(n-1 downto 0);
  data_1_reset : IN STD_LOGIC_VECTOR(n-1 downto 0);
  data_2_reset : IN STD_LOGIC_VECTOR(n-1 downto 0);
  data_3_reset : IN STD_LOGIC_VECTOR(n-1 downto 0);
  data_1_out : OUT STD_LOGIC_VECTOR(n-1 downto 0);
  data_2_out : OUT STD_LOGIC_VECTOR(n-1 downto 0);
  data_3_out : OUT STD_LOGIC_VECTOR(n-1 downto 0);
  data_4_out : OUT STD_LOGIC_VECTOR(n-1 downto 0);
  data_5_out : OUT STD_LOGIC_VECTOR(n-1 downto 0)
  
);

end reg_n_SECDECSET_regs_init;


ARCHITECTURE reg_n_SECDECSET_regs_init_architecture OF reg_n_SECDECSET_regs_init IS


begin

regs: process (clk, reset, enable, data_1_in, data_2_in, data_3_in, data_1_init, data_2_init, data_3_init)

variable i : integer;

begin
  
	for i in 0 to n-1 loop

		if rising_edge(clk) then
			if (reset(0) = '1') then
				data_1_out(i) <= data_1_reset(i);
				data_2_out(i) <= data_1_reset(i);
			elsif (init(0) = '1') then
				data_1_out(i) <= data_1_init(i);
				data_2_out(i) <= data_1_init(i);
			elsif (enable(0) = '1') then
				data_1_out(i) <= data_1_in(i);
				data_2_out(i) <= data_1_in(i);
			end if;
		end if;	
  
		if rising_edge(clk) then
			if (reset(1) = '1') then
				data_3_out(i) <= data_2_reset(i);
				data_4_out(i) <= data_2_reset(i);
			elsif (init(1) = '1') then
				data_3_out(i) <= data_2_init(i);
				data_4_out(i) <= data_2_init(i);
			elsif (enable(1) = '1') then
				data_3_out(i) <= data_2_in(i);
				data_4_out(i) <= data_2_in(i);
			end if;
		end if;	
  
 		if rising_edge(clk) then
			if (reset(2) = '1') then
				data_5_out(i) <= data_3_reset(i);
			elsif (init(1) = '1') then
				data_5_out(i) <= data_3_init(i);
			elsif (enable(2) = '1') then
				data_5_out(i) <= data_3_in(i);
			end if;
		end if;	
  
   
	end loop;
	
end process;
	      
END reg_n_SECDECSET_regs_init_architecture;
