-------------------------------------------------------------------------------
--
-- Title       : DEU_MAN_EN
-- Design      : Double Event Effect signal manager
-- Author      : Nico
-- Company     : NC

library IEEE;
use IEEE.STD_LOGIC_1164.all; 
USE IEEE.std_logic_arith.all;	
use IEEE.std_logic_unsigned.all;
USE work.fifo_pkg.all;

-- Entity declarations --

entity DEU_MAN_EN is		 
	generic(
		N_WORD : positive;
		N_ADDR : positive
		);
	port(	
		 rst_l : in STD_LOGIC;
		 clk : in STD_LOGIC;
		 write_scrub : in STD_LOGIC;   
		 scrub_addr : in STD_LOGIC_VECTOR(N_ADDR-1 downto 0);
		 deu_in: in STD_LOGIC;
		 deu_out : out STD_LOGIC;
		 enable : in std_logic 

	     );
end DEU_MAN_EN;

-- Architecture -- 

architecture DEU_MAN1_EN of DEU_MAN_EN is 
	signal write_sel: std_logic_vector(N_WORD-1 downto 0); 
	signal data_out_sig: std_logic_vector(N_WORD-1 downto 0);

	component fifo_reg_N_RHSEU_EN 
		generic (N : integer);
		port (  data_in : in std_logic;
				clk : in std_logic;
				rst_L : in std_logic; 
				write : in std_logic_vector(n-1 downto 0); 
				data_out : out std_logic_vector(n-1 downto 0);
				enable : in std_logic

				);  
	end component;

 begin
	 
	 reg_RH: fifo_reg_N_RHSEU_EN
	 				generic map( N=> N_WORD)
	 				port map( 	data_in  =>	 deu_in,
								clk 	 =>	 clk,
								rst_L  	 =>	 rst_l,
								write  	 =>	 write_sel,
								data_out =>  data_out_sig,
								enable => enable
							);
	 process (write_scrub, scrub_addr) 	 		
	 begin
		for i in 0 to N_WORD-1 loop
-- QUARTUS WARNING
--			if write_scrub = '0' then
--				write_sel(i) <= '0';
--			else 
--				if i = conv_integer(scrub_addr) then 
--				    write_sel(conv_integer(scrub_addr)) <= '1';
--				else
--				write_sel(i) <= '0'; 
--				end if;
--			end if;
			if write_scrub = '0' then
				write_sel(i) <= '0';
			elsif (i = conv_integer(scrub_addr)) then 
				write_sel(i) <= '1';
			else
				write_sel(i) <= '0'; 
			end if;
		end loop;
	end process;

	deu_out <= '0' when deu_in = '0' else 
			   '0' when deu_in = '1'and data_out_sig(conv_integer(scrub_addr))='1' else
			   '1' when deu_in = '1' and data_out_sig(conv_integer(scrub_addr))='0' else
			   '0';

	
end DEU_MAN1_EN;
