
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use work.interfaces_pkg.all;

entity FA_Control1_RHSEU is 
	port (
       -- transmission ("fast") clock
       clk : in std_logic;
       clk_enable : in std_logic;
       --------------------------
		FD_err: in STD_LOGIC;
		FDC_rcv: in STD_LOGIC;
		last_W: in STD_LOGIC;
		rst_L: in STD_LOGIC;
		sel_shr: in STD_LOGIC;
		en_WCNT: out STD_LOGIC;
		rst_WCNT_L: out STD_LOGIC;
		write_FD_FIFO: out STD_LOGIC;
		write_FD_reg: out STD_LOGIC;
		write_word: out STD_LOGIC;
		seu : out STD_LOGIC
		);
end FA_Control1_RHSEU;

architecture FA_Control1_1 of FA_Control1_RHSEU is

-- BINARY ENCODED state machine: FA_CTRL1_Sreg0
subtype FA_CTRL1_Sreg0_type is STD_LOGIC_VECTOR (0 to 1);
constant IDLE: FA_CTRL1_Sreg0_type := "00";
constant WAIT_WORD1: FA_CTRL1_Sreg0_type := "01";
constant WAIT_FDC: FA_CTRL1_Sreg0_type := "10";
constant WAIT_WORD0: FA_CTRL1_Sreg0_type := "11";
-- attribute enum_encoding of FA_CTRL1_Sreg0_type: type is ... -- enum_encoding attribute is not supported for state register defined by using subtype keyword

signal FA_CTRL1_Sreg0: FA_CTRL1_Sreg0_type;
signal next_state, current_state_a, current_state_b, current_state_c, current_state_corr: FA_CTRL1_Sreg0_type;
signal status :  STD_LOGIC_VECTOR(2 downto 0);
signal a_eq_b: std_logic;
signal b_eq_c : std_logic; 
signal a_eq_c : std_logic;	
signal seu_states : std_logic;	

begin

	fsm_state: process (clk)
begin
	if rising_edge(clk) then 
		if rst_L = '0' then	
			current_state_a <= IDLE;
			current_state_b <= IDLE;
			current_state_c <= IDLE;
		
		elsif (clk_enable = '1') then	 
			current_state_a <= next_state;
			current_state_b <= next_state;
			current_state_c <= next_state;
			
		end if;
	end if;
end process; 

fsm_comb_logic: process (current_state_corr, FDC_rcv, sel_shr, FD_err, last_W )
begin
case current_state_corr is
				when IDLE =>
					if FDC_rcv = '1' then	
						next_state <= WAIT_FDC;
					else	
						next_state <= IDLE;
					end if;
				when WAIT_WORD1 =>
					if (sel_shr = '1') AND (FD_err = '0') then	
						next_state <= WAIT_WORD1;
					elsif (sel_shr = '0') AND (last_W = '1') AND (FDC_rcv = '0') AND (FD_err = '0') then	
						next_state <= IDLE;
					elsif (sel_shr = '0') AND (FDC_rcv = '1') AND (FD_err = '0') then	
						next_state <= WAIT_FDC;
					elsif FD_err = '1' then	
						next_state <= IDLE;
					else	
						next_state <= WAIT_WORD0;
					end if;
				when WAIT_FDC =>
					if (FDC_rcv = '0') AND (sel_shr = '0') then	
						next_state <= WAIT_WORD0;
					elsif FDC_rcv = '1' then	
						next_state <= WAIT_FDC;
					else	
						next_state <= WAIT_WORD1;
					end if;
				when WAIT_WORD0 =>
					if (sel_shr = '0') AND (FD_err = '0') then	
						next_state <= WAIT_WORD0;
					elsif (sel_shr = '1') AND (last_W = '1') AND (FDC_rcv = '0')  AND (FD_err = '0') then	
						next_state <= IDLE;
					elsif (sel_shr = '1') AND (FDC_rcv = '1') AND (FD_err = '0') then	
						next_state <= WAIT_FDC;
					elsif FD_err = '1' then	
						next_state <= IDLE;
					else	
						next_state <= WAIT_WORD1;
					end if;
--vhdl_cover_off
				when others =>
					next_state <= IDLE;
--vhdl_cover_on
			end case;
end process;


  process (current_state_a, current_state_b, current_state_c)
	begin
        
		if (current_state_a = current_state_b) then
			a_eq_b <= '1';
		else
			a_eq_b <= '0';
		end if;
			
		if (current_state_a = current_state_c) then
			a_eq_c <= '1';
		else
			a_eq_c <= '0';
		end if;
				
		if (current_state_b = current_state_c) then
			b_eq_c <= '1';
		else
			b_eq_c <= '0';
		end if;
  end process;
  
  status <= a_eq_b & a_eq_c & b_eq_c;
  
  process (status, current_state_a, current_state_b)
    begin
		case (status) is 
			when "111" =>     			--All equal
				current_state_corr <= current_state_a; 
				seu_states <= '0'; 
			when "100" =>    			--c different
				current_state_corr <= current_state_a; 
				seu_states <= '1'; 
			when "010" => 				--b different
				current_state_corr <= current_state_a;
				seu_states <= '1'; 
			when "001" => 				--a different
				current_state_corr <= current_state_b;
				seu_states <= '1'; 
			when others => 			  -- errors in majority logic	
				current_state_corr <= IDLE;
				seu_states <= '1'; 
		end case;
	end process;
    
-- signal assignment statements for combinatorial outputs
write_FD_reg_assignment:
write_FD_reg <= '0' when (current_state_corr = IDLE and FDC_rcv = '1') else
                '0' when (current_state_corr = IDLE and (FDC_rcv = '0' and not (FDC_rcv = '1'))) else
                '0' when (current_state_corr = IDLE) else
                '0' when (current_state_corr = WAIT_WORD1 and ((sel_shr = '1') AND (FD_err = '0'))) else
                '0' when (current_state_corr = WAIT_WORD1 and ((sel_shr = '0') AND (last_W = '1') 
AND (FDC_rcv = '0') AND (FD_err = '0') and not ((sel_shr = '1') AND (FD_err = '0')))) else
                '0' when (current_state_corr = WAIT_WORD1 and ((sel_shr = '0') AND (FDC_rcv = '1')
 AND (FD_err = '0') and not ((sel_shr = '1') AND (FD_err = '0')) and not ((sel_shr = '0') AND (last_W = '1') 
AND (FDC_rcv = '0') AND (FD_err = '0')))) else
                '0' when (current_state_corr = WAIT_WORD1 and (FD_err = '1' and not ((sel_shr = '1') AND (FD_err = '0')) and not ((sel_shr = '0') AND (last_W = '1') 
AND (FDC_rcv = '0') AND (FD_err = '0')) and not ((sel_shr = '0') AND (FDC_rcv = '1')
 AND (FD_err = '0')))) else
                '0' when (current_state_corr = WAIT_WORD1 and ((sel_shr = '0') AND (last_W = '0')
 AND (FDC_rcv = '0') AND (FD_err = '0') and not ((sel_shr = '1') AND (FD_err = '0')) and not ((sel_shr = '0') AND (last_W = '1') 
AND (FDC_rcv = '0') AND (FD_err = '0')) and not ((sel_shr = '0') AND (FDC_rcv = '1')
 AND (FD_err = '0')) and not (FD_err = '1'))) else
                '1' when (current_state_corr = WAIT_FDC and ((FDC_rcv = '0') AND (sel_shr = '0'))) else
                '0' when (current_state_corr = WAIT_FDC and (FDC_rcv = '1' and not ((FDC_rcv = '0') AND (sel_shr = '0')))) else
                '1' when (current_state_corr = WAIT_FDC and ((FDC_rcv = '0') AND (sel_shr = '1') and not ((FDC_rcv = '0') AND (sel_shr = '0')) and not (FDC_rcv = '1'))) else
                '0' when (current_state_corr = WAIT_WORD0 and ((sel_shr = '0') AND (FD_err = '0'))) else
                '0' when (current_state_corr = WAIT_WORD0 and ((sel_shr = '1') AND (last_W = '1')
AND (FDC_rcv = '0')  AND (FD_err = '0') and not ((sel_shr = '0') AND (FD_err = '0')))) else
                '0' when (current_state_corr = WAIT_WORD0 and ((sel_shr = '1') AND (FDC_rcv = '1')
 AND (FD_err = '0') and not ((sel_shr = '0') AND (FD_err = '0')) and not ((sel_shr = '1') AND (last_W = '1')
AND (FDC_rcv = '0')  AND (FD_err = '0')))) else
                '0' when (current_state_corr = WAIT_WORD0 and (FD_err = '1' and not ((sel_shr = '0') AND (FD_err = '0')) and not ((sel_shr = '1') AND (last_W = '1')
AND (FDC_rcv = '0')  AND (FD_err = '0')) and not ((sel_shr = '1') AND (FDC_rcv = '1')
 AND (FD_err = '0')))) else
                '0' when (current_state_corr = WAIT_WORD0 and ((sel_shr = '1') AND (last_W = '0') 
AND (FDC_rcv = '0') AND (FD_err = '0') and not ((sel_shr = '0') AND (FD_err = '0')) and not ((sel_shr = '1') AND (last_W = '1')
AND (FDC_rcv = '0')  AND (FD_err = '0')) and not ((sel_shr = '1') AND (FDC_rcv = '1')
 AND (FD_err = '0')) and not (FD_err = '1'))) else
                '0';

write_FD_FIFO_assignment:
write_FD_FIFO <= '0' when (current_state_corr = IDLE and FDC_rcv = '1') else
                 '0' when (current_state_corr = IDLE and (FDC_rcv = '0' and not (FDC_rcv = '1'))) else
                 '0' when (current_state_corr = IDLE) else
                 '0' when (current_state_corr = WAIT_WORD1 and ((sel_shr = '1') AND (FD_err = '0'))) else
                 '1' when (current_state_corr = WAIT_WORD1 and ((sel_shr = '0') AND (last_W = '1') 
AND (FDC_rcv = '0') AND (FD_err = '0') and not ((sel_shr = '1') AND (FD_err = '0')))) else
                 '1' when (current_state_corr = WAIT_WORD1 and ((sel_shr = '0') AND (FDC_rcv = '1')
 AND (FD_err = '0') and not ((sel_shr = '1') AND (FD_err = '0')) and not ((sel_shr = '0') AND (last_W = '1') 
AND (FDC_rcv = '0') AND (FD_err = '0')))) else
                 '1' when (current_state_corr = WAIT_WORD1 and (FD_err = '1' and not ((sel_shr = '1') AND (FD_err = '0')) and not ((sel_shr = '0') AND (last_W = '1') 
AND (FDC_rcv = '0') AND (FD_err = '0')) and not ((sel_shr = '0') AND (FDC_rcv = '1')
 AND (FD_err = '0')))) else
                 '0' when (current_state_corr = WAIT_WORD1 and ((sel_shr = '0') AND (last_W = '0')
 AND (FDC_rcv = '0') AND (FD_err = '0') and not ((sel_shr = '1') AND (FD_err = '0')) and not ((sel_shr = '0') AND (last_W = '1') 
AND (FDC_rcv = '0') AND (FD_err = '0')) and not ((sel_shr = '0') AND (FDC_rcv = '1')
 AND (FD_err = '0')) and not (FD_err = '1'))) else
                 '0' when (current_state_corr = WAIT_FDC and ((FDC_rcv = '0') AND (sel_shr = '0'))) else
                 '0' when (current_state_corr = WAIT_FDC and (FDC_rcv = '1' and not ((FDC_rcv = '0') AND (sel_shr = '0')))) else
                 '0' when (current_state_corr = WAIT_FDC and ((FDC_rcv = '0') AND (sel_shr = '1') and not ((FDC_rcv = '0') AND (sel_shr = '0')) and not (FDC_rcv = '1'))) else
                 '0' when (current_state_corr = WAIT_WORD0 and ((sel_shr = '0') AND (FD_err = '0'))) else
                 '1' when (current_state_corr = WAIT_WORD0 and ((sel_shr = '1') AND (last_W = '1')
AND (FDC_rcv = '0')  AND (FD_err = '0') and not ((sel_shr = '0') AND (FD_err = '0')))) else
                 '1' when (current_state_corr = WAIT_WORD0 and ((sel_shr = '1') AND (FDC_rcv = '1')
 AND (FD_err = '0') and not ((sel_shr = '0') AND (FD_err = '0')) and not ((sel_shr = '1') AND (last_W = '1')
AND (FDC_rcv = '0')  AND (FD_err = '0')))) else
                 '1' when (current_state_corr = WAIT_WORD0 and (FD_err = '1' and not ((sel_shr = '0') AND (FD_err = '0')) and not ((sel_shr = '1') AND (last_W = '1')
AND (FDC_rcv = '0')  AND (FD_err = '0')) and not ((sel_shr = '1') AND (FDC_rcv = '1')
 AND (FD_err = '0')))) else
                 '0' when (current_state_corr = WAIT_WORD0 and ((sel_shr = '1') AND (last_W = '0') 
AND (FDC_rcv = '0') AND (FD_err = '0') and not ((sel_shr = '0') AND (FD_err = '0')) and not ((sel_shr = '1') AND (last_W = '1')
AND (FDC_rcv = '0')  AND (FD_err = '0')) and not ((sel_shr = '1') AND (FDC_rcv = '1')
 AND (FD_err = '0')) and not (FD_err = '1'))) else
                 '0';

write_word_assignment:
write_word <= '0' when (current_state_corr = IDLE and FDC_rcv = '1') else
              '0' when (current_state_corr = IDLE and (FDC_rcv = '0' and not (FDC_rcv = '1'))) else
              '0' when (current_state_corr = IDLE) else
              '0' when (current_state_corr = WAIT_WORD1 and ((sel_shr = '1') AND (FD_err = '0'))) else
              '1' when (current_state_corr = WAIT_WORD1 and ((sel_shr = '0') AND (last_W = '1') 
AND (FDC_rcv = '0') AND (FD_err = '0') and not ((sel_shr = '1') AND (FD_err = '0')))) else
              '1' when (current_state_corr = WAIT_WORD1 and ((sel_shr = '0') AND (FDC_rcv = '1')
 AND (FD_err = '0') and not ((sel_shr = '1') AND (FD_err = '0')) and not ((sel_shr = '0') AND (last_W = '1') 
AND (FDC_rcv = '0') AND (FD_err = '0')))) else
              '0' when (current_state_corr = WAIT_WORD1 and (FD_err = '1' and not ((sel_shr = '1') AND (FD_err = '0')) and not ((sel_shr = '0') AND (last_W = '1') 
AND (FDC_rcv = '0') AND (FD_err = '0')) and not ((sel_shr = '0') AND (FDC_rcv = '1')
 AND (FD_err = '0')))) else
              '1' when (current_state_corr = WAIT_WORD1 and ((sel_shr = '0') AND (last_W = '0')
 AND (FDC_rcv = '0') AND (FD_err = '0') and not ((sel_shr = '1') AND (FD_err = '0')) and not ((sel_shr = '0') AND (last_W = '1') 
AND (FDC_rcv = '0') AND (FD_err = '0')) and not ((sel_shr = '0') AND (FDC_rcv = '1')
 AND (FD_err = '0')) and not (FD_err = '1'))) else
              '0' when (current_state_corr = WAIT_FDC and ((FDC_rcv = '0') AND (sel_shr = '0'))) else
              '0' when (current_state_corr = WAIT_FDC and (FDC_rcv = '1' and not ((FDC_rcv = '0') AND (sel_shr = '0')))) else
              '0' when (current_state_corr = WAIT_FDC and ((FDC_rcv = '0') AND (sel_shr = '1') and not ((FDC_rcv = '0') AND (sel_shr = '0')) and not (FDC_rcv = '1'))) else
              '0' when (current_state_corr = WAIT_WORD0 and ((sel_shr = '0') AND (FD_err = '0'))) else
              '1' when (current_state_corr = WAIT_WORD0 and ((sel_shr = '1') AND (last_W = '1')
AND (FDC_rcv = '0')  AND (FD_err = '0') and not ((sel_shr = '0') AND (FD_err = '0')))) else
              '1' when (current_state_corr = WAIT_WORD0 and ((sel_shr = '1') AND (FDC_rcv = '1')
 AND (FD_err = '0') and not ((sel_shr = '0') AND (FD_err = '0')) and not ((sel_shr = '1') AND (last_W = '1')
AND (FDC_rcv = '0')  AND (FD_err = '0')))) else
              '0' when (current_state_corr = WAIT_WORD0 and (FD_err = '1' and not ((sel_shr = '0') AND (FD_err = '0')) and not ((sel_shr = '1') AND (last_W = '1')
AND (FDC_rcv = '0')  AND (FD_err = '0')) and not ((sel_shr = '1') AND (FDC_rcv = '1')
 AND (FD_err = '0')))) else
              '1' when (current_state_corr = WAIT_WORD0 and ((sel_shr = '1') AND (last_W = '0') 
AND (FDC_rcv = '0') AND (FD_err = '0') and not ((sel_shr = '0') AND (FD_err = '0')) and not ((sel_shr = '1') AND (last_W = '1')
AND (FDC_rcv = '0')  AND (FD_err = '0')) and not ((sel_shr = '1') AND (FDC_rcv = '1')
 AND (FD_err = '0')) and not (FD_err = '1'))) else
              '0';

en_WCNT_assignment:
en_WCNT <= '0' when (current_state_corr = IDLE and FDC_rcv = '1') else
           '0' when (current_state_corr = IDLE and (FDC_rcv = '0' and not (FDC_rcv = '1'))) else
           '0' when (current_state_corr = IDLE) else
           '0' when (current_state_corr = WAIT_WORD1 and ((sel_shr = '1') AND (FD_err = '0'))) else
           '0' when (current_state_corr = WAIT_WORD1 and ((sel_shr = '0') AND (last_W = '1') 
AND (FDC_rcv = '0') AND (FD_err = '0') and not ((sel_shr = '1') AND (FD_err = '0')))) else
           '0' when (current_state_corr = WAIT_WORD1 and ((sel_shr = '0') AND (FDC_rcv = '1')
 AND (FD_err = '0') and not ((sel_shr = '1') AND (FD_err = '0')) and not ((sel_shr = '0') AND (last_W = '1') 
AND (FDC_rcv = '0') AND (FD_err = '0')))) else
           '0' when (current_state_corr = WAIT_WORD1 and (FD_err = '1' and not ((sel_shr = '1') AND (FD_err = '0')) and not ((sel_shr = '0') AND (last_W = '1') 
AND (FDC_rcv = '0') AND (FD_err = '0')) and not ((sel_shr = '0') AND (FDC_rcv = '1')
 AND (FD_err = '0')))) else
           '1' when (current_state_corr = WAIT_WORD1 and ((sel_shr = '0') AND (last_W = '0')
 AND (FDC_rcv = '0') AND (FD_err = '0') and not ((sel_shr = '1') AND (FD_err = '0')) and not ((sel_shr = '0') AND (last_W = '1') 
AND (FDC_rcv = '0') AND (FD_err = '0')) and not ((sel_shr = '0') AND (FDC_rcv = '1')
 AND (FD_err = '0')) and not (FD_err = '1'))) else
           '0' when (current_state_corr = WAIT_FDC and ((FDC_rcv = '0') AND (sel_shr = '0'))) else
           '0' when (current_state_corr = WAIT_FDC and (FDC_rcv = '1' and not ((FDC_rcv = '0') AND (sel_shr = '0')))) else
           '0' when (current_state_corr = WAIT_FDC and ((FDC_rcv = '0') AND (sel_shr = '1') and not ((FDC_rcv = '0') AND (sel_shr = '0')) and not (FDC_rcv = '1'))) else
           '0' when (current_state_corr = WAIT_WORD0 and ((sel_shr = '0') AND (FD_err = '0'))) else
           '0' when (current_state_corr = WAIT_WORD0 and ((sel_shr = '1') AND (last_W = '1')
AND (FDC_rcv = '0')  AND (FD_err = '0') and not ((sel_shr = '0') AND (FD_err = '0')))) else
           '0' when (current_state_corr = WAIT_WORD0 and ((sel_shr = '1') AND (FDC_rcv = '1')
 AND (FD_err = '0') and not ((sel_shr = '0') AND (FD_err = '0')) and not ((sel_shr = '1') AND (last_W = '1')
AND (FDC_rcv = '0')  AND (FD_err = '0')))) else
           '0' when (current_state_corr = WAIT_WORD0 and (FD_err = '1' and not ((sel_shr = '0') AND (FD_err = '0')) and not ((sel_shr = '1') AND (last_W = '1')
AND (FDC_rcv = '0')  AND (FD_err = '0')) and not ((sel_shr = '1') AND (FDC_rcv = '1')
 AND (FD_err = '0')))) else
           '1' when (current_state_corr = WAIT_WORD0 and ((sel_shr = '1') AND (last_W = '0') 
AND (FDC_rcv = '0') AND (FD_err = '0') and not ((sel_shr = '0') AND (FD_err = '0')) and not ((sel_shr = '1') AND (last_W = '1')
AND (FDC_rcv = '0')  AND (FD_err = '0')) and not ((sel_shr = '1') AND (FDC_rcv = '1')
 AND (FD_err = '0')) and not (FD_err = '1'))) else
           '0';

rst_WCNT_L_assignment:
rst_WCNT_L <= '0' when (current_state_corr = IDLE and FDC_rcv = '1') else
              '0' when (current_state_corr = IDLE and (FDC_rcv = '0' and not (FDC_rcv = '1'))) else
              '0' when (current_state_corr = IDLE) else
              '1' when (current_state_corr = WAIT_WORD1 and ((sel_shr = '1') AND (FD_err = '0'))) else
              '0' when (current_state_corr = WAIT_WORD1 and ((sel_shr = '0') AND (last_W = '1') 
AND (FDC_rcv = '0') AND (FD_err = '0') and not ((sel_shr = '1') AND (FD_err = '0')))) else
              '0' when (current_state_corr = WAIT_WORD1 and ((sel_shr = '0') AND (FDC_rcv = '1')
 AND (FD_err = '0') and not ((sel_shr = '1') AND (FD_err = '0')) and not ((sel_shr = '0') AND (last_W = '1') 
AND (FDC_rcv = '0') AND (FD_err = '0')))) else
              '0' when (current_state_corr = WAIT_WORD1 and (FD_err = '1' and not ((sel_shr = '1') AND (FD_err = '0')) and not ((sel_shr = '0') AND (last_W = '1') 
AND (FDC_rcv = '0') AND (FD_err = '0')) and not ((sel_shr = '0') AND (FDC_rcv = '1')
 AND (FD_err = '0')))) else
              '1' when (current_state_corr = WAIT_WORD1 and ((sel_shr = '0') AND (last_W = '0')
 AND (FDC_rcv = '0') AND (FD_err = '0') and not ((sel_shr = '1') AND (FD_err = '0')) and not ((sel_shr = '0') AND (last_W = '1') 
AND (FDC_rcv = '0') AND (FD_err = '0')) and not ((sel_shr = '0') AND (FDC_rcv = '1')
 AND (FD_err = '0')) and not (FD_err = '1'))) else
              '0' when (current_state_corr = WAIT_FDC and ((FDC_rcv = '0') AND (sel_shr = '0'))) else
              '0' when (current_state_corr = WAIT_FDC and (FDC_rcv = '1' and not ((FDC_rcv = '0') AND (sel_shr = '0')))) else
              '0' when (current_state_corr = WAIT_FDC and ((FDC_rcv = '0') AND (sel_shr = '1') and not ((FDC_rcv = '0') AND (sel_shr = '0')) and not (FDC_rcv = '1'))) else
              '1' when (current_state_corr = WAIT_WORD0 and ((sel_shr = '0') AND (FD_err = '0'))) else
              '0' when (current_state_corr = WAIT_WORD0 and ((sel_shr = '1') AND (last_W = '1')
AND (FDC_rcv = '0')  AND (FD_err = '0') and not ((sel_shr = '0') AND (FD_err = '0')))) else
              '0' when (current_state_corr = WAIT_WORD0 and ((sel_shr = '1') AND (FDC_rcv = '1')
 AND (FD_err = '0') and not ((sel_shr = '0') AND (FD_err = '0')) and not ((sel_shr = '1') AND (last_W = '1')
AND (FDC_rcv = '0')  AND (FD_err = '0')))) else
              '0' when (current_state_corr = WAIT_WORD0 and (FD_err = '1' and not ((sel_shr = '0') AND (FD_err = '0')) and not ((sel_shr = '1') AND (last_W = '1')
AND (FDC_rcv = '0')  AND (FD_err = '0')) and not ((sel_shr = '1') AND (FDC_rcv = '1')
 AND (FD_err = '0')))) else
              '1' when (current_state_corr = WAIT_WORD0 and ((sel_shr = '1') AND (last_W = '0') 
AND (FDC_rcv = '0') AND (FD_err = '0') and not ((sel_shr = '0') AND (FD_err = '0')) and not ((sel_shr = '1') AND (last_W = '1')
AND (FDC_rcv = '0')  AND (FD_err = '0')) and not ((sel_shr = '1') AND (FDC_rcv = '1')
 AND (FD_err = '0')) and not (FD_err = '1'))) else
              '0';

			  seu <= seu_states;

end FA_Control1_1;
