
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use work.interfaces_pkg.all;

entity FA_Control2_RHSEU is 
	port (
       -- transmission ("fast") clock
       clk : in std_logic;
       clk_enable : in std_logic;
       --------------------------
		Frame_Lost: in STD_LOGIC;
		Frame_Ready: in STD_LOGIC;
		Get_Data: in STD_LOGIC;
		last_W: in STD_LOGIC;
		rst_L: in STD_LOGIC;
		Data_Valid: out STD_LOGIC;
		en_WCNT: out STD_LOGIC;
		read_FD_FIFO: out STD_LOGIC;
		rst_WCNT_L: out STD_LOGIC;
		seu: out STD_LOGIC
		);
end FA_Control2_RHSEU;

architecture FA_Control2_1 of FA_Control2_RHSEU is

-- BINARY ENCODED state machine: FA_CTRL2_Sreg0
subtype FA_CTRL2_Sreg0_type is STD_LOGIC;
constant DELIV_FRAME: FA_CTRL2_Sreg0_type := '0';
constant IDLE: FA_CTRL2_Sreg0_type := '1';
-- attribute enum_encoding of FA_CTRL2_Sreg0_type: type is ... -- enum_encoding attribute is not supported for state register defined by using subtype keyword

signal next_state, current_state_a, current_state_b, current_state_c, current_state_corr: FA_CTRL2_Sreg0_type;
signal status :  STD_LOGIC_VECTOR(2 downto 0);
signal Data_Valid_int : STD_LOGIC;
signal Data_Valid_sig:  STD_LOGIC_VECTOR(0 to 2);
signal corr_data_Data_Valid : STD_LOGIC;
signal a_eq_b: std_logic;
signal b_eq_c : std_logic; 
signal a_eq_c : std_logic;	
signal seu_states : std_logic;	
signal seu_Data_Valid : std_logic;

begin

fsm_state: process (clk)
begin
	if rising_edge(clk) then 
		if rst_L = '0' then	
			current_state_a <= IDLE;
			current_state_b <= IDLE;
			current_state_c <= IDLE;
		
			Data_Valid_sig <= (OTHERS => '0');
             
		
		elsif (clk_enable = '1') then	 
			current_state_a <= next_state;
			current_state_b <= next_state;
			current_state_c <= next_state;
			
		for i in 0 to 2 loop
			Data_Valid_sig(i) <= Data_Valid_int;
		end loop;
			
		end if;
	end if;
end process; 

fsm_comb_logic: process (current_state_corr, last_W, Get_Data, Frame_Lost, Frame_Ready )
begin
			case current_state_corr is
				when DELIV_FRAME =>
					if Get_Data = '0' then	
						next_state <= DELIV_FRAME;
						Data_Valid_int <= '1';
					elsif (last_W = '1') OR (Frame_Lost = '1') then	
						next_state <= IDLE;
						Data_Valid_int <= '0';
					else
						next_state <= DELIV_FRAME;
						Data_Valid_int <= '1';
					end if;
				when IDLE =>
					if Frame_Ready = '0' then	
						next_state <= IDLE;
						Data_Valid_int <= '0';
					else	
						next_state <= DELIV_FRAME;
						Data_Valid_int <= '1';
					end if;
				
				when others =>
					null;
--vhdl_cover_on
			end case;
end process;  

  process (current_state_a, current_state_b, current_state_c)
	begin
        
		if (current_state_a = current_state_b) then
			a_eq_b <= '1';
		else
			a_eq_b <= '0';
		end if;
			
		if (current_state_a = current_state_c) then
			a_eq_c <= '1';
		else
			a_eq_c <= '0';
		end if;
				
		if (current_state_b = current_state_c) then
			b_eq_c <= '1';
		else
			b_eq_c <= '0';
		end if;
  end process;
  
  status <= a_eq_b & a_eq_c & b_eq_c;
  
  process (status, current_state_a, current_state_b)
    begin
		case (status) is 
			when "111" =>     			--All equal
				current_state_corr <= current_state_a; 
				seu_states <= '0'; 
			when "100" =>    			--c different
				current_state_corr <= current_state_a; 
				seu_states <= '1'; 
			when "010" => 				--b different
				current_state_corr <= current_state_a;
				seu_states <= '1'; 
			when "001" => 				--a different
				current_state_corr <= current_state_b;
				seu_states <= '1'; 
			when others => 			  -- errors in majority logic	
				current_state_corr <= IDLE;
				seu_states <= '1'; 
		end case;
	end process;
	
	
majority_logic_Data_Valid: process (Data_Valid_sig)
	begin
		case (Data_Valid_sig) is

			-- data = 0 - NO SEU
			when "000" => 	corr_data_Data_Valid <= '0';  
							seu_Data_Valid <= '0';	
			
			-- data = 1 - NO SEU
			when "111" => 	corr_data_Data_Valid <= '1';  
							seu_Data_Valid <= '0';
			
			-- data = 0 - SEU
			when "100" => 	corr_data_Data_Valid <= '0';
							seu_Data_Valid <= '1';	
			when "010" =>	corr_data_Data_Valid <= '0';
							seu_Data_Valid <= '1';
			when "001" => 	corr_data_Data_Valid <= '0';
							seu_Data_Valid <= '1';
                  
			-- data = 1 - SEU            
			when "110" => 	corr_data_Data_Valid <= '1';
							seu_Data_Valid <= '1';
			when "101" => 	corr_data_Data_Valid <= '1';
							seu_Data_Valid <= '1';
			when "011" => 	corr_data_Data_Valid <= '1'; 
							seu_Data_Valid <= '1';	
			
			when others => 	corr_data_Data_Valid <= '0';
							seu_Data_Valid <= '0';
		end case;	                                     
	end process;
-- signal assignment statements for combinatorial outputs
en_WCNT_assignment:
en_WCNT <= '0' when (current_state_corr = IDLE and Frame_Ready = '0') else
           '0' when (current_state_corr = IDLE and (Frame_Ready = '1' and not (Frame_Ready = '0'))) else
           '0' when (current_state_corr = IDLE) else
           '0' when (current_state_corr = DELIV_FRAME and (((last_W = '1') AND (Get_Data = '1')) OR (Frame_Lost = '1'))) else
           '1' when (current_state_corr = DELIV_FRAME and ((last_W = '0') AND (Get_Data = '1') AND (Frame_Lost = '0') and not ((((last_W = '1') AND (Get_Data = '1')) OR (Frame_Lost = '1'))))) else
           '0' when (current_state_corr = DELIV_FRAME and (Get_Data = '0' and not ((((last_W = '1') AND (Get_Data = '1')) OR (Frame_Lost = '1'))) and not ((last_W = '0') AND (Get_Data = '1') AND (Frame_Lost = '0')))) else
           '0';

rst_WCNT_L_assignment:
rst_WCNT_L <= '0' when (current_state_corr = IDLE and Frame_Ready = '0') else
              '1' when (current_state_corr = IDLE and (Frame_Ready = '1' and not (Frame_Ready = '0'))) else
              '0' when (current_state_corr = IDLE) else
              '0' when (current_state_corr = DELIV_FRAME and (((last_W = '1') AND (Get_Data = '1')) OR (Frame_Lost = '1'))) else
              '1' when (current_state_corr = DELIV_FRAME and ((last_W = '0') AND (Get_Data = '1') AND (Frame_Lost = '0') and not ((((last_W = '1') AND (Get_Data = '1')) OR (Frame_Lost = '1'))))) else
              '1' when (current_state_corr = DELIV_FRAME and (Get_Data = '0' and not ((((last_W = '1') AND (Get_Data = '1')) OR (Frame_Lost = '1'))) and not ((last_W = '0') AND (Get_Data = '1') AND (Frame_Lost = '0')))) else
              '0';

read_FD_FIFO_assignment:
read_FD_FIFO <= '0' when (current_state_corr = IDLE and Frame_Ready = '0') else
                '0' when (current_state_corr = IDLE and (Frame_Ready = '1' and not (Frame_Ready = '0'))) else
                '0' when (current_state_corr = IDLE) else
                '1' when (current_state_corr = DELIV_FRAME and (((last_W = '1') AND (Get_Data = '1')) OR (Frame_Lost = '1'))) else
                '0' when (current_state_corr = DELIV_FRAME and ((last_W = '0') AND (Get_Data = '1') AND (Frame_Lost = '0') and not ((((last_W = '1') AND (Get_Data = '1')) OR (Frame_Lost = '1'))))) else
                '0' when (current_state_corr = DELIV_FRAME and (Get_Data = '0' and not ((((last_W = '1') AND (Get_Data = '1')) OR (Frame_Lost = '1'))) and not ((last_W = '0') AND (Get_Data = '1') AND (Frame_Lost = '0')))) else
                '0';
				
seu <= seu_states OR seu_Data_Valid;

Data_Valid <= corr_data_Data_Valid;

end FA_Control2_1;
