-- FIFO_2216_Encoder

-- Encoder for the FIFO 64*16 bit -- single word


LIBRARY IEEE;
USE IEEE.std_logic_1164.all;
USE work.fifo_pkg.all;

entity FIFO_16to22_Encoder is
	generic(
      N_BIT_COD : positive ;
      N_BIT_DEC : positive ;
	  N_BIT_PAR : positive 
	  );
	port(
		 FIFO_din : in std_logic_vector(N_BIT_DEC-1 downto 0);
		 FIFO_dout : out STD_LOGIC_VECTOR(N_BIT_COD-1 downto 0)
	     );
end FIFO_16to22_Encoder;


architecture H22_16 of FIFO_16to22_Encoder is	-- (22,16) SECDED Hamming Encoder 

	signal x : std_logic_vector(0 to N_BIT_DEC-1); -- message vector 
	signal p : std_logic_vector(1 to N_BIT_PAR); -- vector of parity bits
	
begin  -- here FIFO_din is supposed to be 16 bits long for the H(22,16) encoding
		
	x(0 to N_BIT_DEC-1) <= FIFO_din (N_BIT_DEC-1 downto 0);
	
	p(1) <= x(0) XOR x(1) XOR x(2) XOR x(3) XOR x(4) XOR x(5) XOR x(10) XOR x(14);	
	p(2) <= x(0) XOR x(1) XOR x(2) XOR x(6) XOR x(7) XOR x(8) XOR x(9) XOR x(12);	
	p(3) <= x(0) XOR x(4) XOR x(6) XOR x(7) XOR x(8) XOR x(13) XOR x(14) XOR x(15);
	p(4) <= x(5) XOR x(6) XOR x(9) XOR x(10) XOR x(11) XOR x(13) XOR x(14) XOR x(15);
	p(5) <= x(1) XOR x(3) XOR x(7) XOR x(9) XOR x(10) XOR x(11) XOR x(12) XOR x(13);  
	p(6) <= x(3) XOR x(4) XOR x(5) XOR x(11) XOR x(12) XOR x(2) XOR x(8) XOR x(15);
	
	FIFO_dout(N_BIT_COD - 1 downto N_BIT_PAR) <=  x(0 to N_BIT_DEC-1);
	FIFO_dout(N_BIT_PAR-1 downto 0) <=  p(1 to N_BIT_PAR);

end H22_16;
