-- FD_Decoder
--
-- Decoder for the FD field


LIBRARY IEEE;
USE IEEE.std_logic_1164.all;  
USE work.fifo_pkg.all;
use IEEE.std_logic_misc.all;

entity FIFO_18to12_Decoder is
	generic(
      N_BIT_COD : positive ;
      N_BIT_DEC : positive ;
	  N_BIT_PAR : positive 
	  ); 
	port(
		 FIFO_din : in std_logic_vector(N_BIT_COD-1 downto 0);
		 FIFO_dout : out STD_LOGIC_VECTOR(N_BIT_DEC-1 downto 0);
		 serr : out std_logic; 	-- signals single bit error, self correct
		 derr : out std_logic) ;	-- signals double bit error
end FIFO_18to12_Decoder;


architecture H18_12 of FIFO_18to12_Decoder is	

	signal x : std_logic_vector(1 to N_BIT_DEC ); -- data vector received (except parity bits) 
	signal p : std_logic_vector(1 to N_BIT_PAR ); -- parity vector received 
	signal s : std_logic_vector(1 to N_BIT_PAR); -- vector of 5 syndrome bits plus the overall parity bit s(6)
	signal e : std_logic_vector(1 to N_BIT_DEC); -- error vector
	signal d_err_int : std_logic;

begin  
		
	x(1 to N_BIT_DEC ) <= FIFO_din (N_BIT_COD-1 downto N_BIT_PAR);
	p(1 to N_BIT_PAR) <= FIFO_din (N_BIT_PAR-1 downto 0);
	
	-- syndrome calculation
	s(1) <= x(1) XOR x(2) XOR x(4) XOR x(5) XOR x(7) XOR x(9) XOR x(11) XOR x(12) XOR p(1);	
	s(2) <= x(1) XOR x(3) XOR x(4) XOR x(6) XOR x(7) XOR x(10) XOR x(11) XOR p(2);
	s(3) <= x(2) XOR x(3) XOR x(4) XOR x(8) XOR x(9) XOR x(10) XOR x(11) XOR p(3);
	s(4) <= x(5) XOR x(6) XOR x(7) XOR x(8) XOR x(9) XOR x(10) XOR x(11) XOR p(4);
	s(5) <= x(12) XOR p(5);
	
	-- overall parity bit calculation
	s(6) <= x(1) XOR x(2) XOR x(3) XOR x(4) XOR x(5) XOR x(6) XOR x(7) XOR x(8) XOR x(9) XOR x(10) XOR x(11) XOR x(12)
      XOR p(1) XOR p(2) XOR p(3) XOR p(4) XOR p(5) XOR p(6);
	
	-- error vector calculation			 
	e(1)  <= '1' when s(1 to 5) = "11000" else '0';
	e(2)  <= '1' when s(1 to 5) = "10100" else '0';
	e(3)  <= '1' when s(1 to 5) = "01100" else '0';
	e(4)  <= '1' when s(1 to 5) = "11100" else '0';
	e(5)  <= '1' when s(1 to 5) = "10010" else '0';
	e(6)  <= '1' when s(1 to 5) = "01010" else '0';
	e(7)  <= '1' when s(1 to 5) = "11010" else '0'; 
	e(8)  <= '1' when s(1 to 5) = "00110" else '0';
	e(9)  <= '1' when s(1 to 5) = "10110" else '0';
	e(10) <= '1' when s(1 to 5) = "01110" else '0';
	e(11) <= '1' when s(1 to 5) = "11110" else '0';
	e(12) <= '1' when s(1 to 5) = "10001" else '0';
		
    serr <= OR_REDUCE(e) and not(d_err_int);
    
	-- correction logic
	FIFO_dout(11 downto 0) <=  x(1 to 12) XOR e(1 to 12);
-- QUARTUS WARNING	d_err_int <= '1' when (s(1 to 5) /= "000") AND (s(6) = '0') else '0'; 
	d_err_int <= '1' when (s(1 to 5) /= "00000") AND (s(6) = '0') else '0'; 
	derr <= d_err_int;
end H18_12;
