-- FD_Decoder
--
-- Decoder for the FD field


LIBRARY IEEE;
USE IEEE.std_logic_1164.all;  
USE work.fifo_pkg.all;


entity FIFO_22to16_Decoder is
	generic(
      N_BIT_COD : positive ;
      N_BIT_DEC : positive ;
	  N_BIT_PAR : positive 
	  ); 
	port(
		 FIFO_din : in std_logic_vector(N_BIT_COD-1 downto 0);
		 FIFO_dout : out STD_LOGIC_VECTOR(N_BIT_DEC-1 downto 0);
		 serr : out std_logic; 	-- signals single bit error, self correct
		 derr : out std_logic) ;	-- signals double bit error
end FIFO_22to16_Decoder;


architecture H22_16 of FIFO_22to16_Decoder is	-- (22,16) SECDED Hamming Decoder 

	signal y : std_logic_vector(0 to N_BIT_DEC-1 ); -- data vector received (except parity bits) 
	signal p : std_logic_vector(1 to N_BIT_PAR ); -- parity vector received 
	signal s : std_logic_vector(1 to N_BIT_PAR); -- vector of 5 syndrome bits plus the overall parity bit s(6)
	signal e : std_logic_vector(0 to N_BIT_DEC-1); -- error vector
	
begin  -- here FIFO_din is supposed to be 22 bits long for the H(22,16) decoding
		
	y(0 to N_BIT_DEC-1 ) <= FIFO_din (N_BIT_COD-1 downto N_BIT_PAR);
	p(1 to N_BIT_PAR) <= FIFO_din (N_BIT_PAR-1 downto 0);
	-- syndrome calculation
	s(1) <= y(0) XOR y(1) XOR y(2) XOR y(3) XOR y(4) XOR y(5) XOR y(10) XOR y(14) XOR p(1);	
	s(2) <= y(0) XOR y(1) XOR y(2) XOR y(6) XOR y(7) XOR y(8) XOR y(9) XOR y(12) XOR p(2);	 
	s(3) <= y(0) XOR y(4) XOR y(6) XOR y(7) XOR y(8) XOR y(13) XOR y(14) XOR y(15) XOR p(3);	
	s(4) <=	y(5) XOR y(6) XOR y(9) XOR y(10) XOR y(11) XOR y(13) XOR y(14) XOR y(15) XOR p(4);		 
	s(5) <=	y(1) XOR y(3) XOR y(7) XOR y(9) XOR y(10) XOR y(11) XOR y(12) XOR y(13) XOR p(5);
	s(6) <=	y(2) XOR y(3) XOR y(4) XOR y(5) XOR y(8) XOR y(11) XOR y(12) XOR y(15) XOR p(6);
	
	
	-- error vector calculation			 
	e(0) <= '1' when s(1 to 6) = "111000" else '0';
	e(1) <= '1' when s(1 to 6) = "110010" else '0';
	e(2) <= '1' when s(1 to 6) = "110001" else '0';
	e(3) <= '1' when s(1 to 6) = "100011" else '0';
	e(4) <= '1' when s(1 to 6) = "101001" else '0';
	e(5) <= '1' when s(1 to 6) = "100101" else '0';
	e(6) <= '1' when s(1 to 6) = "011100" else '0';
	e(7) <= '1' when s(1 to 6) = "011010" else '0'; 
	e(8) <= '1' when s(1 to 6) = "011001" else '0';
	e(9) <= '1' when s(1 to 6) = "010110" else '0';
	e(10) <= '1' when s(1 to 6) = "100110" else '0';
	e(11) <= '1' when s(1 to 6) = "000111" else '0';
	e(12) <= '1' when s(1 to 6) = "010011" else '0';
	e(13) <= '1' when s(1 to 6) = "001110" else '0';
	e(14) <= '1' when s(1 to 6) = "101100" else '0'; 
	e(15) <= '1' when s(1 to 6) = "001101" else '0';
		
		
	-- correction logic
	FIFO_dout(N_BIT_DEC-1 downto 0) <=  y(0 to N_BIT_DEC-1) XOR e(0 to N_BIT_DEC-1);
	serr <=  '1' when ((s(1) XOR s(2) XOR s(3) XOR s(4) XOR s(5) XOR s(6)) = '1' ) else '0';
	derr <= '1' when (((s(1) XOR s(2) XOR s(3) XOR s(4) XOR s(5) XOR s(6)) = '0') AND (s(1 to 6) /= "000000")) else '0';	

end H22_16;
