
  
library IEEE;
use IEEE.STD_LOGIC_1164.all; 
USE IEEE.std_logic_arith.all;	
use IEEE.std_logic_unsigned.all; 
use IEEE.math_real.all;
use IEEE.numeric_std.all;
USE work.fifo_pkg.all;

-- Entity declarations --

entity FIFO_CORE_32x14_EN is		 
	generic(
	 N_WORD: positive ; 
	 N_MUX_WORD: positive ; 
	 N_WORD_ADDR: positive ; 
	 N_BIT_COD: positive ; 
	 N_ADDR: positive 	  
	);  	
	port(	
		 
		 rst_l : in STD_LOGIC;
		 read_core : in STD_LOGIC;
		 write_core : in STD_LOGIC;
		 clk : in STD_LOGIC;
		 write_scrub : in STD_LOGIC;   
		 scrub_addr : in STD_LOGIC_VECTOR(N_ADDR-1 downto 0);
		 full : out STD_LOGIC;
		 empty : out STD_LOGIC;	 	 
		 word_in : in STD_LOGIC_VECTOR(N_BIT_COD-1 downto 0);
		 word_out : out STD_LOGIC_VECTOR(N_BIT_COD-1 downto 0);
		 word_out_scrub : out STD_LOGIC_VECTOR(N_BIT_COD-1 downto 0);
		 w_pnt_out : out STD_LOGIC_VECTOR(N_ADDR-1 downto 0); 
		 r_pnt_out : out STD_LOGIC_VECTOR(N_ADDR-1 downto 0);
		 
		 -- Interface to the array of FF-E
		 en_int: out STD_LOGIC_VECTOR(N_WORD-1 downto 0);
		 word_in_int : out STD_LOGIC_VECTOR(N_BIT_COD-1 downto 0);
		 word_out_int : fifo_array_type_RX_FD;
		 word_addr : out STD_LOGIC_VECTOR(N_WORD_ADDR-1 downto 0);

		 enable : in STD_logic
	   	 
	     );
end FIFO_CORE_32x14_EN;

-- Architecture -- 

 architecture FIFO_CORE_32x14_EN of FIFO_CORE_32x14_EN is
	 
	signal W_pnt : INTEGER range 0 to N_WORD-1; -- write pointer
	signal R_pnt : INTEGER range 0 to N_WORD-1;  -- read pointer
	signal scrub_pnt : INTEGER range 0 to N_WORD-1;  -- scrubbing pointer
	signal out_bus_sel : INTEGER range 0 to N_MUX_WORD-1; -- selection mux word for fifo output
	signal scrub_bus_sel : INTEGER range 0 to N_MUX_WORD-1; -- selection mux word for scrubbing
	signal word_addr_func : STD_LOGIC_VECTOR (N_WORD_ADDR-1 downto 0);
	signal word_addr_scr : STD_LOGIC_VECTOR (N_WORD_ADDR-1 downto 0);
	signal empty_sig : STD_LOGIC;
	signal full_sig : STD_LOGIC;
		  		
begin
	process (clk) -- write process	 
	
	variable w_p, r_p : INTEGER range 0 to N_WORD-1; -- write/read pointer local variables
	begin
		
		if rising_edge(clk) then
			if rst_l = '0' then			
				W_pnt <= 0;
				R_pnt <= 0;
				empty_sig <= '1' ; 
				full_sig <= '0' ;
			elsif (enable = '1') then  
				if write_core = '1' and full_sig = '0' then 
					if W_pnt < N_WORD-1 then
						w_p := W_pnt + 1;
					elsif W_pnt = N_WORD-1 then
						w_p := 0;
					end if;
					W_pnt <= w_p  ;
					if (read_core /= '1') OR (empty_sig = '1') then   
						empty_sig <= '0'  ;
						if w_p = R_pnt then
							full_sig <= '1'  ;
						end if;
					end if;
				end if;		 
				
				if read_core = '1' and empty_sig = '0' then 
					if R_pnt < N_WORD-1 then
						r_p := R_pnt + 1;
					elsif R_pnt = N_WORD-1 then
						r_p := 0;
					end if;
					R_pnt <= r_p  ;
					if (write_core /= '1') OR (full_sig= '1') then -- empty and full flags must be updated only if a writing has been actually performed
						full_sig <= '0'  ;
						if r_p = W_pnt then
							empty_sig <= '1'  ;
						end if;
					end if;
				end if;	 
			end if;
		end if;
	end process;   
	
	process (W_pnt, scrub_addr, write_core, full_sig, write_scrub)   
	begin		 
		if write_core = '1' and full_sig = '0' then	  
			for i in 0 to N_WORD-1 loop
				if i = W_pnt then	
				    en_int(i)<= '1';
				else 
					en_int(i) <= '0';
				end if;		  
			end loop;  
		elsif write_scrub = '1' and write_core = '0' then			
			for i in 0 to N_WORD-1 loop
				if i = CONV_INTEGER(scrub_addr) then	
					en_int(i)<= '1';
				else 
					en_int(i) <= '0';
				end if;		  
			end loop;	
		else 
			for i in 0 to N_WORD-1 loop
				en_int(i) <= '0';  
			end loop;  
		end if;
 
	end process;
	
    process (R_pnt, scrub_pnt)	   
	begin 
		if R_pnt >= 0 and R_pnt <= 3 then
			word_addr_func <= CONV_STD_LOGIC_VECTOR(R_pnt, N_WORD_ADDR);
			out_bus_sel <= 0;
		elsif R_pnt >= 4 and R_pnt <= 7 then
			word_addr_func <= std_logic_vector(rotate_left(TO_UNSIGNED(R_pnt-4, N_WORD_ADDR), 2));
			out_bus_sel <= 1;
		elsif R_pnt >= 8 and R_pnt <= 11 then
			word_addr_func <= std_logic_vector(rotate_left(TO_UNSIGNED(R_pnt-8, N_WORD_ADDR),4));
			out_bus_sel <= 2;
		elsif R_pnt >= 12 and R_pnt <= 15 then	
			word_addr_func <= std_logic_vector(rotate_left(TO_UNSIGNED(R_pnt-12, N_WORD_ADDR),6));
			out_bus_sel <= 3;
		elsif R_pnt >= 16 and R_pnt <= 19 then
			word_addr_func <= std_logic_vector(rotate_left(TO_UNSIGNED(R_pnt-16, N_WORD_ADDR),8));
			out_bus_sel <= 4;
		elsif R_pnt >= 20 and R_pnt <= 23 then
			word_addr_func <= std_logic_vector(rotate_left(TO_UNSIGNED(R_pnt-20, N_WORD_ADDR),10));
			out_bus_sel <= 5;
		elsif R_pnt >= 24 and R_pnt <= 27 then
			word_addr_func <= std_logic_vector(rotate_left(TO_UNSIGNED(R_pnt-24, N_WORD_ADDR),12));
			out_bus_sel <= 6;
		else
			word_addr_func <= std_logic_vector(rotate_left(TO_UNSIGNED(R_pnt-28, N_WORD_ADDR),14));
			out_bus_sel <= 7;
		end if;	  
		
		
		if scrub_pnt >= 0 and scrub_pnt <= 3 then
			word_addr_scr <= CONV_STD_LOGIC_VECTOR(scrub_pnt, N_WORD_ADDR);
			scrub_bus_sel <= 0;
		elsif scrub_pnt >= 4 and scrub_pnt <= 7 then
			word_addr_scr <= std_logic_vector(rotate_left(TO_UNSIGNED(scrub_pnt-4, N_WORD_ADDR),2));
			scrub_bus_sel <= 1;
		elsif scrub_pnt >= 8 and scrub_pnt <= 11 then
			word_addr_scr <= std_logic_vector(rotate_left(TO_UNSIGNED(scrub_pnt-8, N_WORD_ADDR),4));
			scrub_bus_sel <= 2;
		elsif scrub_pnt >= 12 and scrub_pnt <= 15 then	
			word_addr_scr <= std_logic_vector(rotate_left(TO_UNSIGNED(scrub_pnt-12, N_WORD_ADDR),6));
			scrub_bus_sel <= 3;
		elsif scrub_pnt >= 16 and scrub_pnt <= 19 then
			word_addr_scr <= std_logic_vector(rotate_left(TO_UNSIGNED(scrub_pnt-16, N_WORD_ADDR),8));
			scrub_bus_sel <= 4;
		elsif scrub_pnt >= 20 and scrub_pnt <= 23 then
			word_addr_scr <= std_logic_vector(rotate_left(TO_UNSIGNED(scrub_pnt-20, N_WORD_ADDR),10));
			scrub_bus_sel <= 5;
		elsif scrub_pnt >= 24 and scrub_pnt <= 27 then
			word_addr_scr <= std_logic_vector(rotate_left(TO_UNSIGNED(scrub_pnt-24, N_WORD_ADDR),12));
			scrub_bus_sel <= 6;
		else
			word_addr_scr <= std_logic_vector(rotate_left(TO_UNSIGNED(scrub_pnt-28, N_WORD_ADDR),14));
			scrub_bus_sel <= 7;
		end if;	
		
	end process;  
	

	word_addr <= (word_addr_scr or word_addr_func) when ((scrub_pnt > R_pnt + 3) or (scrub_pnt < R_pnt - 3)) else
				word_addr_func;
	scrub_pnt <= CONV_INTEGER(scrub_addr);
	
	word_in_int <= word_in;	 
	word_out <= word_out_int(out_bus_sel); 
	word_out_scrub <= word_out_int(scrub_bus_sel);		 
	full <= full_sig;
	empty <= empty_sig;		
	w_pnt_out <= CONV_STD_LOGIC_VECTOR(W_pnt, N_ADDR); 
	r_pnt_out <= CONV_STD_LOGIC_VECTOR(R_pnt, N_ADDR);

end FIFO_CORE_32x14_EN;
