-- FIFO_mux

-- The proper signals are selected between different interfaces versions and test or functional mode


LIBRARY IEEE;
USE IEEE.std_logic_1164.all;
USE work.fifo_pkg.all;

entity FIFO_mux is
	generic(
      N_BIT_DEC : positive 
	  );
	port(
		-- Selection Signals
		fifo_sel : in std_logic_vector(1 downto 0);
		fifo_ded_en : in std_logic;
		fifo_test_en : in std_logic;
		
		-- Data Signals

		--word_in
		seu_4x_word_in : in std_logic_vector(N_BIT_DEC-1 downto 0);
		seu_8x_word_in : in std_logic_vector(N_BIT_DEC-1 downto 0);
		seu_16x_word_in : in std_logic_vector(N_BIT_DEC-1 downto 0);
		ded_4x_word_in : in std_logic_vector(N_BIT_DEC-1 downto 0);
		ded_8x_word_in : in std_logic_vector(N_BIT_DEC-1 downto 0);
		ded_16x_word_in : in std_logic_vector(N_BIT_DEC-1 downto 0);
		test_word_in : in std_logic_vector(N_BIT_DEC-1 downto 0);
		-- write
		seu_4x_write : in std_logic;
		seu_8x_write : in std_logic;
		seu_16x_write : in std_logic;
		ded_4x_write : in std_logic;
		ded_8x_write : in std_logic;
		ded_16x_write : in std_logic;
		test_write : in std_logic;
		--read
		seu_4x_read : in std_logic;
		seu_8x_read : in std_logic;
		seu_16x_read : in std_logic;
		ded_4x_read : in std_logic;
		ded_8x_read : in std_logic;
		ded_16x_read : in std_logic;
		test_read : in std_logic;
		
		--signal2FIFO_logic	
		word_in : out std_logic_vector(N_BIT_DEC-1 downto 0); 
		write : out std_logic;
		read : out std_logic
		
		);     
end FIFO_mux;


architecture FIFO_mux1 of FIFO_mux is	
		--word_in
		signal seu_word_in : std_logic_vector(N_BIT_DEC-1 downto 0);  
		signal ded_word_in : std_logic_vector(N_BIT_DEC-1 downto 0); 
		signal word_in_sig : std_logic_vector(N_BIT_DEC-1 downto 0);
		--write
		signal seu_write : std_logic;  
		signal ded_write : std_logic; 
		signal write_sig : std_logic;
		--read
		signal seu_read : std_logic;  
		signal ded_read : std_logic; 
		signal read_sig : std_logic;
		

begin  
	--word_in
seu_word_in <= 	seu_4x_word_in when fifo_sel = "01" else
					seu_8x_word_in when fifo_sel = "10" else 
					seu_16x_word_in when fifo_sel = "11" else
					(others => '0'); 
					
ded_word_in <= 	ded_4x_word_in when fifo_sel = "01" else
					ded_8x_word_in when fifo_sel = "10" else 
					ded_16x_word_in when fifo_sel = "11" else
					(others => '0'); 	 
					
word_in_sig <= 		seu_word_in when fifo_ded_en = '0' else
					ded_word_in when fifo_ded_en  = '1' ;
	
word_in <= 			word_in_sig when fifo_test_en = '0' else
					test_word_in when fifo_test_en = '1' ;
	--write	
seu_write <= 	seu_4x_write when fifo_sel = "01" else
					seu_8x_write when fifo_sel = "10" else 
					seu_16x_write when fifo_sel = "11" else
					'0'; 
					
ded_write <= 	ded_4x_write when fifo_sel = "01" else
					ded_8x_write when fifo_sel = "10" else 
					ded_16x_write when fifo_sel = "11" else
					'0'; 	 
					
write_sig <= 		seu_write when fifo_ded_en = '0' else
					ded_write when fifo_ded_en  = '1' ;
	
write <= 			write_sig when fifo_test_en = '0' else
					test_write when fifo_test_en = '1' ; 
	--read
seu_read <= 		seu_4x_read when fifo_sel = "01" else
					seu_8x_read when fifo_sel = "10" else 
					seu_16x_read when fifo_sel = "11" else
					'0'; 
					
ded_read <= 		ded_4x_read when fifo_sel = "01" else
					ded_8x_read when fifo_sel = "10" else 
					ded_16x_read when fifo_sel = "11" else
					'0'; 	 
					
read_sig <= 		seu_read when fifo_ded_en = '0' else
					ded_read when fifo_ded_en  = '1' ;
	
read <= 			read_sig when fifo_test_en = '0' else
					test_read when fifo_test_en = '1';	
	

end FIFO_mux1;
