-- reg_N
--
-- N-bit register with triple redundancy (loop to recover correct value from the output) 

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_misc.all;

entity fifo_reg_N_RHSEU is
	generic (N : integer );
	port (
		data_in : in std_logic;
		clk : in std_logic;
		rst_L : in std_logic; -- active low reset
		write : in std_logic_vector(n-1 downto 0); -- write command
		data_out : out std_logic_vector(N-1 downto 0)
	);
end entity;

architecture reg1 of fifo_reg_N_RHSEU is
	type array_nx3 is array (n-1 downto 0) of std_logic_vector(0 to 2);
	signal reg : array_nx3;
	signal maj_out : std_logic_vector(n-1 downto 0);  

	component fifo_maj_voter 
		port (  a : in std_logic;
				b : in std_logic;
				c : in std_logic;
				q : out std_logic
				);  
	end component;

begin 
	
    process (clk)
	begin
   
		for i in 0 to n-1 loop
			if rising_edge(clk) then
				if (rst_L = '0') then
					reg(i) <= (others => '0');
				elsif (write(i) = '1') then
					reg(i) <= (others => data_in);
				else
					reg(i) <= (others => maj_out(i));
				end if;
			end if;
		end loop;      
	end process;
	      
	maj_vot_inst: for i in 0 to n-1 generate  
			maj_vot: fifo_maj_voter PORT MAP (reg(i)(0), reg(i)(1), reg(i)(2), maj_out(i));		
	end generate; 
	
	data_out <= maj_out;					 
end architecture;
