
-------------------------------------------------------------------------------
--
-- Description : 
--
-------------------------------------------------------------------------------
-- Design unit header --
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_misc.all;
use IEEE.numeric_std.all;
use work.interfaces_pkg.all;


-- active library clause
 

entity Frame_Analyser_RHSEU is
  port(
       -- transmission ("fast") clock
       clk : in std_logic;
       clk_enable : in std_logic;
       --------------------------
       FDC_rcv : in std_logic;
       Get_Data : in STD_LOGIC;
       rst_L : in std_logic;
       sel_shr : in std_logic;
       FD_FIFO_empty : in std_logic;
       word_in : in std_logic_vector(N_WORD-1 downto 0);
       FD_FIFO_out : in STD_LOGIC_VECTOR (N_FD downto 0);
       FD_FIFO_in : out STD_LOGIC_VECTOR (N_FD downto 0);
       Data_Type : out STD_LOGIC;
       Data_Valid : out STD_LOGIC;
       Frame_Lost : out STD_LOGIC;
       Label_On : out STD_LOGIC;
       Last_Frame : out STD_LOGIC;
       write_word : out STD_LOGIC;
       write_FD_FIFO : out STD_LOGIC;
       read_FD_FIFO : out STD_LOGIC;
       FD_SEU : out std_logic; 
	   FD_DEU : out std_logic;	 
       seu : out STD_LOGIC     
  );
end Frame_Analyser_RHSEU;

architecture Frame_Analyser1 of Frame_Analyser_RHSEU is

---- Component declarations -----

component Counter_N_RHSEU
  generic(
       N : INTEGER;
       modul : INTEGER
  );
  port (
       clk : in STD_LOGIC;
       en : in STD_LOGIC;
       rst_L : in STD_LOGIC;
       count : out STD_LOGIC_VECTOR(N-1 downto 0);  
       seu : out STD_LOGIC     
  );
end component;
component FA_Control1_RHSEU
  port (
       -- transmission ("fast") clock
       clk : in std_logic;
       clk_enable : in std_logic;
       --------------------------
       FDC_rcv : in STD_LOGIC;
       FD_err : in STD_LOGIC;
       last_W : in STD_LOGIC;
       rst_L : in STD_LOGIC;
       sel_shr : in STD_LOGIC;
       en_WCNT : out STD_LOGIC;
       rst_WCNT_L : out STD_LOGIC;
       write_FD_FIFO : out STD_LOGIC;
       write_FD_reg : out STD_LOGIC;
       write_word : out STD_LOGIC;  
       seu : out STD_LOGIC     
  );
end component;
component FA_Control2_RHSEU
  port (
       -- transmission ("fast") clock
       clk : in std_logic;
       clk_enable : in std_logic;
       --------------------------
       Frame_Lost : in STD_LOGIC;
       Frame_Ready : in STD_LOGIC;
       Get_Data : in STD_LOGIC;
       last_W : in STD_LOGIC;
       rst_L : in STD_LOGIC;
       Data_Valid : out STD_LOGIC;
       en_WCNT : out STD_LOGIC;
       read_FD_FIFO : out STD_LOGIC;
       rst_WCNT_L : out STD_LOGIC;  
       seu : out STD_LOGIC     
  );
end component;
component FD_Decoder
  port (
		 FDC : in std_logic_vector(N_FDC-1 downto 0);
		 FD : out STD_LOGIC_VECTOR(N_FD-1 downto 0);
		 FD_SEU : out std_logic;
		 FD_DEU : out std_logic
  );
end component;
component reg_N_RHSEU
  generic(
       N : INTEGER
  );
  port (
       clk : in STD_LOGIC;
       data_in : in STD_LOGIC_VECTOR(N-1 downto 0);
       rst_L : in STD_LOGIC;
       write : in STD_LOGIC;
       data_out : out STD_LOGIC_VECTOR(N-1 downto 0);  
       seu : out STD_LOGIC     
  );
end component;

---- Signal declarations used on the diagram ----

signal en_WCNT1 : STD_LOGIC;
signal en_WCNT1_eff : STD_LOGIC;
signal en_WCNT2 : STD_LOGIC;
signal en_WCNT2_eff : STD_LOGIC;
signal FD_SEU_int : STD_LOGIC;
signal FD_DEU_int : STD_LOGIC;
signal FD_DEU_int1 : STD_LOGIC;
signal Frame_Lost_int : STD_LOGIC;
signal Frame_Ready : STD_LOGIC;
signal last_W1 : STD_LOGIC;
signal last_W2 : STD_LOGIC;
signal rst_WCNT1_L : STD_LOGIC;
signal rst_WCNT1_eff_L : STD_LOGIC;
signal rst_WCNT2_L : STD_LOGIC;
signal rst_WCNT2_eff_L : STD_LOGIC;
signal write_FD_reg : STD_LOGIC;
signal write_FD_reg_eff : STD_LOGIC;
signal countW1 : STD_LOGIC_VECTOR (N_FL-1 downto 0);
signal countW2 : STD_LOGIC_VECTOR (N_FL-1 downto 0);
signal FD : STD_LOGIC_VECTOR (N_FD-1 downto 0);
signal FDC : STD_LOGIC_VECTOR (N_FDC-1 downto 0);
signal FD_reg_in : STD_LOGIC_VECTOR (N_FD+1 downto 0);
signal FD_reg_out : STD_LOGIC_VECTOR (N_FD+1 downto 0);
signal FD_FIFO_in_int : STD_LOGIC_VECTOR (N_FD downto 0);
signal FL1 : STD_LOGIC_VECTOR (N_FL-1 downto 0);
signal FL2 : STD_LOGIC_VECTOR (N_FL-1 downto 0);
signal seu_CTRL1, seu_CTRL2, seu_FD_REG, seu_WCNT1, seu_WCNT2: STD_LOGIC;

begin

---- User Signal Assignments ----
FDC <=word_in(N_WORD-1 downto( N_WORD-N_FDC));
last_W1 <= '1' when countW1 = FL1 else '0';
FL1 <= FD_reg_out(N_FD-1 downto N_FD-N_FL);
FD_DEU_int1 <= FD_reg_out(N_FD);
FL2 <= FD_FIFO_out(N_FD-1 downto N_FD-N_FL); 
Label_On <= FD_FIFO_out(N_FD-N_FL-1);
Data_Type <= FD_FIFO_out(N_FD-N_FL-2);
Last_Frame <= FD_FIFO_out(N_FD-N_FL-3);
Frame_Lost_int <= FD_FIFO_out(N_FD);
last_W2 <= '1' when countW2 = FL2 else '0';
FD_reg_in(N_FD-1 downto 0) <= FD;
FD_reg_in(N_FD) <= FD_DEU_int;
FD_reg_in(N_FD+1) <= FD_SEU_int;

----  Component instantiations  ----

CTRL1 : FA_Control1_RHSEU
  port map(
       FDC_rcv => FDC_rcv,
       FD_err => FD_DEU_int1,
       clk => clk,
       clk_enable => clk_enable,
       en_WCNT => en_WCNT1,
       last_W => last_W1,
       rst_L => rst_L,
       rst_WCNT_L => rst_WCNT1_L,
       sel_shr => sel_shr,
       write_FD_FIFO => write_FD_FIFO,
       write_FD_reg => write_FD_reg,
       write_word => write_word,
       seu => seu_CTRL1
  );

CTRL2 : FA_Control2_RHSEU
  port map(
       Data_Valid => Data_Valid,
       Frame_Lost => Frame_Lost_int,
       Frame_Ready => Frame_Ready,
       Get_Data => Get_Data,
       clk => clk,
       clk_enable => clk_enable,
       en_WCNT => en_WCNT2,
       last_W => last_W2,
       read_FD_FIFO => read_FD_FIFO,
       rst_L => rst_L,
       rst_WCNT_L => rst_WCNT2_L,
       seu => seu_CTRL2
  );

FD_DEC : FD_Decoder
  port map(
       FD => FD( N_FD-1 downto 0 ),
       FDC => FDC( N_FDC-1 downto 0 ),
       FD_SEU => FD_SEU_int,
       FD_DEU => FD_DEU_int
  );


FD_reg : reg_N_RHSEU
  generic map (
       N => N_FD+2
  )
  port map(
       clk => clk,
       data_in => FD_reg_in( N_FD+1 downto 0 ),
       data_out => FD_reg_out, 
       rst_L => rst_L,
       write => write_FD_reg_eff,
       seu => seu_FD_REG
  );

FD_FIFO_in_int <= FD_reg_out(N_FD downto 0);  
FD_DEU <= FD_reg_out(N_FD);  
FD_SEU <= FD_reg_out(N_FD+1);
  
write_FD_reg_eff <= write_FD_reg and clk_enable;

Frame_Ready <= not(FD_FIFO_empty);

WORD_CNT1 : Counter_N_RHSEU
  generic map (
       N => N_FL,
       modul => N_FLpwr
  )
  port map(
       clk => clk,
       count => countW1( N_FL-1 downto 0 ),
       en => en_WCNT1_eff,
       rst_L => rst_WCNT1_eff_L,
       seu => seu_WCNT1
  );

  en_WCNT1_eff <= en_WCNT1 and clk_enable;
  rst_WCNT1_eff_L <= rst_L and (rst_WCNT1_L or not(clk_enable));
  
WORD_CNT2 : Counter_N_RHSEU
  generic map (
       N => N_FL,
       modul => N_FLpwr
  )
  port map(
       clk => clk,
       count => countW2( N_FL-1 downto 0 ),
       en => en_WCNT2_eff,
       rst_L => rst_WCNT2_eff_L,
       seu => seu_WCNT2
  );

  en_WCNT2_eff <= en_WCNT2 and clk_enable;
  rst_WCNT2_eff_L <= rst_L and (rst_WCNT2_L or not(clk_enable));

---- Terminal assignment ----

    -- Output\buffer terminals
	Frame_Lost <= Frame_Lost_int;

    FD_FIFO_in <= FD_FIFO_in_int;
    
    seu <= seu_CTRL1 or seu_CTRL2 or seu_FD_REG or seu_WCNT1 or seu_WCNT2;

    
end Frame_Analyser1;
