
-------------------------------------------------------------------------------
--
-- Description : 
--
-------------------------------------------------------------------------------
-- Design unit header --
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_misc.all;
use IEEE.math_real.all;
use IEEE.numeric_std.all;
use work.interfaces_pkg.all;


-- active library clause


entity Frame_Builder_RHSEU is
  port(
       Data_Type : in STD_LOGIC;
       Data_Valid : in STD_LOGIC;
       Get_Data : in STD_LOGIC;
       Label_On : in STD_LOGIC;
       clk : in STD_LOGIC;
       delay_HDR : in STD_LOGIC;
       rst_L : in STD_LOGIC;
       sel_shr : in STD_LOGIC;
       FDC_FIFO_empty : in STD_LOGIC;
       FDC_FIFO_out : in STD_LOGIC_VECTOR(N_FDC-1 downto 0);
       HDR_cmd : out STD_LOGIC;
       Read_TX_Buf : out STD_LOGIC;
       stop_SER : out STD_LOGIC;
       read_FDC_FIFO : out STD_LOGIC;
       write_FDC_FIFO : out STD_LOGIC;
       FDC_FIFO_in : out STD_LOGIC_VECTOR (N_FDC-1 downto 0);
	   seu_out : out  STD_LOGIC
  );
end Frame_Builder_RHSEU;

architecture Frame_Builder1 of Frame_Builder_RHSEU is

---- Component declarations -----

component Counter_N_RHSEU
  generic(
       N : INTEGER;
       modul : INTEGER
  );
  port (
       clk : in STD_LOGIC;
       en : in STD_LOGIC;
       rst_L : in STD_LOGIC;
       count : out STD_LOGIC_VECTOR(N-1 downto 0); 
	   seu : out  STD_LOGIC
  );
end component;
component FB_Control1_RHSEU
  port (
       DV : in STD_LOGIC;
       Frm_Lim : in STD_LOGIC;
       GD : in STD_LOGIC;
       clk : in STD_LOGIC;
       rst_L : in STD_LOGIC;
       LF : out STD_LOGIC;
       rst_WCNT_L : out STD_LOGIC;
       write_FLLF : out STD_LOGIC;
       write_LODT : out STD_LOGIC;
	   seu : out  STD_LOGIC

  );
end component;
component FB_Control2_RHSEU
  port (
       Frame_ready : in STD_LOGIC;
       clk : in STD_LOGIC;
       delay_HDR : in STD_LOGIC;
       last_W : in STD_LOGIC;
       rst_L : in STD_LOGIC;
       sel_shr : in STD_LOGIC;
       HDR_cmd : out STD_LOGIC;
       en_WCNT : out STD_LOGIC;
       read_FDC_FIFO : out STD_LOGIC;
       read_TX_Buf : out STD_LOGIC;
       rst_WCNT_L : out STD_LOGIC;
       stop_SER : out STD_LOGIC;
	   seu : out  STD_LOGIC

  );
end component;
component fd_encoder
  port (
       FD : in STD_LOGIC_VECTOR(N_FD-1 downto 0);
       FDC : out STD_LOGIC_VECTOR(N_FDC-1 downto 0)
  );
end component;
component ffd_RHSEU
  port (
       CLR : in STD_LOGIC;
       D : in STD_LOGIC;
       clk : in STD_LOGIC;
       en : in STD_LOGIC;
       Q : out STD_LOGIC;
       QN : out STD_LOGIC;
	   seu : out  STD_LOGIC

  );
end component;
component reg_N_RHSEU
  generic(
       N : INTEGER
  );
  port (
       clk : in STD_LOGIC;
       data_in : in STD_LOGIC_VECTOR(N-1 downto 0);
       rst_L : in STD_LOGIC;
       write : in STD_LOGIC;
       data_out : out STD_LOGIC_VECTOR(N-1 downto 0);
	   seu : out  STD_LOGIC

  );
end component;

----     Constants     -----
constant VCC_CONSTANT   : STD_LOGIC := '1';

---- Signal declarations used on the diagram ----

signal DT : STD_LOGIC;
signal en_WCNT1 : STD_LOGIC;
signal en_WCNT2 : STD_LOGIC;
signal Frame_Limit : STD_LOGIC;
signal Frame_ready : STD_LOGIC;
signal HDR_cmd_int : STD_LOGIC;
signal last_W : STD_LOGIC;
signal LF : STD_LOGIC;
signal LF_pre : STD_LOGIC;
signal LO : STD_LOGIC;
signal rst_WCNT1_L : STD_LOGIC;
signal rst_WCNT2_L : STD_LOGIC;
signal VCC : STD_LOGIC;
signal write_FLLF : STD_LOGIC;
signal write_LODT : STD_LOGIC;
signal countW1 : STD_LOGIC_VECTOR (N_FL-1 downto 0);
signal countW2 : STD_LOGIC_VECTOR (N_FL-1 downto 0);
signal FD : STD_LOGIC_VECTOR (N_FD-1 downto 0);
signal FL1 : STD_LOGIC_VECTOR (N_FL-1 downto 0);
signal FL2 : STD_LOGIC_VECTOR (N_FL-1 downto 0);
signal seu_out_DEL , seu_out_DT_reg , seu_out_FB_ctr1 , seu_out_FB_ctr2 ,seu_out_FL_reg , seu_out_LF_reg , 
	   seu_out_LO_reg , seu_out_WORD_CNT1, seu_out_WORD_CNT2: STD_LOGIC;


begin

---- User Signal Assignments ----
FD(N_FD-1 downto N_FD-N_FL) <= FL1; 
FD(N_FD-N_FL-1) <= LO; 
FD(N_FD-N_FL-2) <= DT; 
FD(N_FD-N_FL-3) <= LF;
Frame_Limit <= '1' when countW1 = CONV_STD_LOGIC_VECTOR(N_FLpwr-1, N_FL) else '0';
FL2 <= FDC_FIFO_out(N_FDC-1 downto N_FDC-N_FL);
last_W <= '1' when countW2 = FL2 else '0';

----  Component instantiations  ----

DEL : ffd_RHSEU
  port map(
       CLR => rst_L,
       D => write_FLLF,
       Q => write_FDC_FIFO,
       clk => clk,
       en => VCC,
	   seu => seu_out_DEL
  );

DT_reg : ffd_RHSEU
  port map(
       CLR => rst_L,
       D => Data_Type,
       Q => DT,
       clk => clk,
       en => write_LODT,
	   seu => seu_out_DT_reg
  );

FB_CTRL1 : FB_Control1_RHSEU
  port map(
       DV => Data_Valid,
       Frm_Lim => Frame_Limit,
       GD => Get_Data,
       LF => LF_pre,
       clk => clk,
       rst_L => rst_L,
       rst_WCNT_L => rst_WCNT1_L,
       write_FLLF => write_FLLF,
       write_LODT => write_LODT,
	   seu => seu_out_FB_ctr1
  );

FB_CTRL2 : FB_Control2_RHSEU
  port map(
       Frame_ready => Frame_ready,
       HDR_cmd => HDR_cmd_int,
       clk => clk,
       delay_HDR => delay_HDR,
       en_WCNT => en_WCNT2,
       last_W => last_W,
       read_FDC_FIFO => read_FDC_FIFO,
       read_TX_Buf => Read_TX_Buf,
       rst_L => rst_L,
       rst_WCNT_L => rst_WCNT2_L,
       sel_shr => sel_shr,
       stop_SER => stop_SER,
	   seu => seu_out_FB_ctr2
  );


FD_ENC : fd_encoder
  port map(
       FD => FD( N_FD-1 downto 0 ),
       FDC => FDC_FIFO_in( N_FDC-1 downto 0 )
  );

FL_reg : reg_N_RHSEU
  generic map (
       N => N_FL
  )
  port map(
       clk => clk,
       data_in => countW1( N_FL-1 downto 0 ),
       data_out => FL1( N_FL-1 downto 0 ),
       rst_L => rst_L,
       write => write_FLLF,
	   seu => seu_out_FL_reg
  );

LF_reg : ffd_RHSEU
  port map(
       CLR => rst_L,
       D => LF_pre,
       Q => LF,
       clk => clk,
       en => write_FLLF,
	   seu => seu_out_LF_reg
  );

LO_reg : ffd_RHSEU
  port map(
       CLR => rst_L,
       D => Label_On,
       Q => LO,
       clk => clk,
       en => write_LODT,
	   seu => seu_out_LO_reg
  );

Frame_ready <= not(FDC_FIFO_empty);

en_WCNT1 <= Get_Data and Data_Valid;

WORD_CNT1 : Counter_N_RHSEU
  generic map (
       N => N_FL,
       modul => N_FLpwr
  )
  port map(
       clk => clk,
       count => countW1( N_FL-1 downto 0 ),
       en => en_WCNT1,
       rst_L => rst_WCNT1_L,
	   seu => seu_out_WORD_CNT1
  );

WORD_CNT2 : Counter_N_RHSEU
  generic map (
       N => N_FL,
       modul => N_FLpwr
  )
  port map(
       clk => clk,
       count => countW2( N_FL-1 downto 0 ),
       en => en_WCNT2,
       rst_L => rst_WCNT2_L,
	   seu => seu_out_WORD_CNT2
  );


---- Power , ground assignment ----

VCC <= VCC_CONSTANT;

---- Terminal assignment ----

    -- Output\buffer terminals
	HDR_cmd <= HDR_cmd_int;
	seu_out <= seu_out_DEL OR seu_out_DT_reg OR seu_out_FB_ctr1 OR seu_out_FB_ctr2 OR seu_out_FL_reg OR seu_out_LF_reg OR 
	   seu_out_LO_reg OR seu_out_WORD_CNT1 OR seu_out_WORD_CNT2;

end Frame_Builder1;
