
library IEEE;
use IEEE.std_logic_1164.all;
use work.fifo_pkg.all;


entity NEW_FIFO_32X18_CL is
	generic(
		N_WORD: positive := 32; 
		N_MUX_WORD: positive := 8; 
		N_WORD_ADDR: positive := 16; 
		N_BIT_COD: positive := 18; 
		N_BIT_DEC: positive := 12;
		N_ADDR: positive := 5;	  
		N_BIT_PAR: positive := 6 	 
	);
	port(
       clk : in STD_LOGIC;
       fifo_ded_en : in STD_LOGIC;
       fifo_test_en : in STD_LOGIC;
       rst_l : in STD_LOGIC;
       test_read : in STD_LOGIC;
       test_write : in STD_LOGIC;
       ded_16x_read : in STD_LOGIC;
       ded_16x_write : in STD_LOGIC;
       ded_4x_read : in STD_LOGIC;
       ded_4x_write : in STD_LOGIC;
       ded_8x_read : in STD_LOGIC;
       ded_8x_write : in STD_LOGIC;
       seu_16x_read : in STD_LOGIC;
       seu_16x_write : in STD_LOGIC;
       seu_4x_read : in STD_LOGIC;
       seu_4x_write : in STD_LOGIC;
       seu_8x_read : in STD_LOGIC;
       seu_8x_write : in STD_LOGIC;
       OUTB_11_8 : in STD_LOGIC_VECTOR(N_BIT_COD-1 downto 0);
       OUTB_15_12 : in STD_LOGIC_VECTOR(N_BIT_COD-1 downto 0);
       OUTB_19_16 : in STD_LOGIC_VECTOR(N_BIT_COD-1 downto 0);
       OUTB_23_20 : in STD_LOGIC_VECTOR(N_BIT_COD-1 downto 0);
       OUTB_27_24 : in STD_LOGIC_VECTOR(N_BIT_COD-1 downto 0);
       OUTB_31_28 : in STD_LOGIC_VECTOR(N_BIT_COD-1 downto 0);
       OUTB_3_0 : in STD_LOGIC_VECTOR(N_BIT_COD-1 downto 0);
       OUTB_7_3 : in STD_LOGIC_VECTOR(N_BIT_COD-1 downto 0);
       fifo_sel : in STD_LOGIC_VECTOR(1 downto 0);
       test_word_in : in STD_LOGIC_VECTOR(N_BIT_DEC-1 downto 0);
       ded_16x_word_in : in STD_LOGIC_VECTOR(N_BIT_DEC-1 downto 0);
       ded_4x_word_in : in STD_LOGIC_VECTOR(N_BIT_DEC-1 downto 0);
       ded_8x_word_in : in STD_LOGIC_VECTOR(N_BIT_DEC-1 downto 0);
       seu_16x_word_in : in STD_LOGIC_VECTOR(N_BIT_DEC-1 downto 0);
       seu_4x_word_in : in STD_LOGIC_VECTOR(N_BIT_DEC-1 downto 0);
       seu_8x_word_in : in STD_LOGIC_VECTOR(N_BIT_DEC-1 downto 0);
       deu : out STD_LOGIC;
       empty : out STD_LOGIC;
       full : out STD_LOGIC;
       seu : out STD_LOGIC;
       en_int : out STD_LOGIC_VECTOR(N_WORD-1 downto 0);
       word_addr : out STD_LOGIC_VECTOR(N_WORD_ADDR-1 downto 0);
       word_in_int : out STD_LOGIC_VECTOR(N_BIT_COD-1 downto 0);
       word_out : out STD_LOGIC_VECTOR(N_BIT_DEC-1 downto 0)
  );
end NEW_FIFO_32X18_CL;

architecture NEW_FIFO_32X18_CL of NEW_FIFO_32X18_CL is

---- Component declarations -----

component FIFO_12to18_Encoder
  generic(
      N_BIT_COD : positive ;
      N_BIT_DEC : positive ;
	  N_BIT_PAR : positive 
	  );
  port (
       FIFO_din : in STD_LOGIC_VECTOR(N_BIT_DEC-1 downto 0);
       FIFO_dout : out STD_LOGIC_VECTOR(N_BIT_COD-1 downto 0)
  		);
end component;

component FIFO_18to12_Decoder
  generic(
      N_BIT_COD : positive ;
      N_BIT_DEC : positive ;
	  N_BIT_PAR : positive 
	  ); 
  port (
       FIFO_din : in STD_LOGIC_VECTOR(N_BIT_COD-1 downto 0);
       FIFO_dout : out STD_LOGIC_VECTOR(N_BIT_DEC-1 downto 0);
       derr : out STD_LOGIC;
       serr : out STD_LOGIC
  );
end component;	

component FIFO_18to12_Decoder_noFlagBF
	generic(
      N_BIT_COD : positive ;
      N_BIT_DEC : positive ;
	  N_BIT_PAR : positive 
	  ); 
  port (
       FIFO_din : in STD_LOGIC_VECTOR(N_BIT_COD-1 downto 0);
       FIFO_dout : out STD_LOGIC_VECTOR(N_BIT_DEC-1 downto 0)
  );
end component;
component FIFO_CORE_32X18
	generic(
	 N_WORD: positive; 
	 N_MUX_WORD: positive; 
	 N_WORD_ADDR: positive ; 
	 N_BIT_COD: positive ; 
	 N_ADDR: positive 	  
	); 
  	port (
       clk : in STD_LOGIC;
       read_core : in STD_LOGIC;
       rst_l : in STD_LOGIC;
       scrub_addr : in STD_LOGIC_VECTOR(N_ADDR-1 downto 0);
       word_in : in STD_LOGIC_VECTOR(N_BIT_COD-1 downto 0);
       word_out_int : in fifo_array_type_TX_FD;
       write_core : in STD_LOGIC;
       write_scrub : in STD_LOGIC;
       empty : out STD_LOGIC;
       en_int : out STD_LOGIC_VECTOR(N_WORD-1 downto 0);
       full : out STD_LOGIC;
       r_pnt_out : out STD_LOGIC_VECTOR(N_ADDR-1 downto 0);
       w_pnt_out : out STD_LOGIC_VECTOR(N_ADDR-1 downto 0);
       word_addr : out STD_LOGIC_VECTOR(N_WORD_ADDR-1 downto 0);
       word_in_int : out STD_LOGIC_VECTOR(N_BIT_COD-1 downto 0);
       word_out : out STD_LOGIC_VECTOR(N_BIT_COD-1 downto 0);
       word_out_scrub : out STD_LOGIC_VECTOR(N_BIT_COD-1 downto 0)
  );
end component;

component FIFO_mux
  generic(
      N_BIT_DEC : positive 
	  );
  port (
       fifo_ded_en : in STD_LOGIC;
       fifo_test_en : in STD_LOGIC;
       fifo_sel : in STD_LOGIC_VECTOR(1 downto 0);
       test_read : in STD_LOGIC;
       test_word_in : in STD_LOGIC_VECTOR(N_BIT_DEC-1 downto 0);
       test_write : in STD_LOGIC;
       ded_16x_read : in STD_LOGIC;
       ded_16x_word_in : in STD_LOGIC_VECTOR(N_BIT_DEC-1 downto 0);
       ded_16x_write : in STD_LOGIC;
       ded_4x_read : in STD_LOGIC;
       ded_4x_word_in : in STD_LOGIC_VECTOR(N_BIT_DEC-1 downto 0);
       ded_4x_write : in STD_LOGIC;
       ded_8x_read : in STD_LOGIC;
       ded_8x_word_in : in STD_LOGIC_VECTOR(N_BIT_DEC-1 downto 0);
       ded_8x_write : in STD_LOGIC;
       seu_16x_read : in STD_LOGIC;
       seu_16x_word_in : in STD_LOGIC_VECTOR(N_BIT_DEC-1 downto 0);
       seu_16x_write : in STD_LOGIC;
       seu_4x_read : in STD_LOGIC;
       seu_4x_word_in : in STD_LOGIC_VECTOR(N_BIT_DEC-1 downto 0);
       seu_4x_write : in STD_LOGIC;
       seu_8x_read : in STD_LOGIC;
       seu_8x_word_in : in STD_LOGIC_VECTOR(N_BIT_DEC-1 downto 0);
       seu_8x_write : in STD_LOGIC;
       read : out STD_LOGIC;
       word_in : out STD_LOGIC_VECTOR(N_BIT_DEC-1 downto 0);
       write : out STD_LOGIC
  );
end component;
component SCRUB_CTR_32X18
	generic(
      N_BIT_DEC : positive ;
	  N_ADDR : positive ;
	  N_WORD : positive 
	  );
  port (
       clk : in STD_LOGIC;
       empty_fifo : in STD_LOGIC;
       r_fifo_pnt : in STD_LOGIC_VECTOR(N_ADDR-1 downto 0);
       read : in STD_LOGIC;
       rst_l : in STD_LOGIC;
       w_fifo_pnt : in STD_LOGIC_VECTOR(N_ADDR-1 downto 0);
       word_fifo_out : in STD_LOGIC_VECTOR(N_BIT_DEC-1 downto 0);
       word_in : in STD_LOGIC_VECTOR(N_BIT_DEC-1 downto 0);
       write : in STD_LOGIC;
       read_core : out STD_LOGIC;
       scrub_addr : out STD_LOGIC_VECTOR(N_ADDR-1 downto 0);
       word_fifo_in : out STD_LOGIC_VECTOR(N_BIT_DEC-1 downto 0);
       write_core : out STD_LOGIC;
       write_scrub : out STD_LOGIC
  );
end component;

component DEU_MAN
	generic(
		N_WORD : positive;
		N_ADDR : positive
		);
	port(	
		 rst_l : in STD_LOGIC;
		 clk : in STD_LOGIC;
		 write_scrub : in STD_LOGIC;   
		 scrub_addr : in STD_LOGIC_VECTOR(N_ADDR-1 downto 0);
		 deu_in: in STD_LOGIC;
		 deu_out : out STD_LOGIC	 	 	
	     );
end component;


---- Signal declarations used on the diagram ----

signal empty_int : STD_LOGIC;
signal NET10572 : STD_LOGIC;
signal NET14120 : STD_LOGIC;
signal NET4315 : STD_LOGIC;
signal read : STD_LOGIC;
signal write : STD_LOGIC;
signal write_scr : STD_LOGIC;
signal r_pnt_out : STD_LOGIC_VECTOR (4 downto 0);
signal scrub_addr : STD_LOGIC_VECTOR (N_ADDR-1 downto 0);
signal word_fifo_in : STD_LOGIC_VECTOR (N_BIT_DEC-1 downto 0);
signal word_in : STD_LOGIC_VECTOR (N_BIT_DEC-1 downto 0);
signal word_in_cod : STD_LOGIC_VECTOR (21 downto 0);
signal word_out_cod : STD_LOGIC_VECTOR (N_BIT_COD-1 downto 0);
signal word_out_int : fifo_array_type_TX_FD;
signal word_out_scrub : STD_LOGIC_VECTOR (N_BIT_DEC-1 downto 0);
signal word_out_scrub_cod : STD_LOGIC_VECTOR (N_BIT_COD-1 downto 0);
signal w_pnt_out : STD_LOGIC_VECTOR (4 downto 0);
signal seu_sig : STD_LOGIC;
signal deu_sig : STD_LOGIC;

begin

---- User Signal Assignments ----
word_out_int(0) <= OUTB_3_0(N_BIT_COD-1 downto 0);
word_out_int(1) <= OUTB_7_3(N_BIT_COD-1 downto 0);
word_out_int(2) <= OUTB_11_8(N_BIT_COD-1 downto 0);
word_out_int(3) <= OUTB_15_12(N_BIT_COD-1 downto 0);
word_out_int(4) <= OUTB_19_16(N_BIT_COD-1 downto 0);
word_out_int(5) <= OUTB_23_20(N_BIT_COD-1 downto 0);
word_out_int(6) <= OUTB_27_24(N_BIT_COD-1 downto 0);
word_out_int(7) <= OUTB_31_28(N_BIT_COD-1 downto 0);

----  Component instantiations  ----

U1 : FIFO_CORE_32X18
  generic map(
  	   N_WORD => N_WORD,
  	   N_MUX_WORD => N_MUX_WORD,
  	   N_WORD_ADDR =>  N_WORD_ADDR,
  	   N_BIT_COD => N_BIT_COD,
  	   N_ADDR => N_ADDR
	   )
  port map(
       clk => clk,
       empty => empty_int,								
       en_int => en_int( 31 downto 0 ),
       full => full,
       r_pnt_out => r_pnt_out( 4 downto 0 ),
       read_core => NET4315,
       rst_l => rst_l,
       scrub_addr => scrub_addr( N_ADDR-1 downto 0 ),
       w_pnt_out => w_pnt_out( 4 downto 0 ),
       word_addr => word_addr( N_WORD_ADDR-1 downto 0 ),
       word_in => word_in_cod( N_BIT_COD-1 downto 0 ),
       word_in_int => word_in_int( N_BIT_COD-1 downto 0 ),
       word_out => word_out_cod( N_BIT_COD-1 downto 0 ),
       word_out_int => word_out_int,
       word_out_scrub => word_out_scrub_cod( N_BIT_COD-1 downto 0 ),
       write_core => NET10572,
       write_scrub => write_scr
  );

U2 : FIFO_18to12_Decoder_noFlagBF
    generic map(
	   N_BIT_COD => N_BIT_COD,
	   N_BIT_DEC => N_BIT_DEC,
  	   N_BIT_PAR => N_BIT_PAR
	   )
    port map(
       FIFO_din => word_out_cod( N_BIT_COD-1 downto 0 ),
       FIFO_dout => word_out( N_BIT_DEC-1 downto 0 )
     	);

U3 : DEU_MAN
  generic map(
	   N_WORD => N_WORD,
	   N_ADDR => N_ADDR
	   )
   port map(
		rst_l =>  rst_l,
		clk   =>   clk,
		write_scrub => write_scr,
		scrub_addr =>  scrub_addr,
		deu_in =>  deu_sig,
		deu_out => deu
		);		 

U4 : FIFO_18to12_Decoder
 generic map(
	   N_BIT_COD => N_BIT_COD,
	   N_BIT_DEC => N_BIT_DEC,
  	   N_BIT_PAR => N_BIT_PAR
	   )
  port map(
       FIFO_din => word_out_scrub_cod( N_BIT_COD-1 downto 0 ),
       FIFO_dout => word_out_scrub( N_BIT_DEC-1 downto 0 ),
       derr => deu_sig,
       serr => seu_sig
  );

U5 : FIFO_12to18_Encoder
  generic map(
	   N_BIT_COD => N_BIT_COD,
	   N_BIT_DEC => N_BIT_DEC,
  	   N_BIT_PAR => N_BIT_PAR
	   )
  port map(
       FIFO_din => word_fifo_in( N_BIT_DEC-1 downto 0 ),
       FIFO_dout => word_in_cod( N_BIT_COD-1 downto 0 )
  );

U6 : SCRUB_CTR_32X18
 generic map(
       N_BIT_DEC => N_BIT_DEC,
	   N_ADDR =>  N_ADDR,
	   N_WORD =>  N_WORD
	  )
  port map(
       clk => clk,
       empty_fifo => empty_int,
       r_fifo_pnt => r_pnt_out( 4 downto 0 ),
       read => read,
       read_core => NET4315,
       rst_l => rst_l,
       scrub_addr => scrub_addr( N_ADDR-1 downto 0 ),
       w_fifo_pnt => w_pnt_out( 4 downto 0 ),
       word_fifo_in => word_fifo_in( N_BIT_DEC-1 downto 0 ),
       word_fifo_out => word_out_scrub( N_BIT_DEC-1 downto 0 ),
       word_in => word_in( N_BIT_DEC-1 downto 0 ),
       write => write,
       write_core => NET10572,
       write_scrub => write_scr
  );

U7 : FIFO_mux
  generic map(
      N_BIT_DEC => N_BIT_DEC  
	  )
  port map(
       fifo_ded_en => fifo_ded_en,
       fifo_test_en => fifo_test_en,
       fifo_sel => fifo_sel,
       read => read,
       test_read => test_read,
       test_word_in => test_word_in( N_BIT_DEC-1 downto 0 ),
       test_write => test_write,
       ded_16x_read => ded_16x_read,
       ded_16x_word_in => ded_16x_word_in( N_BIT_DEC-1 downto 0 ),
       ded_16x_write => ded_16x_write,
       ded_4x_read => ded_4x_read,
       ded_4x_word_in => ded_4x_word_in( N_BIT_DEC-1 downto 0 ),
       ded_4x_write => ded_4x_write,
       ded_8x_read => ded_8x_read,
       ded_8x_word_in => ded_8x_word_in( N_BIT_DEC-1 downto 0 ),
       ded_8x_write => ded_8x_write,
       seu_16x_read => seu_16x_read,
       seu_16x_word_in => seu_16x_word_in( N_BIT_DEC-1 downto 0 ),
       seu_16x_write => seu_16x_write,
       seu_4x_read => seu_4x_read,
       seu_4x_word_in => seu_4x_word_in( N_BIT_DEC-1 downto 0 ),
       seu_4x_write => seu_4x_write,
       seu_8x_read => seu_8x_read,
       seu_8x_word_in => seu_8x_word_in( N_BIT_DEC-1 downto 0 ),
       seu_8x_write => seu_8x_write,
       word_in => word_in( N_BIT_DEC-1 downto 0 ),
       write => write
  );


empty <= empty_int;
seu <= seu_sig and write_scr;


end NEW_FIFO_32X18_CL;
