
-------------------------------------------------------------------------------
--
-- Description : FF_RX RHSEU with external RX_buffer, external FD_FIFO and clk_enable
-- the deserializer is in the 8x_4b_6b version
-------------------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_misc.all;
use IEEE.numeric_std.all;
use work.interfaces_pkg.all;


entity RX_8x_4b_6b_RHSEU is
  generic(
       N_FRM : integer := 2; -- number of FRMbits in each clk40 cycle (=2 @4xF, =6 @8xF, =14 @16x_F)
       N_CYC : integer := 4; -- number of clk_tx cycles in each clk40 cycle (=2 @4xF, =6 @8xF, =14 @16x_F)
       N_CYC_b : integer := 2; -- = n of bits of the CYCLE counter: must be = or > than base 2 log of N_CYC
       FLF_param_b : INTEGER RANGE 6 TO 8 := 6 -- number of bits of FLF_param
  );
  port(
       DAT_in : in STD_LOGIC;
       FLF_on : in STD_LOGIC;
       Get_Data : in STD_LOGIC;
       clk_enable : in std_logic; -- enables clk_tx for one cycle on the rising edge of clk_ref
       clk_tx : in STD_LOGIC;
       rst_L : in std_logic;
       FLF_param : in STD_LOGIC_VECTOR(FLF_param_b-1 downto 0);
       FD_FIFO_empty : in std_logic;
       FD_FIFO_out : in STD_LOGIC_VECTOR (N_FD downto 0);
       write_RX_BUF : out STD_LOGIC;
       read_RX_BUF : out STD_LOGIC;
       Data_Type : out STD_LOGIC;
       Data_Valid : out STD_LOGIC;
       Frame_Lost : out STD_LOGIC;
       Label_On : out STD_LOGIC;
       Last_Frame : out STD_LOGIC;
       TRG_out : out STD_LOGIC;
       clk_out : out STD_LOGIC;
       FLF_data_out : out STD_LOGIC_VECTOR(N_FRM-1 downto 0);
       RX_BUF_in : out STD_LOGIC_VECTOR(N_WORD-1 downto 0);
       FD_FIFO_in : out STD_LOGIC_VECTOR (N_FD downto 0);
       write_FD_FIFO : out STD_LOGIC;
       read_FD_FIFO : out STD_LOGIC;
       sync   : out  std_logic; -- signals that synchronization has been obtained
       resync : out  std_logic;
       THS_seu : out std_logic;
       FD_SEU : out std_logic; 
	   FD_DEU : out std_logic;	
       seu_out : out STD_LOGIC       
  );
end RX_8x_4b_6b_RHSEU;

architecture RX_architecture1 of RX_8x_4b_6b_RHSEU is

---- Component declarations -----

component deserializer_4b_6b_8x_RHSEU
  generic(
       N_FRM : INTEGER
  );
  port (
       DAT_in : in STD_LOGIC;
       FLF_going : in STD_LOGIC;
       HDR : in STD_LOGIC;
       clk : in STD_LOGIC;
       rst_L : in STD_LOGIC;
       sel_THS_del : in STD_LOGIC;
       FDC_rcv : out STD_LOGIC;
       FLF_data_out : out STD_LOGIC_VECTOR(N_FRM-1 downto 0);
       THS_out : out STD_LOGIC_VECTOR(5 downto 0);
       sel_shr : out STD_LOGIC;
       word_out : out STD_LOGIC_VECTOR(N_WORD-1 downto 0);
       seu : out STD_LOGIC
  );
end component;
component frame_analyser_RHSEU
  port(
       FDC_rcv : in std_logic;
       Get_Data : in STD_LOGIC;
       -- transmission ("fast") clock
       clk : in std_logic;
       clk_enable : in std_logic;
       --------------------------
       rst_L : in std_logic;
       sel_shr : in std_logic;
       FD_FIFO_empty : in std_logic;
       word_in : in std_logic_vector(N_WORD-1 downto 0);
       FD_FIFO_out : in STD_LOGIC_VECTOR (N_FD downto 0);
       FD_FIFO_in : out STD_LOGIC_VECTOR (N_FD downto 0);
       Data_Type : out STD_LOGIC;
       Data_Valid : out STD_LOGIC;
       Frame_Lost : out STD_LOGIC;
       Label_On : out STD_LOGIC;
       Last_Frame : out STD_LOGIC;
       write_word : out STD_LOGIC;
       write_FD_FIFO : out STD_LOGIC;
       read_FD_FIFO : out STD_LOGIC;
       FD_SEU : out std_logic; 
	   FD_DEU : out std_logic;	 
       seu : out STD_LOGIC     
  );
end component;
component sel_ths_gen_RX_RHSEU
  generic(
       N_CYC : INTEGER;
       N_CYC_b : INTEGER
  );
  port (
       clk40_RX : in STD_LOGIC;
       clk_tx : in STD_LOGIC;
       THS_valid : out STD_LOGIC;
       sel_THS : out STD_LOGIC;
       sel_THS_del : out STD_LOGIC;
       seu : out STD_LOGIC
  );
end component;
component Synchronizer_8x_RHSEU
  generic(
       CNT_b : INTEGER;
       CNT_mod : INTEGER;
       N1 : INTEGER;
       N2 : INTEGER;
       N_CYC : INTEGER;
       N_CYC_b : INTEGER
  );
  port (
       THS_det : in STD_LOGIC;
       clk : in STD_LOGIC;
       rst_L : in STD_LOGIC;
       clk_out : out STD_LOGIC;
       sync : out STD_LOGIC;
       seu : out STD_LOGIC;
       resync : out STD_LOGIC
  );
end component;
component THS_Detector_RHSEU
  port (
       THS_in : in STD_LOGIC_VECTOR(N_THS-1 downto 0);
       THS_valid : in STD_LOGIC;
       clk : in STD_LOGIC;
       rst_L : in STD_LOGIC;
       sync : in STD_LOGIC;
       HDR : out STD_LOGIC;
       THS_det : out STD_LOGIC;
       TRG : out STD_LOGIC;
       THS_seu : out std_logic;
       seu : out STD_LOGIC
  );
end component;
component TRG_shrk_strch_8x_16x_RHSEU
  generic(
       N_wakeup_b : integer
  );
  port (
       FLF_on : in STD_LOGIC;
       N_wakeup : in STD_LOGIC_VECTOR(N_wakeup_b-1 downto 0);
       TRG_in : in STD_LOGIC;
       clk_enable : in std_logic;
       clk_tx : in STD_LOGIC;
       rst_L : in STD_LOGIC;
       FLF_going : out STD_LOGIC;
       TRG_out : out STD_LOGIC;
       seu : out STD_LOGIC
  );
end component;


component ffd_RHSEU
  port (
       CLR : in STD_LOGIC;
       D : in STD_LOGIC;
       clk : in STD_LOGIC;
       en : in STD_LOGIC;
       Q : out STD_LOGIC;
       QN : out STD_LOGIC;
       seu : out STD_LOGIC
  );
end component;

---- Signal declarations used on the diagram ----

signal clk40_rx : STD_LOGIC;
signal Data_Valid_int : STD_LOGIC;
signal FDC_rcv : STD_LOGIC;
signal FLF_going : STD_LOGIC;
signal Frame_Lost_int : STD_LOGIC;
signal HDR : STD_LOGIC;
signal sel_shr : STD_LOGIC;
signal sel_THS_del : STD_LOGIC;
signal sync_int : STD_LOGIC;
signal THS_det : STD_LOGIC;
signal THS_valid : STD_LOGIC;
signal TRG_out_1 : STD_LOGIC;
signal DES_word_out : STD_LOGIC_VECTOR (N_WORD-1 downto 0);
signal FLF_param_m : STD_LOGIC_VECTOR (FLF_param_b-1 downto 0);
signal THS_out : STD_LOGIC_VECTOR (N_THS-1 downto 0);
signal seu_DES, seu_FRM_AN, seu_SELTHS, seu_SYNC, seu_THSDET, seu_TRG_STR : STD_LOGIC;

signal resync_fast : STD_LOGIC;
signal seu_fast : STD_LOGIC;
signal ths_seu_fast : STD_LOGIC;
signal ff_seu0_out, ff_seu1_out, ff_resync0_out, ff_resync1_out, ff_ths_seu0_out, ff_ths_seu1_out : STD_LOGIC;
signal ff_seu0_rst_L, ff_resync0_rst_L, ff_ths_seu0_rst_L: STD_LOGIC;
signal seu_ff_seu0, seu_ff_seu1, seu_ff_resync0, seu_ff_resync1, seu_ff_ths_seu0, seu_ff_ths_seu1 : STD_LOGIC;


begin

---- User Signal Assignments ----
FLF_param_m <= FLF_param when FLF_on = '1' else CONV_STD_LOGIC_VECTOR(((3*N_CYC)-4),FLF_param_b);
-- if FLF are disabled, FLF_param must be set to the value it would have if Nc_FLF = 3
-- so that 3-clk-cycles-spaced triggers are correctly received

RX_BUF_in <= DES_word_out;

----  Component instantiations  ----

DES : deserializer_4b_6b_8x_RHSEU
  generic map (
       N_FRM => N_FRM
  )
  port map(
       DAT_in => DAT_in,
       FDC_rcv => FDC_rcv,
       FLF_data_out => FLF_data_out( N_FRM-1 downto 0 ),
       FLF_going => FLF_going,
       HDR => HDR,
       THS_out => THS_out( N_THS-1 downto 0 ),
       clk => clk_tx,
       rst_L => rst_L,
       sel_THS_del => sel_THS_del,
       sel_shr => sel_shr,
       word_out => DES_word_out( N_WORD-1 downto 0),
       seu => seu_DES
  );

FRM_AN : frame_analyser_RHSEU
  port map(
       Data_Type => Data_Type,
       Data_Valid => Data_Valid_int,
       FDC_rcv => FDC_rcv,
       Frame_Lost => Frame_Lost_int,
       Get_Data => Get_Data,
       Label_On => Label_On,
       Last_Frame => Last_Frame,
       clk => clk_tx,
       clk_enable => clk_enable,
       rst_L => rst_L,
       sel_shr => sel_shr,
       word_in => DES_word_out( N_WORD-1 downto 0 ),
       write_word => write_RX_BUF,     
       FD_FIFO_empty => FD_FIFO_empty,
       FD_FIFO_out => FD_FIFO_out,
       FD_FIFO_in => FD_FIFO_in,
       write_FD_FIFO => write_FD_FIFO,
       read_FD_FIFO => read_FD_FIFO,
       FD_SEU => FD_SEU,
       FD_DEU => FD_DEU,
       seu => seu_FRM_AN
  );


SELTHS : sel_ths_gen_RX_RHSEU
  generic map (
       N_CYC => N_CYC,
       N_CYC_b => N_CYC_b
  )
  port map(
       THS_valid => THS_valid,
       clk40_RX => clk40_rx,
       clk_tx => clk_tx,
       sel_THS_del => sel_THS_del,
       seu => seu_SELTHS
  );

SYNCH : Synchronizer_8x_RHSEU
  generic map (
       CNT_b => 3,
       CNT_mod => 8,
       N1 => 4,
       N2 => 7,
       N_CYC => N_CYC,
       N_CYC_b => N_CYC_b
  )
  port map(
       THS_det => THS_det,
       clk => clk_tx,
       clk_out => clk40_rx,
       rst_L => rst_L,
       sync => sync_int,
       resync => resync_fast,
       seu => seu_SYNC
  );

THSDET : THS_Detector_RHSEU
  port map(
       HDR => HDR,
       THS_det => THS_det,
       THS_in => THS_out( N_THS-1 downto 0 ),
       THS_valid => THS_valid,
       TRG => TRG_out_1,
       clk => clk_tx,
       rst_L => rst_L,
       sync => sync_int,
       THS_seu => THS_SEU_fast,
       seu => seu_THSDET
  );

TRG_STR : TRG_shrk_strch_8x_16x_RHSEU
  generic map (
       N_wakeup_b => FLF_param_b
  )
  port map(
       FLF_going => FLF_going,
       FLF_on => FLF_on,
       N_wakeup => FLF_param_m( FLF_param_b-1 downto 0 ),
       TRG_in => TRG_out_1,
       TRG_out => TRG_out,
       clk_enable => clk_enable,
       clk_tx => clk_tx,
       rst_L => rst_L,
       seu => seu_TRG_STR
  );

read_RX_Buf <= not(Frame_Lost_int) and Get_Data and Data_Valid_int;


---- Terminal assignment ----

    -- Output\buffer terminals
	Data_Valid <= Data_Valid_int;
	Frame_Lost <= Frame_Lost_int;
	clk_out <= clk40_rx;

    seu_fast <= seu_DES or seu_FRM_AN or seu_SELTHS or seu_SYNC or seu_THSDET or seu_TRG_STR;

-- extension of seu signal

FF_seu0 : ffd_RHSEU
  port map(
       CLR => ff_seu0_rst_L,
       D => '1',
       Q => ff_seu0_out,
       clk => clk_tx,
       en => seu_fast,
       seu => seu_FF_seu0
  );
  
ff_seu0_rst_L <= rst_L and not(ff_seu1_out);

FF_seu1 : ffd_RHSEU
  port map(
       CLR => rst_L,
       D => ff_seu0_out,
       Q => ff_seu1_out,
       clk => clk_tx,
       en => clk_enable,
       seu => seu_FF_seu1
  );

seu_out <= ff_seu1_out or seu_FF_seu1 or seu_FF_resync1 or seu_FF_ths_seu1;

-- extension of resync signal

FF_resync0 : ffd_RHSEU
  port map(
       CLR => ff_resync0_rst_L,
       D => '1',
       Q => ff_resync0_out,
       clk => clk_tx,
       en => resync_fast,
       seu => seu_FF_resync0
  );
  
ff_resync0_rst_L <= rst_L and not(ff_resync1_out);

FF_resync1 : ffd_RHSEU
  port map(
       CLR => rst_L,
       D => ff_resync0_out,
       Q => ff_resync1_out,
       clk => clk_tx,
       en => clk_enable,
       seu => seu_FF_resync1
  );

resync <= ff_resync1_out;
    
-- extension of THS_seu signal

FF_ths_seu0 : ffd_RHSEU
  port map(
       CLR => ff_ths_seu0_rst_L,
       D => '1',
       Q => ff_ths_seu0_out,
       clk => clk_tx,
       en => THS_seu_fast,
       seu => seu_FF_ths_seu0
  );
  
ff_ths_seu0_rst_L <= rst_L and not(ff_ths_seu1_out);

FF_ths_seu1 : ffd_RHSEU
  port map(
       CLR => rst_L,
       D => ff_ths_seu0_out,
       Q => ff_ths_seu1_out,
       clk => clk_tx,
       en => clk_enable,
       seu => seu_FF_ths_seu1
  );

ths_seu <= ff_ths_seu1_out;
    
       
sync <= sync_int;    
    
end RX_architecture1;
