
library IEEE;
use IEEE.STD_LOGIC_1164.all; 
USE IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

USE work.fifo_pkg.all;

-- Entity declarations --

entity SCRUB_CTR_32x22_EN is		 
	generic(
      N_BIT_DEC : positive ;
	  N_ADDR : positive ;
	  N_WORD : positive 
	  );   	
	port(	
		 
	word_in : in STD_LOGIC_VECTOR(N_BIT_DEC-1 downto 0); 
	read : in STD_LOGIC; 
	write : in STD_LOGIC;
	word_fifo_out : in STD_LOGIC_VECTOR(N_BIT_DEC-1 downto 0);	
	rst_l : in STD_LOGIC; 
	clk : in STD_LOGIC;
	w_fifo_pnt : in STD_LOGIC_VECTOR(N_ADDR-1 downto 0); 
	r_fifo_pnt : in STD_LOGIC_VECTOR(N_ADDR-1 downto 0);	
	empty_fifo : in STD_LOGIC;
	
	word_fifo_in : out STD_LOGIC_VECTOR(N_BIT_DEC-1 downto 0);
	scrub_addr : out STD_LOGIC_VECTOR(N_ADDR-1 downto 0); 
	write_scrub : out STD_LOGIC;
	write_core : out STD_LOGIC;
	read_core : out	STD_LOGIC;
	
	enable: in std_logic
	     );
end SCRUB_CTR_32x22_EN;

-- Architecture -- 

architecture SCRUB_CTR_32x22_EN of SCRUB_CTR_32x22_EN is
	 
	signal scrub_cnt : INTEGER range 0 to N_WORD-1; -- scrubbing pointer
	signal write_scrub_sig : STD_LOGIC;
	  		
begin	
	
process (clk)
begin
	if rising_edge(clk) then
		if rst_l = '0' then
			scrub_cnt <= 0;
			write_scrub_sig <= '0';
		elsif (enable = '1') then	  --en
			if write = '0' and empty_fifo /= '1' then    --en
						if (conv_integer(w_fifo_pnt) > conv_integer(r_fifo_pnt)) and (conv_integer(w_fifo_pnt) <= conv_integer(r_fifo_pnt) + 4) then
							write_scrub_sig <= '0';		
						elsif (conv_integer(w_fifo_pnt) > conv_integer(r_fifo_pnt)) and (conv_integer(w_fifo_pnt) > conv_integer(r_fifo_pnt) + 4) then
							if scrub_cnt > conv_integer(r_fifo_pnt) + 3  then
								if scrub_cnt < (conv_integer(w_fifo_pnt)-1) then --start
									write_scrub_sig <= '1';
								    scrub_cnt <= (scrub_cnt + 1);
								else 							  	
									if read = '0' then
									write_scrub_sig <= '1';
									scrub_cnt <= conv_integer(r_fifo_pnt) + 4;  
									else
										if conv_integer(r_fifo_pnt) < (N_WORD - 6) then
										write_scrub_sig <= '1';
									    scrub_cnt <= conv_integer(r_fifo_pnt) + 5;
										else
										write_scrub_sig <= '0';
										end if;
									end if;
								end if; 		 					   --stop
							else 
								if read = '0' then
									write_scrub_sig <= '1';
									scrub_cnt <= conv_integer(r_fifo_pnt) + 4;  
								else
									if conv_integer(r_fifo_pnt) < (N_WORD - 6) then
										write_scrub_sig <= '1';
									    scrub_cnt <= conv_integer(r_fifo_pnt) + 5;
									else
										write_scrub_sig <= '0';
									end if;
								end if;	  
							end if;		 
						elsif (conv_integer(w_fifo_pnt) < conv_integer(r_fifo_pnt)) and (conv_integer(w_fifo_pnt) >= conv_integer(r_fifo_pnt) - 4) then 
							if scrub_cnt > conv_integer(r_fifo_pnt) + 3  then 
								if scrub_cnt /= N_WORD-1 then
									write_scrub_sig <= '1';
									scrub_cnt <= scrub_cnt + 1;
								else 
									if conv_integer(w_fifo_pnt) = 0 then
										if read = '0' then
											write_scrub_sig <= '1';
											scrub_cnt <= conv_integer(r_fifo_pnt) + 4;  
										else
											write_scrub_sig <= '1';
									    	scrub_cnt <= conv_integer(r_fifo_pnt) + 5;
										end if;
									end if;	
								end if;
							elsif (scrub_cnt >= conv_integer(w_fifo_pnt)) and (scrub_cnt <= (conv_integer(r_fifo_pnt) + 3 )) then
								if (conv_integer(r_fifo_pnt) <= (N_WORD - 6 )) then
										if read = '0' then
											write_scrub_sig <= '1';
											scrub_cnt <= conv_integer(r_fifo_pnt) + 4;  
										else
											write_scrub_sig <= '1';
									    	scrub_cnt <= conv_integer(r_fifo_pnt) + 5;
										end if;								
								else 
										write_scrub_sig <= '1';
									    scrub_cnt <= 0;
								end if;		
							elsif (scrub_cnt < conv_integer(w_fifo_pnt)) then
								 if scrub_cnt /= (conv_integer(w_fifo_pnt)-1) and scrub_cnt < conv_integer(r_fifo_pnt) - 4 then
									write_scrub_sig <= '1';
									scrub_cnt <= scrub_cnt + 1;
								elsif scrub_cnt = (conv_integer(w_fifo_pnt)-1) then
									if conv_integer(r_fifo_pnt) <= N_WORD - 6 then
										if read = '0' then
											write_scrub_sig <= '1';
											scrub_cnt <= conv_integer(r_fifo_pnt) + 4;  
										else
											write_scrub_sig <= '1';
									    	scrub_cnt <= conv_integer(r_fifo_pnt) + 5;
										end if;
									else 
										write_scrub_sig <= '1';
										scrub_cnt <= 0;
									end if;
								end if; 
							 end if;    
						elsif (conv_integer(w_fifo_pnt) < conv_integer(r_fifo_pnt)) and (conv_integer(w_fifo_pnt) < conv_integer(r_fifo_pnt) - 4) then
							if scrub_cnt > conv_integer(r_fifo_pnt) + 3  then 
								if scrub_cnt /= N_WORD-1 then
									write_scrub_sig <= '1';
									scrub_cnt <= scrub_cnt + 1;
								else 
									if conv_integer(w_fifo_pnt) /= 0 then
										write_scrub_sig <= '1';
										scrub_cnt <= 0;
									else 
										if conv_integer(r_fifo_pnt) <= N_WORD - 6 then
											if read = '0' then
												write_scrub_sig <= '1';
												scrub_cnt <= conv_integer(r_fifo_pnt) + 4;  
											else
												write_scrub_sig <= '1';
									    		scrub_cnt <= conv_integer(r_fifo_pnt) + 5;
											end if;
										else 
											write_scrub_sig <= '0';
										end if;
									end if;
								end if;					 
							elsif (scrub_cnt >= conv_integer(w_fifo_pnt)) and scrub_cnt <= (conv_integer(r_fifo_pnt)+3)  then
								if conv_integer(r_fifo_pnt) <= N_WORD - 6 then
									if read = '0' then
										write_scrub_sig <= '1';
										scrub_cnt <= conv_integer(r_fifo_pnt) + 4;  
									else
										write_scrub_sig <= '1';
									    scrub_cnt <= conv_integer(r_fifo_pnt) + 5;
									end if;
								else
									if conv_integer(w_fifo_pnt) /= 0 then
										write_scrub_sig <= '1';
										scrub_cnt <= 0;
									else 
										write_scrub_sig <= '0';
									end if;	
								end if;
							elsif (scrub_cnt < conv_integer(w_fifo_pnt)) then 		  
								if scrub_cnt /= (conv_integer(w_fifo_pnt)-1) then 	-------
									if conv_integer(r_fifo_pnt) = N_WORD-1 and read = '1' then
											if conv_integer(w_fifo_pnt) <= 4 then	
												write_scrub_sig <= '0';
											else 
												write_scrub_sig <= '1';
												scrub_cnt <= 4;
											end if;
									else 
										write_scrub_sig <= '1';
										scrub_cnt <= scrub_cnt + 1;
									end if;			                        ----------
								else 
									if conv_integer(r_fifo_pnt) <= N_WORD - 6 then
										if read = '0' then
											write_scrub_sig <= '1';
											scrub_cnt <= conv_integer(r_fifo_pnt) + 4;  
										else
											write_scrub_sig <= '1';
									    	scrub_cnt <= conv_integer(r_fifo_pnt) + 5;
										end if;
									else 
										write_scrub_sig <= '1';
										scrub_cnt <= 0;
									end if;
								end if;
							end if;		
										
						elsif (conv_integer(w_fifo_pnt) = conv_integer(r_fifo_pnt)) then
							if empty_fifo = '1'	then
								write_scrub_sig <= '0';	 
							else 
								write_scrub_sig <= '1';	
								if scrub_cnt > conv_integer(r_fifo_pnt) + 3  then
									if scrub_cnt /= N_WORD-1 then
										scrub_cnt <= scrub_cnt + 1;
									else 
										if conv_integer(r_fifo_pnt) <= 3 then
											scrub_cnt <= 4;
										else 
											scrub_cnt <= 0;
										end if;
									end if;	   
								elsif scrub_cnt <= conv_integer(r_fifo_pnt) + 3 and scrub_cnt >= conv_integer(r_fifo_pnt) - 3  then 
									if conv_integer(r_fifo_pnt) < N_WORD - 6  then
										 if read = '0' then
											write_scrub_sig <= '1';
											scrub_cnt <= conv_integer(r_fifo_pnt) + 4;  
										else
											write_scrub_sig <= '1';
									    	scrub_cnt <= conv_integer(r_fifo_pnt) + 5;
										end if;
									else 
										write_scrub_sig <= '1';
										scrub_cnt <= 0;
									end if;
								elsif scrub_cnt < conv_integer(r_fifo_pnt) - 3 then

									if scrub_cnt /= (conv_integer(r_fifo_pnt)-4) then
										write_scrub_sig <= '1';
										scrub_cnt <= scrub_cnt + 1;
									else 
										if conv_integer(r_fifo_pnt) <= N_WORD - 6 then
											if read = '0' then
												write_scrub_sig <= '1';
												scrub_cnt <= conv_integer(r_fifo_pnt) + 4;  
											else
												write_scrub_sig <= '1';
									    		scrub_cnt <= conv_integer(r_fifo_pnt) + 5;
											end if;
										else 
											write_scrub_sig <= '1';
											scrub_cnt <= 0;
										end if;
									 end if;	--
								 end if;--
							end if;	
						end if;	
					else 		
						write_scrub_sig <= '0';		
				end if;
		 end if;
	end if;
end process;    	  
			

	word_fifo_in <= word_fifo_out when write_scrub_sig = '1' and write = '0' else
				word_in;	
	write_core <= write;
	read_core <= read;
	write_scrub <= write_scrub_sig;
	scrub_addr <= CONV_STD_LOGIC_VECTOR(scrub_cnt,N_ADDR); 		
	
end SCRUB_CTR_32x22_EN;
