
library ieee;
use work.interfaces_pkg.all;
use ieee.std_logic_1164.all;
use IEEE.std_logic_arith.all;

entity Ser_control_RHSEU is 
	port (
		clk: in STD_LOGIC;
		countB: in STD_LOGIC_VECTOR (N_WORD_b-1 downto 0);
		HDR_cmd: in STD_LOGIC;
		rst_L: in STD_LOGIC;
		stop: in STD_LOGIC;
		rst_BCNT_L: out STD_LOGIC;
		sel_shr: out STD_LOGIC;
		seu : out STD_LOGIC
		);
end Ser_control_RHSEU;

architecture Ser_control_1 of Ser_control_RHSEU is

signal FDC_END: BOOLEAN;
signal W_END: BOOLEAN;
type SER_CTRL_Sreg0_type is (
    WORD0, WORD1, FDC0, IDLE
);
signal next_state, current_state_a, current_state_b, current_state_c, current_state_corr: SER_CTRL_Sreg0_type ;
signal status :  STD_LOGIC_VECTOR(2 downto 0);
signal sel_shr_int : STD_LOGIC;
signal sel_shr_sig:  STD_LOGIC_VECTOR(0 to 2);
signal corr_data_sel_shr : STD_LOGIC;
signal a_eq_b: std_logic;
signal b_eq_c : std_logic; 
signal a_eq_c : std_logic;	
signal seu_states : std_logic;	
signal seu_sel_shr : std_logic;	

begin

-- concurrent signals assignments

FDC_END <= (countB = CONV_STD_LOGIC_VECTOR(N_FDC-1, N_WORD_b));
W_END <= (countB = CONV_STD_LOGIC_VECTOR(N_WORD-1, N_WORD_b));

----------------------------------------------------------------------
-- Machine: SER_CTRL_Sreg0
----------------------------------------------------------------------
fsm_state: process (clk)
begin
	if clk'event and clk = '1' then
		if rst_L = '0' then	
			current_state_a <= IDLE;
			current_state_b <= IDLE;
			current_state_c <= IDLE;
		
		for i in 0 to 2 loop
			sel_shr_sig(i) <= '0';
		end loop; 
		
		else	 
			current_state_a <= next_state;
			current_state_b <= next_state;
			current_state_c <= next_state;
			
		for i in 0 to 2 loop
			sel_shr_sig(i) <= sel_shr_int;
		end loop;
			
		end if;
	end if;
end process; 

fsm_comb_logic: process (current_state_corr, W_END, stop, HDR_cmd, FDC_END )
begin
			case  current_state_corr is
				when WORD0 =>
					if NOT(W_END) then	
						next_state <= WORD0;
						sel_shr_int <= '0';
					elsif (stop = '1') AND (HDR_cmd = '0') then	
						next_state <= IDLE;
						sel_shr_int <= '0';
					elsif (HDR_cmd = '1') then	
						next_state <= FDC0;
						sel_shr_int <= '0';
					else	
						next_state <= WORD1;
						sel_shr_int <= '1';
					end if;
				when WORD1 =>
					if (W_END) AND (stop = '0') AND (HDR_cmd = '0') then	
						next_state <= WORD0;
						sel_shr_int <= '0';
					elsif NOT(W_END) then	
						next_state <= WORD1;
						sel_shr_int <= '1';
					elsif (HDR_cmd = '1') then	
						next_state <= FDC0;
						sel_shr_int <= '0';
					else
						next_state <= IDLE;
						sel_shr_int <= '0';
					end if;
				when FDC0 =>
					if FDC_END then	
						next_state <= WORD1;
						sel_shr_int <= '1';
					else	
						next_state <= FDC0;
						sel_shr_int <= '0';
					end if;
				when IDLE =>
					if HDR_cmd = '0' then	
						next_state <= IDLE;
						sel_shr_int <= '0';
					else	
						next_state <= FDC0;
						sel_shr_int <= '0';
					end if;
				when others =>
					null;
			end case;
end process;

  process (current_state_a, current_state_b, current_state_c)
	begin
        
		if (current_state_a = current_state_b) then
			a_eq_b <= '1';
		else
			a_eq_b <= '0';
		end if;
			
		if (current_state_a = current_state_c) then
			a_eq_c <= '1';
		else
			a_eq_c <= '0';
		end if;
				
		if (current_state_b = current_state_c) then
			b_eq_c <= '1';
		else
			b_eq_c <= '0';
		end if;
  end process;
  
  status <= a_eq_b & a_eq_c & b_eq_c;
  
  process (status, current_state_a, current_state_b)
    begin
		case (status) is 
			when "111" =>     			--All equal
				current_state_corr <= current_state_a; 
				seu_states <= '0'; 
			when "100" =>    			--c different
				current_state_corr <= current_state_a; 
				seu_states <= '1'; 
			when "010" => 				--b different
				current_state_corr <= current_state_a;
				seu_states <= '1'; 
			when "001" => 				--a different
				current_state_corr <= current_state_b;
				seu_states <= '1'; 
			when others => 			  -- errors in majority logic	
				current_state_corr <= IDLE;
				seu_states <= '1'; 
		end case;
	end process;   

majority_logic_sel_shr: process (sel_shr_sig)
	begin
		case (sel_shr_sig) is

			-- data = 0 - NO SEU
			when "000" => 	corr_data_sel_shr <= '0';  
							seu_sel_shr <= '0';	
			
			-- data = 1 - NO SEU
			when "111" => 	corr_data_sel_shr <= '1';  
							seu_sel_shr <= '0';
			
			-- data = 0 - SEU
			when "100" => 	corr_data_sel_shr <= '0';
							seu_sel_shr <= '1';	
			when "010" =>	corr_data_sel_shr <= '0';
							seu_sel_shr <= '1';
			when "001" => 	corr_data_sel_shr <= '0';
							seu_sel_shr <= '1';
                  
			-- data = 1 - SEU            
			when "110" => 	corr_data_sel_shr <= '1';
							seu_sel_shr <= '1';
			when "101" => 	corr_data_sel_shr <= '1';
							seu_sel_shr <= '1';
			when "011" => 	corr_data_sel_shr <= '1'; 
							seu_sel_shr <= '1';	
			
			when others => 	corr_data_sel_shr <= '0';
							seu_sel_shr <= '0';
		end case;	                                     
	end process;

-- signal assignment statements for combinatorial outputs
rst_BCNT_L_assignment:
rst_BCNT_L <= '0' when (current_state_corr = IDLE and HDR_cmd = '0') else
              '0' when (current_state_corr = IDLE and (HDR_cmd = '1' and not (HDR_cmd = '0'))) else
              '0' when (current_state_corr = IDLE) else
              '1' when (current_state_corr = WORD0 and NOT(W_END)) else
              '0' when (current_state_corr = WORD0 and ((W_END) AND (stop = '1') AND (HDR_cmd = '0') and not (NOT(W_END)))) else
              '0' when (current_state_corr = WORD0 and ((W_END) AND (HDR_cmd = '1') and not (NOT(W_END)) and not ((W_END) AND (stop = '1') AND (HDR_cmd = '0')))) else
              '0' when (current_state_corr = WORD0 and ((W_END) AND (stop = '0') AND (HDR_cmd = '0') and not (NOT(W_END)) and not ((W_END) AND (stop = '1') AND (HDR_cmd = '0')) and not ((W_END) AND (HDR_cmd = '1')))) else
              '0' when (current_state_corr = WORD1 and ((W_END) AND (stop = '0') AND (HDR_cmd = '0'))) else
              '1' when (current_state_corr = WORD1 and (NOT(W_END) and not ((W_END) AND (stop = '0') AND (HDR_cmd = '0')))) else
              '0' when (current_state_corr = WORD1 and ((W_END) AND (HDR_cmd = '1') and not ((W_END) AND (stop = '0') AND (HDR_cmd = '0')) and not (NOT(W_END)))) else
              '0' when (current_state_corr = WORD1 and ((W_END) AND (stop = '1') AND (HDR_cmd = '0') and not ((W_END) AND (stop = '0') AND (HDR_cmd = '0')) and not (NOT(W_END)) and not ((W_END) AND (HDR_cmd = '1')))) else
              '0' when (current_state_corr = FDC0 and FDC_END) else
              '1' when (current_state_corr = FDC0 and (NOT(FDC_END) and not (FDC_END))) else
              '0';
			  
seu <= seu_states OR seu_sel_shr;
sel_shr <= corr_data_sel_shr;

end Ser_control_1;
