
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_misc.all;
use IEEE.numeric_std.all;
use work.interfaces_pkg.all;


entity Sync_Counter_RHSEU is
  generic(
       CNT_mod : integer; -- modulus of Counter (must be > N2)
       CNT_b : integer; -- number of bits of Counter
       N1 : integer; -- first threshold
       N2 : integer -- second threshold
  );
  port(
       CNT_en : in STD_LOGIC;
       clk : in STD_LOGIC;
       rst_CNT_L : in STD_LOGIC;
       rst_FF_L : in STD_LOGIC;
       rst_L : in STD_LOGIC;
       clr_all_CNT_L : out STD_LOGIC;
       reset_sync : out STD_LOGIC;
       sel : out STD_LOGIC;
       set_sync : out STD_LOGIC;
       count_out : out STD_LOGIC_VECTOR(CNT_b-1 downto 0);
       seu : out STD_LOGIC
  );
end Sync_Counter_RHSEU;

architecture Sync_Counter1 of Sync_Counter_RHSEU is

---- Component declarations -----

component Counter_N_RHSEU
  generic(
       N : INTEGER;
       modul : INTEGER
  );
  port (
       clk : in STD_LOGIC;
       en : in STD_LOGIC;
       rst_L : in STD_LOGIC;
       count : out STD_LOGIC_VECTOR(N-1 downto 0);
       seu : out STD_LOGIC
  );
end component;
component ffd_RHSEU
  port (
       CLR : in STD_LOGIC;
       D : in STD_LOGIC;
       clk : in STD_LOGIC;
       en : in STD_LOGIC;
       Q : out STD_LOGIC;
       QN : out STD_LOGIC;
       seu : out STD_LOGIC
  );
end component;

----     Constants     -----
constant VCC_CONSTANT   : STD_LOGIC := '1';

---- Signal declarations used on the diagram ----

signal CNT_rst_L : STD_LOGIC;
signal FF_rst_L : STD_LOGIC;
signal N1_reach : STD_LOGIC;
signal N2_reach : STD_LOGIC;
signal NET1187 : STD_LOGIC;
signal NET1300 : STD_LOGIC;
signal sel_int : STD_LOGIC;
signal VCC : STD_LOGIC;
signal count : STD_LOGIC_VECTOR (CNT_b-1 downto 0);
signal seu_CNT, seu_FF : STD_LOGIC;

begin

---- User Signal Assignments ----
N1_reach <= '1' when count = CONV_STD_LOGIC_VECTOR(N1, CNT_b) else '0';
N2_reach <= '1' when count = CONV_STD_LOGIC_VECTOR(N2, CNT_b) else '0';

----  Component instantiations  ----

CNT : Counter_N_RHSEU
  generic map (
       N => CNT_b,
       modul => CNT_mod
  )
  port map(
       clk => clk,
       count => count( CNT_b-1 downto 0 ),
       en => CNT_en,
       rst_L => CNT_rst_L,
       seu => seu_CNT
  );

FF : ffd_RHSEU
  port map(
       CLR => FF_rst_L,
       D => VCC,
       Q => sel_int,
       clk => clk,
       en => N2_reach,
       seu => seu_FF
  );

clr_all_CNT_L <= not(sel_int and CNT_en);

NET1187 <= rst_FF_L or N2_reach;

reset_sync <= not(sel_int) and N1_reach;

FF_rst_L <= rst_L and NET1187;

CNT_rst_L <= NET1300 and rst_L;

NET1300 <= rst_CNT_L or sel_int;


---- Power , ground assignment ----

VCC <= VCC_CONSTANT;

---- Terminal assignment ----

    -- Output\buffer terminals
	count_out <= count( CNT_b-1 downto 0 );
	sel <= sel_int;
	set_sync <= N2_reach;

    seu <= seu_CNT or seu_FF;

end Sync_Counter1;
