
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_misc.all;
use IEEE.numeric_std.all;
use work.interfaces_pkg.all;


entity TRG_shrk_strch_8x_16x_RHSEU is
  generic(
       N_wakeup_b : integer -- = number of bits of N_wakeup
  );
  port(
       FLF_on : in STD_LOGIC;
       TRG_in : in STD_LOGIC;
       clk_enable : in std_logic;
       clk_tx : in STD_LOGIC;
       rst_L : in STD_LOGIC;
       N_wakeup : in STD_LOGIC_VECTOR(N_wakeup_b-1 downto 0);
       FLF_going : out STD_LOGIC;
       TRG_out : out STD_LOGIC;
       seu : out STD_LOGIC
  );
end TRG_shrk_strch_8x_16x_RHSEU;

architecture TRG_shrk_strch_8x_16x_1 of TRG_shrk_strch_8x_16x_RHSEU is

---- Component declarations -----

component Counter_N_RHSEU
  generic(
       N : INTEGER;
       modul : INTEGER
  );
  port (
       clk : in STD_LOGIC;
       en : in STD_LOGIC;
       rst_L : in STD_LOGIC;
       count : out STD_LOGIC_VECTOR(N-1 downto 0);
       seu : out STD_LOGIC
  );
end component;
component ffd_RHSEU
  port (
       CLR : in STD_LOGIC;
       D : in STD_LOGIC;
       clk : in STD_LOGIC;
       en : in STD_LOGIC;
       Q : out STD_LOGIC;
       QN : out STD_LOGIC;
       seu : out STD_LOGIC
  );
end component;

----     Constants     -----
constant VCC_CONSTANT   : STD_LOGIC := '1';

---- Signal declarations used on the diagram ----

signal clr_L : STD_LOGIC;
signal finish : STD_LOGIC;
signal FLF_going1 : STD_LOGIC;
signal FLF_going_int : STD_LOGIC;
signal TRG1 : STD_LOGIC;
signal TRG2 : STD_LOGIC;
signal TRG_FLF : STD_LOGIC;
signal TRG_strch1 : STD_LOGIC;
signal TRG_strch2 : STD_LOGIC;
signal TRG_VLF : STD_LOGIC;
signal VCC : STD_LOGIC;
signal count : STD_LOGIC_VECTOR (N_wakeup_b-1 downto 0);
signal seu_CNT, seu_FF1, seu_FF2, seu_FF3, seu_FF4, seu_FF5: STD_LOGIC;

begin

---- User Signal Assignments ----
finish <= '0' when (count = N_wakeup) else '1';

----  Component instantiations  ----

CNT : Counter_N_RHSEU
  generic map (
       N => N_wakeup_b,
       modul => 0
  )
  port map(
       clk => clk_tx,
       count => count( N_wakeup_b-1 downto 0 ),
       en => TRG_strch1,
       rst_L => clr_L,
       seu => seu_CNT
  );

FF1 : ffd_RHSEU
  port map(
       CLR => clr_L,
       D => VCC,
       Q => TRG_strch1,
       clk => clk_tx,
       en => TRG_in,
       seu => seu_FF1
  );

FF2 : ffd_RHSEU
  port map(
       CLR => rst_L,
       D => FLF_going1,
       Q => FLF_going_int,
       clk => clk_tx,
       en => VCC,
       seu => seu_FF2
  );

FF3 : ffd_RHSEU
  port map(
       CLR => TRG_strch1,
       D => TRG_strch1,
       Q => TRG1,
       clk => clk_tx,
       en => clk_enable,
       seu => seu_FF3
  );

FF4 : ffd_RHSEU
  port map(
       CLR => TRG_strch1,
       D => TRG1,
       Q => TRG2,
       clk => clk_tx,
       en => clk_enable,
       seu => seu_FF4
  );

FF5 : ffd_RHSEU
  port map(
       CLR => rst_L,
       D => FLF_going_int,
       Q => TRG_FLF,
       clk => clk_tx,
       en => clk_enable,
       seu => seu_FF5
  );

clr_L <= finish and rst_L;

TRG_VLF <= not(TRG2) and TRG1;

TRG_strch2 <= TRG_strch1 or TRG_in;

FLF_going1 <= TRG_strch2 and FLF_on;

TRG_out <= (TRG_VLF and not FLF_on) or (TRG_FLF and FLF_on);


---- Power , ground assignment ----

VCC <= VCC_CONSTANT;

---- Terminal assignment ----

    -- Output\buffer terminals
	FLF_going <= FLF_going_int;

    seu <= seu_CNT or seu_FF1 or seu_FF2 or seu_FF3 or seu_FF4 or seu_FF5;

end TRG_shrk_strch_8x_16x_1;
