-------------------------------------------------------------------------------
--
-- Description : FF_TX with external TX_buffer, external FDC_FIFO and internal FLF_buffer 
-- the serializer is in the 8x_4b_6b version
-------------------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_misc.all;
use IEEE.numeric_std.all;
use work.interfaces_pkg.all;

 

entity TX_8x_4b_6b_RHSEU is
  generic(
       N_FRM : integer := 2; -- number of FRMbits in each clk40 cycle (=2 @160Mbps, =6 @320Mbps, =14 @640Mbps)
       N_CYC : integer := 4; -- number of clk_tx cycles in each clk40 cycle (=4 @160Mbps, =5 @320Mbps, =16 @640Mbps)
       N_CYC_b : integer := 2; -- = must be equal or greater than base 2 log of N_CYC (it's the number of bits of the CYCLE counter)
       FLF_param_b : INTEGER RANGE 1 TO 4 := 4 -- number of bits of FLF_param
  );
  port(
       Data_Type : in STD_LOGIC;
       Data_Valid : in STD_LOGIC;
       FLF_on : in STD_LOGIC;
       Label_On : in STD_LOGIC;
       TRG : in STD_LOGIC;
       TX_BUF_full : in STD_LOGIC;
       TX_BUF_out : in STD_LOGIC_VECTOR(N_WORD-1 downto 0);
       read_TX_Buf : out STD_LOGIC;
       clk40 : in std_logic;
       clk_tx : in STD_LOGIC;
       rst_L : in std_logic;
       FDC_FIFO_empty : in STD_LOGIC;
       FDC_FIFO_full : in STD_LOGIC;
       read_FDC_FIFO : out STD_LOGIC;
       write_FDC_FIFO : out STD_LOGIC;
       FDC_FIFO_out : in STD_LOGIC_VECTOR(N_FDC-1 downto 0);
       FLF_data_in : in STD_LOGIC_VECTOR(N_FRM-1 downto 0);
       FLF_param : in STD_LOGIC_VECTOR(FLF_param_b-1 downto 0);
       DAT_out : out STD_LOGIC;
       Get_Data : out STD_LOGIC;
       FDC_FIFO_in : out STD_LOGIC_VECTOR (N_FDC-1 downto 0);
       seu_out : out STD_LOGIC
  );
end TX_8x_4b_6b_RHSEU;

architecture TX_architecture1 of TX_8x_4b_6b_RHSEU is

---- Component declarations -----

  component ffd_RHSEU is
      port (
          CLR : in std_logic; -- active low reset
          en : in std_logic; -- en command
          clk : in std_logic;
          D : in std_logic;
          Q : out std_logic;
          QN : out std_logic;
          seu: out std_logic
      );
  end component;
  
component FIFO_NxD_RHSEU is
	generic (
	  N : integer := 8; 
	  D : integer := 16
	);
	port(
		word_in : in STD_LOGIC_VECTOR(N-1 downto 0);
		write : in STD_LOGIC;
		clk : in std_logic;
		read : in STD_LOGIC;
		rst_L : in std_logic;
		word_out : out std_logic_vector(N-1 downto 0);
		full : out STD_LOGIC;
		empty : out STD_LOGIC;
		seu : out STD_LOGIC
		);
end component;

component Frame_Builder_RHSEU
  port(
       Data_Type : in STD_LOGIC;
       Data_Valid : in STD_LOGIC;
       Get_Data : in STD_LOGIC;
       Label_On : in STD_LOGIC;
       clk : in STD_LOGIC;
       delay_HDR : in STD_LOGIC;
       rst_L : in STD_LOGIC;
       sel_shr : in STD_LOGIC;
       FDC_FIFO_empty : in STD_LOGIC;
       FDC_FIFO_out : in STD_LOGIC_VECTOR(N_FDC-1 downto 0);
       HDR_cmd : out STD_LOGIC;
       Read_TX_Buf : out STD_LOGIC;
       stop_SER : out STD_LOGIC;
       read_FDC_FIFO : out STD_LOGIC;
       write_FDC_FIFO : out STD_LOGIC;
       FDC_FIFO_in : out STD_LOGIC_VECTOR (N_FDC-1 downto 0);
	   seu_out : out STD_LOGIC
  );
end component;

component pulse_shrk_strch_RHSEU
  generic(
       N_wakeup_b : integer
  );
  port (
       N_wakeup : in STD_LOGIC_VECTOR(N_wakeup_b-1 downto 0);
       clk : in STD_LOGIC;
       pulse_in : in STD_LOGIC;
       rst_L : in STD_LOGIC;
       shrk_pulse : out STD_LOGIC;
       strtch_pulse : out STD_LOGIC;
	   seu_out : out STD_LOGIC
  );
end component;
component sel_THS_gen_TX_RHSEU
  port (
       clk40 : in STD_LOGIC;
       clk_tx : in STD_LOGIC;
       rst_L : in STD_LOGIC;
       sel_THS : out STD_LOGIC;
	   seu_out : out STD_LOGIC
  );
end component;
component serializer_4b_6b_8x_RHSEU
  generic(
       N_CYC : INTEGER;
       N_CYC_b : INTEGER;
       N_FRM : INTEGER
  );
  port (
       FDC_in : in STD_LOGIC_VECTOR(N_FDC-1 downto 0);
       FLF_data_in : in STD_LOGIC_VECTOR(N_FRM-1 downto 0);
       FLF_going : in STD_LOGIC;
       HDR_cmd : in STD_LOGIC;
       SYN_cmd : in STD_LOGIC;
       TRG_cmd : in STD_LOGIC;
       clk : in STD_LOGIC;
       read_TX_Buf : in STD_LOGIC;
       rst_L : in STD_LOGIC;
       sel_THS : in STD_LOGIC;
       stop_in : in STD_LOGIC;
       word_in : in STD_LOGIC_VECTOR(N_WORD-1 downto 0);
       DAT_out : out STD_LOGIC;
       delay_HDR : out STD_LOGIC;
       sel_shr : out STD_LOGIC;
	   seu_out : out STD_LOGIC
  );
end component;
component ths_scheduler_RHSEU
  port (
       HDR_in : in STD_LOGIC;
       TRG_in : in STD_LOGIC;
       clk : in STD_LOGIC;
       rst_L : in STD_LOGIC;
       HDR_out : out STD_LOGIC;
       TRG_out : out STD_LOGIC;
	   seu_out : out STD_LOGIC
  );
end component;


----     Constants     -----
constant GND_CONSTANT   : STD_LOGIC := '0';

---- Signal declarations used on the diagram ----

signal delay_hdr : STD_LOGIC;
signal delay_hdr_eff : STD_LOGIC;
signal FLF_going : STD_LOGIC;
signal FLF_start : STD_LOGIC;
signal Get_Data_int : STD_LOGIC;
signal GND : STD_LOGIC;
signal HDR_in : STD_LOGIC;
signal HDR_out : STD_LOGIC;
signal read_TX_Buf_int : STD_LOGIC;
signal sel_shr : STD_LOGIC;
signal sel_THS : STD_LOGIC;
signal stop_SER : STD_LOGIC;
signal SYN_cmd : STD_LOGIC;
signal TRG_out : STD_LOGIC;
signal TRG_shrk : STD_LOGIC;
signal TRG_FLF : STD_LOGIC;
signal FLF_data : STD_LOGIC_VECTOR (N_FRM-1 downto 0);
signal FLF_param_m : STD_LOGIC_VECTOR (FLF_param_b-1 downto 0);
signal seu_FB , seu_PLS_SHRK , seu_PLS_STR , seu_SELTHS , seu_SER,
       seu_THS_SCH, seu_FLF_FIFO, seu_DAT_OUT_FF : STD_LOGIC;

signal DAT_out_int : STD_LOGIC;


begin

---- User Signal Assignments ----
FLF_param_m <= FLF_param when FLF_on = '1' else CONV_STD_LOGIC_VECTOR(2,FLF_param_b);
-- if FLF are disabled, the TRG shrinker must sleep for just 2 clk40 cycles 
-- so that 3-clk-cycles-spaced triggers can be transmitted


----  Component instantiations  ----

FB : Frame_Builder_RHSEU
  port map(
       Data_Type => Data_Type,
       Data_Valid => Data_Valid,
       Get_Data => Get_Data_int,
       HDR_cmd => HDR_in,
       Label_On => Label_On,
       Read_TX_Buf => read_TX_Buf_int,
       clk => clk40,
       delay_HDR => delay_hdr_eff,
       rst_L => rst_L,
       sel_shr => sel_shr,
       stop_SER => stop_SER, 
       FDC_FIFO_empty => FDC_FIFO_empty,
       FDC_FIFO_out => FDC_FIFO_out,
       read_FDC_FIFO => read_FDC_FIFO,
       write_FDC_FIFO => write_FDC_FIFO,
       FDC_FIFO_in => FDC_FIFO_in,
	   seu_out => seu_FB
  );

  FLF_FIFO : fifo_nxd_rhseu
      generic map (
          N => N_FRM,
          D => 4
      )
      port map (
          word_in => FLF_data_in(N_FRM-1 downto 0),
          write => TRG_FLF,
          clk => clk40,
          read => FLF_going,
          rst_L => rst_L,
          word_out => FLF_data(N_FRM-1 downto 0),
          full => open,
          empty => open,
          seu => seu_FLF_FIFO
      );

PLS_SHRK : pulse_shrk_strch_RHSEU
  generic map (
       N_wakeup_b => FLF_param_b
  )
  port map(
       N_wakeup => FLF_param_m( FLF_param_b-1 downto 0 ),
       clk => clk40,
       pulse_in => TRG,
       rst_L => rst_L,
       shrk_pulse => TRG_shrk,
	   seu_out => seu_PLS_SHRK
  );

PLS_STR : pulse_shrk_strch_RHSEU
  generic map (
       N_wakeup_b => FLF_param_b
  )
  port map(
       N_wakeup => FLF_param_m( FLF_param_b-1 downto 0 ),
       clk => clk40,
       pulse_in => FLF_start,
       rst_L => rst_L,
       strtch_pulse => FLF_going,
	   seu_out => seu_PLS_STR
  );

SELTHS : sel_THS_gen_TX_RHSEU
  port map(
       clk40 => clk40,
       clk_tx => clk_tx,
       rst_L => rst_L,
       sel_THS => sel_THS,
	   seu_out => seu_SELTHS
  );

SER : serializer_4b_6b_8x_RHSEU
  generic map (
       N_CYC => N_CYC,
       N_CYC_b => N_CYC_b,
       N_FRM => N_FRM
  )
  port map(
       DAT_out => DAT_out_int,
       FDC_in => FDC_FIFO_out,
       FLF_data_in => FLF_data( N_FRM-1 downto 0 ),
       FLF_going => FLF_going,
       HDR_cmd => HDR_out,
       SYN_cmd => SYN_cmd,
       TRG_cmd => TRG_out,
       clk => clk_tx,
       delay_HDR => delay_hdr,
       read_TX_Buf => read_TX_Buf_int,
       rst_L => rst_L,
       sel_THS => sel_THS,
       sel_shr => sel_shr,
       stop_in => stop_SER,
       word_in => TX_BUF_out( N_WORD-1 downto 0 ),
	   seu_out => seu_SER
  );

THS_SCH : ths_scheduler_RHSEU
  port map(
       HDR_in => HDR_in,
       HDR_out => HDR_out,
       TRG_in => TRG_shrk,
       TRG_out => TRG_out,
       clk => clk40,
       rst_L => rst_L,
	   seu_out => seu_THS_SCH
  );


Get_Data_int <= not(TX_BUF_full or FDC_FIFO_full);

FLF_start <= FLF_on and TRG_out;

read_TX_Buf <= HDR_out or read_TX_Buf_int;

TRG_FLF <= TRG and FLF_on;

delay_hdr_eff <= FLF_going or delay_hdr or TRG_FLF;


---- Power , ground assignment ----

GND <= GND_CONSTANT;
SYN_cmd <= GND;

---- Terminal assignment ----

    -- Output\buffer terminals
	Get_Data <= Get_Data_int;

  -- DAT_out is registered
  DAT_OUT_FFD : ffd_rhseu
	port map(
		CLR => rst_L,
		en => '1',
		clk => clk_tx,
		D => DAT_out_int,
		Q => DAT_out,
		QN => open,
		seu => seu_DAT_OUT_FF
	);        
  
  seu_out <= seu_FB OR seu_PLS_SHRK OR seu_PLS_STR OR seu_SELTHS OR seu_SER OR seu_THS_SCH or seu_FLF_FIFO or seu_DAT_OUT_FF;

end TX_architecture1;
