
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_misc.all;
use IEEE.numeric_std.all;
use work.interfaces_pkg.all;

entity clk_splitter_8x_RHSEU is
  port(
       clk_in : in STD_LOGIC;
       rst_L : in STD_LOGIC;
       clk_out : out STD_LOGIC_VECTOR(7 downto 0);
       count : out STD_LOGIC_VECTOR(2 downto 0);
       seu : out STD_LOGIC
  );
end clk_splitter_8x_RHSEU;

architecture clk_splitter_8x_1 of clk_splitter_8x_RHSEU is

component Counter_N_RHSEU
  generic(
       N : INTEGER;
       modul : INTEGER
  );
  port (
       clk : in STD_LOGIC;
       en : in STD_LOGIC;
       rst_L : in STD_LOGIC;
       count : out STD_LOGIC_VECTOR(N-1 downto 0);
       seu : out STD_LOGIC
  );
end component;
component ffd_RHSEU
  port (
       CLR : in STD_LOGIC;
       D : in STD_LOGIC;
       clk : in STD_LOGIC;
       en : in STD_LOGIC;
       Q : out STD_LOGIC;
       QN : out STD_LOGIC;
       seu : out STD_LOGIC
  );
end component;
component SIPO_reg_N_ls_RHSEU
  generic(
       N : INTEGER
  );
  port (
       S_in : in STD_LOGIC;
       clk : in STD_LOGIC;
       rst_L : in STD_LOGIC;
       shift_en : in STD_LOGIC;
       P_out : out STD_LOGIC_VECTOR(N-1 downto 0);
       seu : out STD_LOGIC
  );
end component;

----     Constants     -----
constant VCC_CONSTANT   : STD_LOGIC := '1';

---- Signal declarations used on the diagram ----

signal clk : STD_LOGIC;
signal VCC : STD_LOGIC;
signal clk_out_int : STD_LOGIC_VECTOR (7 downto 0);
signal count_int : STD_LOGIC_VECTOR (2 downto 0);
signal seu_U2, seu_CNT, seu_SHR2 : STD_LOGIC;

begin

---- User Signal Assignments ----
clk_out_int(1) <= count_int(2);

----  Component instantiations  ----

SHR2 : SIPO_reg_N_ls_RHSEU
  generic map (
       N => 6
  )
  port map(
       P_out => clk_out_int( 7 downto 2 ),
       S_in => clk_out_int(1),
       clk => clk,
       rst_L => rst_L,
       shift_en => VCC,
       seu => seu_SHR2
  );

U1 : Counter_N_RHSEU
  generic map (
       N => 3,
       modul => 8
  )
  port map(
       clk => clk,
       count => count_int( 2 downto 0 ),
       en => VCC,
       rst_L => rst_L,
       seu => seu_CNT
  );

U2 : ffd_RHSEU
  port map(
       CLR => rst_L,
       D => clk_out_int(7),
       Q => clk_out_int(0),
       clk => clk,
       en => VCC,
       seu => seu_U2
  );


---- Power , ground assignment ----

VCC <= VCC_CONSTANT;

---- Terminal assignment ----

    -- Inputs terminals
	clk <= clk_in;

    -- Output\buffer terminals
	clk_out <= clk_out_int;
	count <= count_int;
 
    seu <= seu_U2 or seu_CNT or seu_SHR2;

end clk_splitter_8x_1;
