
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_misc.all;
use IEEE.math_real.all;
use IEEE.numeric_std.all;
use work.interfaces_pkg.all;


entity deserializer_4b_6b_8x_RHSEU is
  generic(
       N_FRM : INTEGER := 2 -- number of FRMbits in each clk40 cycle
  );
  port(
       DAT_in : in STD_LOGIC;
       FLF_going : in STD_LOGIC;
       HDR : in STD_LOGIC;
       clk : in STD_LOGIC;
       rst_L : in STD_LOGIC;
       sel_THS_del : in STD_LOGIC;
       FDC_rcv : out STD_LOGIC;
       sel_shr : out STD_LOGIC;
       FLF_data_out : out STD_LOGIC_VECTOR(N_FRM-1 downto 0);
       THS_out : out STD_LOGIC_VECTOR(5 downto 0);
       word_out : out STD_LOGIC_VECTOR(N_WORD-1 downto 0);
       seu : out STD_LOGIC
  );
end deserializer_4b_6b_8x_RHSEU;

architecture des1 of deserializer_4b_6b_8x_RHSEU is

---- Component declarations -----

component Counter_N_RHSEU
  generic(
       N : INTEGER;
       modul : INTEGER
  );
  port (
       clk : in STD_LOGIC;
       en : in STD_LOGIC;
       rst_L : in STD_LOGIC;
       count : out STD_LOGIC_VECTOR(N-1 downto 0);
       seu : out STD_LOGIC
  );
end component;

component Des_Control_RHSEU
  port (
       HDR : in STD_LOGIC;
       clk : in STD_LOGIC;
       countB : in STD_LOGIC_VECTOR(N_WORD_b-1 downto 0);
       rst_L : in STD_LOGIC;
       FDC_rcv : out STD_LOGIC;
       rst_BCNT_L : out STD_LOGIC;
       sel_shr : out STD_LOGIC;
       seu : out STD_LOGIC
  );
end component;

component mux2_Nbit
  generic(
       N : INTEGER
  );
  port (
       sel : in STD_LOGIC;
       x0 : in STD_LOGIC_VECTOR(N-1 downto 0);
       x1 : in STD_LOGIC_VECTOR(N-1 downto 0);
       z : out STD_LOGIC_VECTOR(N-1 downto 0)
  );
end component;

component reg_N_RHSEU
  generic(
       N : INTEGER
  );
  port (
       clk : in STD_LOGIC;
       data_in : in STD_LOGIC_VECTOR(N-1 downto 0);
       rst_L : in STD_LOGIC;
       write : in STD_LOGIC;
       data_out : out STD_LOGIC_VECTOR(N-1 downto 0);
       seu : out STD_LOGIC
  );
end component;

component SIPO_reg_N_RHSEU
  generic(
       N : INTEGER
  );
  port (
       S_in : in STD_LOGIC;
       clk : in STD_LOGIC;
       rst_L : in STD_LOGIC;
       shift_en : in STD_LOGIC;
       P_out : out STD_LOGIC_VECTOR(N-1 downto 0);
       seu : out STD_LOGIC
  );
end component;

component SISOPO_reg_N_RHSEU
  generic(
       N : INTEGER
  );
  port (
       S_in : in STD_LOGIC;
       clk : in STD_LOGIC;
       rst_L : in STD_LOGIC;
       shift_en : in STD_LOGIC;
       P_out : out STD_LOGIC_VECTOR(N-1 downto 0);
       S_out : out STD_LOGIC;
       seu : out STD_LOGIC
  );
end component;

component SISO_reg_N_RHSEU
  generic(
       N : INTEGER
  );
  port (
       S_in : in STD_LOGIC;
       clk : in STD_LOGIC;
       rst_L : in STD_LOGIC;
       shift_en : in STD_LOGIC;
       S_out : out STD_LOGIC;
       seu : out STD_LOGIC
  );
end component;

component dec_4b_6b
  port (
       input : in STD_LOGIC_VECTOR(5 downto 0);
       output : out STD_LOGIC_VECTOR(3 downto 0)
  );
end component;

component des_timing_4b_6b_8x_RHSEU is
  port(
       clk : in STD_LOGIC; -- 8x clk
       en : in STD_LOGIC;
       rst_L : in STD_LOGIC;
       sel_THS : in STD_LOGIC;
       pause_BCNT : out STD_LOGIC; 
       shift_en_SHR : out STD_LOGIC; 
	   seu_out : out STD_LOGIC
  );
end component;

component shr_4x4_RHSEU
  port (
    clk : in STD_LOGIC;
    in_16b : in STD_LOGIC_VECTOR(15 downto 0);
    in_4b : in STD_LOGIC_VECTOR(3 downto 0);
    load : in STD_LOGIC;
    rst_L : in STD_LOGIC;
    shift_en : in STD_LOGIC;
    out_16b : out STD_LOGIC_VECTOR(15 downto 0);
    out_4b : out STD_LOGIC_VECTOR(3 downto 0);
	seu : out std_logic
  );
end component;

----     Constants     -----
constant VCC_CONSTANT   : STD_LOGIC := '1';
constant GND_CONSTANT   : STD_LOGIC := '0';

---- Signal declarations used on the diagram ----

signal BCNT_en : STD_LOGIC;
signal GND : STD_LOGIC;
signal NET23151 : STD_LOGIC;
signal pause_BCNT : STD_LOGIC;
signal R1_out : STD_LOGIC;
signal R2_out : STD_LOGIC;
signal R3_out : STD_LOGIC;
signal R4_out : STD_LOGIC;
signal rst_BCNT_L : STD_LOGIC;
signal sel_shr_int : STD_LOGIC;
signal shift_en_4b_6b : STD_LOGIC;
signal shift_FLF : STD_LOGIC;
signal shift_VLF : STD_LOGIC;
signal SHR0_shift_en_eff : STD_LOGIC;
signal SHR1_shift_en_eff : STD_LOGIC;
signal VCC : STD_LOGIC;
signal BUS17080 : STD_LOGIC_VECTOR (15 downto 0);
signal BUS17086 : STD_LOGIC_VECTOR (15 downto 0);
signal countB : STD_LOGIC_VECTOR (N_WORD_b-1 downto 0);
signal nibble : STD_LOGIC_VECTOR (3 downto 0);
signal nibble_enc : STD_LOGIC_VECTOR (5 downto 0);
signal shr0_out : STD_LOGIC_VECTOR (N_WORD-1 downto 0);
signal shr1_out : STD_LOGIC_VECTOR (N_WORD-1 downto 0);
signal shr_flf_out : STD_LOGIC_VECTOR (N_FRM-1 downto 0);
signal THS_int : STD_LOGIC_VECTOR (5 downto 0);
signal seu_BIT_CNT, seu_DES_CTRL, seu_FLF_REG, seu_R1, seu_R2, seu_R3, seu_R4, seu_R5, seu_SHR0,
       seu_SHR1, seu_SHR_FLF, seu_SHR_VLF, seu_HDR_REG, seu_TIMING_4b_6b : STD_LOGIC;

begin

----  Component instantiations  ----

BIT_CNT : Counter_N_RHSEU
  generic map (
       N => N_WORD_b,
       modul => N_WORD
  )
  port map(
       clk => clk,
       count => countB( N_WORD_b-1 downto 0 ),
       en => BCNT_en,
       rst_L => rst_BCNT_L,
       seu => seu_BIT_CNT
  );

DES_CTRL : Des_Control_RHSEU
  port map(
       FDC_rcv => FDC_rcv,
       HDR => NET23151,
       clk => clk,
       countB => countB( N_WORD_b-1 downto 0 ),
       rst_BCNT_L => rst_BCNT_L,
       rst_L => rst_L,
       sel_shr => sel_shr_int,
       seu => seu_DES_CTRL
  );

FLF_REG : reg_N_RHSEU
  generic map (
       N => N_FRM
  )
  port map(
       clk => clk,
       data_in => shr_flf_out( N_FRM-1 downto 0 ),
       data_out => FLF_data_out( N_FRM-1 downto 0 ),
       rst_L => rst_L,
       write => sel_THS_del,
       seu => seu_FLF_REG
  );

HDR_SHR : SISO_reg_N_RHSEU
  generic map (
       N => 8
  )
  port map(
       S_in => HDR,
       S_out => NET23151,
       clk => clk,
       rst_L => rst_L,
       shift_en => VCC,
       seu => seu_HDR_REG
  );

M1 : mux2_Nbit
  generic map (
       N => N_WORD
  )
  port map(
       sel => sel_shr_int,
       x0 => shr1_out( N_WORD-1 downto 0 ),
       x1 => shr0_out( N_WORD-1 downto 0 ),
       z => word_out( N_WORD-1 downto 0 )
  );

R1 : SISOPO_reg_N_RHSEU
  generic map (
       N => 2
  )
  port map(
       P_out => THS_int( 5 downto 4 ),
       S_in => DAT_in,
       S_out => R1_out,
       clk => clk,
       rst_L => rst_L,
       shift_en => VCC,
       seu => seu_R1
  );

R2 : SISO_reg_N_RHSEU
  generic map (
       N => N_FRM
  )
  port map(
       S_in => R1_out,
       S_out => R2_out,
       clk => clk,
       rst_L => rst_L,
       shift_en => VCC,
       seu => seu_R2
  );

R3 : SISOPO_reg_N_RHSEU
  generic map (
       N => 2
  )
  port map(
       P_out => THS_int( 3 downto 2 ),
       S_in => R2_out,
       S_out => R3_out,
       clk => clk,
       rst_L => rst_L,
       shift_en => VCC,
       seu => seu_R3
  );

R4 : SISO_reg_N_RHSEU
  generic map (
       N => N_FRM
  )
  port map(
       S_in => R3_out,
       S_out => R4_out,
       clk => clk,
       rst_L => rst_L,
       shift_en => VCC,
       seu => seu_R4
  );

R5 : SISOPO_reg_N_RHSEU
  generic map (
       N => 2
  )
  port map(
       P_out => THS_int( 1 downto 0 ),
       S_in => R4_out,
       clk => clk,
       rst_L => rst_L,
       shift_en => VCC,
       seu => seu_R5
  );

SHR0 : shr_4x4_RHSEU
  port map(
       clk => clk,
       in_16b => BUS17080,
       in_4b => nibble,
       load => GND,
       out_16b => shr0_out( N_WORD-1 downto 0 ),
       rst_L => rst_L,
       shift_en => SHR0_shift_en_eff,
	   seu => seu_SHR0
  );

SHR1 : shr_4x4_RHSEU
  port map(
       clk => clk,
       in_16b => BUS17086,
       in_4b => nibble,
       load => GND,
       out_16b => shr1_out( N_WORD-1 downto 0 ),
       rst_L => rst_L,
       shift_en => SHR1_shift_en_eff,
	   seu => seu_SHR1
  );

SHR_FLF : SIPO_reg_N_RHSEU
  generic map (
       N => N_FRM
  )
  port map(
       P_out => shr_flf_out( N_FRM-1 downto 0 ),
       S_in => THS_int(1),
       clk => clk,
       rst_L => rst_L,
       shift_en => shift_FLF,
       seu => seu_SHR_FLF
  );

SHR_VLF : SIPO_reg_N_RHSEU
  generic map (
       N => N_FRM
  )
  port map(
       P_out => nibble_enc( 5 downto 0 ),
       S_in => THS_int(1),
       clk => clk,
       rst_L => rst_L,
       shift_en => shift_VLF,
       seu => seu_SHR_VLF
  );

DEC : dec_4b_6b
  port map(
       input => nibble_enc,
       output => nibble
  );

TIMING_4b_6b : des_timing_4b_6b_8x_RHSEU
  port map(
       clk => clk,
       en => VCC,
       pause_BCNT => pause_BCNT,
       rst_L => rst_L,
       sel_THS => sel_THS_del,
       shift_en_SHR => shift_en_4b_6b,
       seu_out => seu_TIMING_4b_6b
  );

shift_VLF <= not(FLF_going) and not(sel_THS_del);

shift_FLF <= not(sel_THS_del) and FLF_going;

SHR0_shift_en_eff <= not(FLF_going) and not(sel_shr_int) and shift_en_4b_6b;

BCNT_en <= not(pause_BCNT) and shift_VLF;

SHR1_shift_en_eff <= not(FLF_going) and sel_shr_int and shift_en_4b_6b;


---- Power , ground assignment ----

VCC <= VCC_CONSTANT;
GND <= GND_CONSTANT;
BUS17080(15) <= GND;
BUS17080(14) <= GND;
BUS17080(13) <= GND;
BUS17080(12) <= GND;
BUS17080(11) <= GND;
BUS17080(10) <= GND;
BUS17080(9) <= GND;
BUS17080(8) <= GND;
BUS17080(7) <= GND;
BUS17080(6) <= GND;
BUS17080(5) <= GND;
BUS17080(4) <= GND;
BUS17080(3) <= GND;
BUS17080(2) <= GND;
BUS17080(1) <= GND;
BUS17080(0) <= GND;
BUS17086(15) <= GND;
BUS17086(14) <= GND;
BUS17086(13) <= GND;
BUS17086(12) <= GND;
BUS17086(11) <= GND;
BUS17086(10) <= GND;
BUS17086(9) <= GND;
BUS17086(8) <= GND;
BUS17086(7) <= GND;
BUS17086(6) <= GND;
BUS17086(5) <= GND;
BUS17086(4) <= GND;
BUS17086(3) <= GND;
BUS17086(2) <= GND;
BUS17086(1) <= GND;
BUS17086(0) <= GND;

---- Terminal assignment ----

    -- Output\buffer terminals
	THS_out <= THS_int;
	sel_shr <= sel_shr_int;

    seu <= seu_BIT_CNT or seu_DES_CTRL or seu_FLF_REG or seu_R1 or seu_R2 or seu_R3 or seu_R4 or seu_R5
      or seu_SHR0 or seu_SHR1 or seu_SHR_FLF or seu_SHR_VLF or seu_HDR_REG or seu_TIMING_4b_6b;


end des1;
