										  -------------------------------------------------------------------------------
--
-- Title       : fb_control2_RHSEU
-- Design      : Interfaces
-- Author      : Nico Costantino
-- Company     : NC
--

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use work.interfaces_pkg.all;  

entity FB_Control2_RHSEU is 
	port (
		clk: in STD_LOGIC;
		delay_HDR: in STD_LOGIC;
		Frame_ready: in STD_LOGIC;
		last_W: in STD_LOGIC;
		rst_L: in STD_LOGIC;
		sel_shr: in STD_LOGIC;
		en_WCNT: out STD_LOGIC;
		HDR_cmd: out STD_LOGIC;
		read_FDC_FIFO: out STD_LOGIC;
		read_TX_Buf: out STD_LOGIC;
		rst_WCNT_L: out STD_LOGIC;
		stop_SER: out STD_LOGIC;
		seu : out STD_LOGIC
		);
end FB_Control2_RHSEU;

architecture FB_Control2_1 of FB_Control2_RHSEU is

type FB_CTRL2_Sreg0_type is (READY, START_FRAME, WAIT_WORD0, WAIT_WORD1);
signal next_state, current_state_a, current_state_b, current_state_c, current_state_corr: FB_CTRL2_Sreg0_type;
signal status :  STD_LOGIC_VECTOR(2 downto 0);
signal HDR_cmd_int : STD_LOGIC;
signal HDR_cmd_sig:  STD_LOGIC_VECTOR(0 to 2);
signal corr_data_HDR_cmd : STD_LOGIC;
signal read_TX_Buf_int : STD_LOGIC;
signal read_TX_Buf_sig:  STD_LOGIC_VECTOR(0 to 2);
signal corr_data_read_TX_Buf : STD_LOGIC;
signal stop_SER_int : STD_LOGIC;
signal stop_SER_sig:  STD_LOGIC_VECTOR(0 to 2);
signal corr_data_stop_SER : STD_LOGIC;
signal a_eq_b: std_logic;
signal b_eq_c : std_logic; 
signal a_eq_c : std_logic;	
signal seu_states : std_logic;	
signal seu_HDR_cmd : std_logic;	
signal seu_read_TX_Buf : std_logic;
signal seu_stop_SER : std_logic;

begin


fsm_state : process (clk)
begin
	if clk'event and clk = '1' then
		if rst_L = '0' then	
			current_state_a <= READY;
			current_state_b <= READY;
			current_state_c <= READY;
			
			for i in 0 to 2 loop
				HDR_cmd_sig(i) <= '0';
				read_TX_Buf_sig(i) <= '0';
				stop_SER_sig(i) <= '0';	
			end loop;  
		
		else
			current_state_a <= next_state;
			current_state_b <= next_state;
			current_state_c <= next_state;
		
			for i in 0 to 2 loop
				HDR_cmd_sig(i) <= HDR_cmd_int;
				read_TX_Buf_sig(i) <= read_TX_Buf_int;
				stop_SER_sig(i) <= stop_SER_int;	
			end loop;	 
			
			end if;
	end if;
end process;

fsm_comb_logic: process (current_state_corr, delay_HDR, Frame_ready,last_W,	sel_shr)
begin
	case current_state_corr is
				when READY =>
					if (Frame_ready = '1') AND (delay_HDR = '0') then	
						next_state <= START_FRAME;
						HDR_cmd_int <= '1';
						read_TX_Buf_int <= '0';
						stop_SER_int <= '0';
					else	
						next_state <= READY;
						HDR_cmd_int <= '0';
						read_TX_Buf_int <= '0';
						stop_SER_int <= '0';
					end if;
				when START_FRAME =>
					if (sel_shr = '1') AND (last_W = '0') then	
						next_state <= WAIT_WORD0;
						HDR_cmd_int <= '0';
						read_TX_Buf_int <= '1';
						stop_SER_int <= '0';
					elsif sel_shr = '0' then	
						next_state <= START_FRAME;
						HDR_cmd_int <= '0';
						read_TX_Buf_int <= '0';
						stop_SER_int <= '0';
					else	
						next_state <= READY;
						HDR_cmd_int <= '0';
						read_TX_Buf_int <= '0';
						stop_SER_int <= '1';
					end if;
				when WAIT_WORD0 =>
					if (sel_shr = '0') AND (last_W = '0') then	
						next_state <= WAIT_WORD1;
						HDR_cmd_int <= '0';
						read_TX_Buf_int <= '1';
						stop_SER_int <= '0';
					elsif (sel_shr = '0') AND (last_W = '1') then	
						next_state <= READY;
						HDR_cmd_int <= '0';
						read_TX_Buf_int <= '0';
						stop_SER_int <= '1';
					else	
						next_state <= WAIT_WORD0;
						HDR_cmd_int <= '0';
						read_TX_Buf_int <= '0';
						stop_SER_int <= '0';
					end if;
				when WAIT_WORD1 =>
					if (sel_shr = '1') AND (last_W = '0') then	
						next_state <= WAIT_WORD0;
						HDR_cmd_int <= '0';
						read_TX_Buf_int <= '1';
						stop_SER_int <= '0';
					elsif (sel_shr = '1') AND (last_W = '1') then	
						next_state <= READY;
						HDR_cmd_int <= '0';
						read_TX_Buf_int <= '0';
						stop_SER_int<= '1';
					else	
						next_state <= WAIT_WORD1;
						HDR_cmd_int <= '0';
						read_TX_Buf_int <= '0';
						stop_SER_int <= '0';
					end if;
				when others =>
						next_state <= READY;
						HDR_cmd_int <= '0';
						read_TX_Buf_int <= '0';
						stop_SER_int <= '0';
			end case;
end process;

	
  process (current_state_a, current_state_b, current_state_c)
	begin
        
		if (current_state_a = current_state_b) then
			a_eq_b <= '1';
		else
			a_eq_b <= '0';
		end if;
			
		if (current_state_a = current_state_c) then
			a_eq_c <= '1';
		else
			a_eq_c <= '0';
		end if;
				
		if (current_state_b = current_state_c) then
			b_eq_c <= '1';
		else
			b_eq_c <= '0';
		end if;
  end process;
  
  status <= a_eq_b & a_eq_c & b_eq_c;
  
  process (status, current_state_a, current_state_b)
    begin
		case (status) is 
			when "111" =>     			--All equal
				current_state_corr <= current_state_a; 
				seu_states <= '0'; 
			when "100" =>    			--c different
				current_state_corr <= current_state_a; 
				seu_states <= '1'; 
			when "010" => 				--b different
				current_state_corr <= current_state_a;
				seu_states <= '1'; 
			when "001" => 				--a different
				current_state_corr <= current_state_b;
				seu_states <= '1'; 
			when others => 			  -- errors in majority logic	
				current_state_corr <= READY;
				seu_states <= '1'; 
		end case;
	end process;   
	
majority_logic_HDR_cmd: process (HDR_cmd_sig)
	begin
		case (HDR_cmd_sig) is

			-- data = 0 - NO SEU
			when "000" => 	corr_data_HDR_cmd <= '0';  
							seu_HDR_cmd <= '0';	
			
			-- data = 1 - NO SEU
			when "111" => 	corr_data_HDR_cmd <= '1';  
							seu_HDR_cmd <= '0';
			
			-- data = 0 - SEU
			when "100" => 	corr_data_HDR_cmd <= '0';
							seu_HDR_cmd <= '1';	
			when "010" =>	corr_data_HDR_cmd <= '0';
							seu_HDR_cmd <= '1';
			when "001" => 	corr_data_HDR_cmd <= '0';
							seu_HDR_cmd <= '1';
                  
			-- data = 1 - SEU            
			when "110" => 	corr_data_HDR_cmd <= '1';
							seu_HDR_cmd <= '1';
			when "101" => 	corr_data_HDR_cmd <= '1';
							seu_HDR_cmd <= '1';
			when "011" => 	corr_data_HDR_cmd <= '1'; 
							seu_HDR_cmd <= '1';	
			
			when others => 	corr_data_HDR_cmd <= '0';
							seu_HDR_cmd <= '0';
		end case;	                                     
	end process;
	
majority_logic_read_TX_Buf: process (read_TX_Buf_sig)
	begin
		case (read_TX_Buf_sig) is

			-- data = 0 - NO SEU
			when "000" => 	corr_data_read_TX_Buf <= '0';  
							seu_read_TX_Buf <= '0';	
			
			-- data = 1 - NO SEU
			when "111" => 	corr_data_read_TX_Buf <= '1';  
							seu_read_TX_Buf <= '0';
			
			-- data = 0 - SEU
			when "100" => 	corr_data_read_TX_Buf <= '0';
							seu_read_TX_Buf <= '1';	
			when "010" =>	corr_data_read_TX_Buf <= '0';
							seu_read_TX_Buf <= '1';
			when "001" => 	corr_data_read_TX_Buf <= '0';
							seu_read_TX_Buf <= '1';
                  
			-- data = 1 - SEU            
			when "110" => 	corr_data_read_TX_Buf <= '1';
							seu_read_TX_Buf <= '1';
			when "101" => 	corr_data_read_TX_Buf <= '1';
							seu_read_TX_Buf <= '1';
			when "011" => 	corr_data_read_TX_Buf <= '1'; 
							seu_read_TX_Buf <= '1';	
			
			when others => 	corr_data_read_TX_Buf <= '0';
							seu_read_TX_Buf <= '0';
		end case;	                                     
	end process;

majority_logic_stop_SER: process (stop_SER_sig)
	begin
		case (stop_SER_sig) is

			-- data = 0 - NO SEU
			when "000" => 	corr_data_stop_SER <= '0';  
							seu_stop_SER <= '0';	
			
			-- data = 1 - NO SEU
			when "111" => 	corr_data_stop_SER <= '1';  
							seu_stop_SER <= '0';
			
			-- data = 0 - SEU
			when "100" => 	corr_data_stop_SER <= '0';
							seu_stop_SER <= '1';	
			when "010" =>	corr_data_stop_SER <= '0';
							seu_stop_SER <= '1';
			when "001" => 	corr_data_stop_SER <= '0';
							seu_stop_SER <= '1';
                  
			-- data = 1 - SEU            
			when "110" => 	corr_data_stop_SER <= '1';
							seu_stop_SER <= '1';
			when "101" => 	corr_data_stop_SER <= '1';
							seu_stop_SER <= '1';
			when "011" => 	corr_data_stop_SER <= '1'; 
							seu_stop_SER <= '1';	
			
			when others => 	corr_data_stop_SER <= '0';
							seu_stop_SER <= '0';
		end case;	                                     
	end process;
-- signal assignment statements for combinatorial outputs
read_FDC_FIFO_assignment:
read_FDC_FIFO <= '0' when (current_state_corr = READY and ((Frame_ready = '1') AND (delay_HDR = '0'))) else
                 '0' when (current_state_corr = READY and (((Frame_ready = '0') OR (delay_HDR = '1')) and not ((Frame_ready = '1') AND (delay_HDR = '0')))) else
                 '0' when (current_state_corr = READY) else
                 '0' when (current_state_corr = START_FRAME and ((sel_shr = '1') AND (last_W = '0'))) else
                 '0' when (current_state_corr = START_FRAME and (sel_shr = '0' and not ((sel_shr = '1') AND (last_W = '0')))) else
                 '1' when (current_state_corr = START_FRAME and ((sel_shr = '1') AND (last_W = '1') and not ((sel_shr = '1') AND (last_W = '0')) and not (sel_shr = '0'))) else
                 '0' when (current_state_corr = WAIT_WORD0 and ((sel_shr = '0') AND (last_W = '0'))) else
                 '1' when (current_state_corr = WAIT_WORD0 and ((sel_shr = '0') AND (last_W = '1') and not ((sel_shr = '0') AND (last_W = '0')))) else
                 '0' when (current_state_corr = WAIT_WORD0 and (sel_shr = '1' and not ((sel_shr = '0') AND (last_W = '0')) and not ((sel_shr = '0') AND (last_W = '1')))) else
                 '0' when (current_state_corr = WAIT_WORD1 and ((sel_shr = '1') AND (last_W = '0'))) else
                 '1' when (current_state_corr = WAIT_WORD1 and ((sel_shr = '1') AND (last_W = '1') and not ((sel_shr = '1') AND (last_W = '0')))) else
                 '0' when (current_state_corr = WAIT_WORD1 and (sel_shr = '0' and not ((sel_shr = '1') AND (last_W = '0')) and not ((sel_shr = '1') AND (last_W = '1')))) else
                 '0';

en_WCNT_assignment:
en_WCNT <= '0' when (current_state_corr = READY and ((Frame_ready = '1') AND (delay_HDR = '0'))) else
           '0' when (current_state_corr = READY and (((Frame_ready = '0') OR (delay_HDR = '1')) and not ((Frame_ready = '1') AND (delay_HDR = '0')))) else
           '0' when (current_state_corr = READY) else
           '1' when (current_state_corr = START_FRAME and ((sel_shr = '1') AND (last_W = '0'))) else
           '0' when (current_state_corr = START_FRAME and (sel_shr = '0' and not ((sel_shr = '1') AND (last_W = '0')))) else
           '0' when (current_state_corr = START_FRAME and ((sel_shr = '1') AND (last_W = '1') and not ((sel_shr = '1') AND (last_W = '0')) and not (sel_shr = '0'))) else
           '1' when (current_state_corr = WAIT_WORD0 and ((sel_shr = '0') AND (last_W = '0'))) else
           '0' when (current_state_corr = WAIT_WORD0 and ((sel_shr = '0') AND (last_W = '1') and not ((sel_shr = '0') AND (last_W = '0')))) else
           '0' when (current_state_corr = WAIT_WORD0 and (sel_shr = '1' and not ((sel_shr = '0') AND (last_W = '0')) and not ((sel_shr = '0') AND (last_W = '1')))) else
           '1' when (current_state_corr = WAIT_WORD1 and ((sel_shr = '1') AND (last_W = '0'))) else
           '0' when (current_state_corr = WAIT_WORD1 and ((sel_shr = '1') AND (last_W = '1') and not ((sel_shr = '1') AND (last_W = '0')))) else
           '0' when (current_state_corr = WAIT_WORD1 and (sel_shr = '0' and not ((sel_shr = '1') AND (last_W = '0')) and not ((sel_shr = '1') AND (last_W = '1')))) else
           '0';

rst_WCNT_L_assignment:
rst_WCNT_L <= '0' when (current_state_corr = READY and ((Frame_ready = '1') AND (delay_HDR = '0'))) else
              '0' when (current_state_corr = READY and (((Frame_ready = '0') OR (delay_HDR = '1')) and not ((Frame_ready = '1') AND (delay_HDR = '0')))) else
              '0' when (current_state_corr = READY) else
              '1' when (current_state_corr = START_FRAME and ((sel_shr = '1') AND (last_W = '0'))) else
              '1' when (current_state_corr = START_FRAME and (sel_shr = '0' and not ((sel_shr = '1') AND (last_W = '0')))) else
              '0' when (current_state_corr = START_FRAME and ((sel_shr = '1') AND (last_W = '1') and not ((sel_shr = '1') AND (last_W = '0')) and not (sel_shr = '0'))) else
              '1' when (current_state_corr = WAIT_WORD0 and ((sel_shr = '0') AND (last_W = '0'))) else
              '0' when (current_state_corr = WAIT_WORD0 and ((sel_shr = '0') AND (last_W = '1') and not ((sel_shr = '0') AND (last_W = '0')))) else
              '1' when (current_state_corr = WAIT_WORD0 and (sel_shr = '1' and not ((sel_shr = '0') AND (last_W = '0')) and not ((sel_shr = '0') AND (last_W = '1')))) else
              '1' when (current_state_corr = WAIT_WORD1 and ((sel_shr = '1') AND (last_W = '0'))) else
              '0' when (current_state_corr = WAIT_WORD1 and ((sel_shr = '1') AND (last_W = '1') and not ((sel_shr = '1') AND (last_W = '0')))) else
              '1' when (current_state_corr = WAIT_WORD1 and (sel_shr = '0' and not ((sel_shr = '1') AND (last_W = '0')) and not ((sel_shr = '1') AND (last_W = '1')))) else
              '0';	   
seu <=	seu_states OR seu_HDR_cmd OR seu_read_TX_Buf OR seu_stop_SER;
stop_SER <= corr_data_stop_SER;
HDR_cmd <= corr_data_HDR_cmd;
read_TX_buf <= corr_data_read_TX_buf;

end FB_Control2_1;
