-- package of types and functions for fifo used inside interfaces --

LIBRARY IEEE;
USE IEEE.std_logic_1164.all;

package fifo_pkg is    
	-- TX_DW parameters
	constant N_WORD_TX_DW: positive := 32; -- # words into the FIFO
	constant N_MUX_WORD_TX_DW: positive := 8; -- # words from FIFO internal muxes
	constant N_WORD_ADDR_TX_DW: positive := 16; -- # address bit for FIFO internal muxes
	constant N_BIT_COD_TX_DW: positive := 22; -- # output coded bits
	constant N_BIT_DEC_TX_DW: positive := 16; -- # output coded bits
	constant N_ADDR_TX_DW: positive := 5;	  -- # address bits, for scrubbing control
	constant N_BIT_PAR_TX_DW: positive := 6; -- # parity check bits 	 
	type fifo_array_type_TX_DW is array (N_MUX_WORD_TX_DW-1 downto 0) of std_logic_vector(N_BIT_COD_TX_DW-1 downto 0);
	
	-- TX_FD parameters
	constant N_WORD_TX_FD: positive := 32; -- # words into the FIFO
	constant N_MUX_WORD_TX_FD: positive := 8; -- # words from FIFO internal muxes
	constant N_WORD_ADDR_TX_FD: positive := 16; -- # address bit for FIFO internal muxes
	constant N_BIT_COD_TX_FD: positive := 18; -- # output coded bits
	constant N_BIT_DEC_TX_FD: positive := 12; -- # output coded bits
	constant N_ADDR_TX_FD: positive := 5;	  -- # address bits, for scrubbing control
	constant N_BIT_PAR_TX_FD: positive := 6; -- # parity check bits 	 
	type fifo_array_type_TX_FD is array (N_MUX_WORD_TX_FD-1 downto 0) of std_logic_vector(N_BIT_COD_TX_FD-1 downto 0); --# definition type to interface the logic to the FF-E matrix 
	
	-- RX_DW parameters
	constant N_WORD_RX_DW: positive := 32; -- # words into the FIFO
	constant N_MUX_WORD_RX_DW: positive := 8; -- # words from FIFO internal muxes
	constant N_WORD_ADDR_RX_DW: positive := 16; -- # address bit for FIFO internal muxes
	constant N_BIT_COD_RX_DW: positive := 22; -- # output coded bits
	constant N_BIT_DEC_RX_DW: positive := 16; -- # output coded bits
	constant N_ADDR_RX_DW: positive := 5;	  -- # address bits, for scrubbing control
	constant N_BIT_PAR_RX_DW: positive := 6; -- # parity check bits 	 
	type fifo_array_type_RX_DW is array (N_MUX_WORD_RX_DW-1 downto 0) of std_logic_vector(N_BIT_COD_RX_DW-1 downto 0); --# definition type to interface the logic to the FF-E matrix 

	-- RX_FD parameters
	constant N_WORD_RX_FD: positive := 32; -- # words into the FIFO
	constant N_MUX_WORD_RX_FD: positive := 8; -- # words from FIFO internal muxes
	constant N_WORD_ADDR_RX_FD: positive := 16; -- # address bit for FIFO internal muxes
	constant N_BIT_COD_RX_FD: positive := 14; -- # output coded bits
	constant N_BIT_DEC_RX_FD: positive := 9; -- # output coded bits
	constant N_ADDR_RX_FD: positive := 5;	  -- # address bits, for scrubbing control
	constant N_BIT_PAR_RX_FD: positive := 5; -- # parity check bits 	 
	type fifo_array_type_RX_FD is array (N_MUX_WORD_RX_FD-1 downto 0) of std_logic_vector(N_BIT_COD_RX_FD-1 downto 0); --# definition type to interface the logic to the FF-E matrix 
	
	
	end fifo_pkg;
	
	package body fifo_pkg is

	
	END fifo_pkg;