-- Hamming (7,3) decoder (SECDED)
--
--


LIBRARY IEEE;
USE IEEE.std_logic_1164.all;


entity hamming_7_3_decoder is
	 port(
		 word_in  : in std_logic_vector(6 downto 0);
		 word_dec : out std_logic_vector(2 downto 0);
		 s_err : out std_logic; -- signals a single-bit error 
		 d_err : out std_logic  -- signals a double-bit error 
	     );
end hamming_7_3_decoder;


architecture arch1 of hamming_7_3_decoder is 

	signal x : std_logic_vector(1 to 3); -- message vector
	signal p : std_logic_vector(1 to 4); -- parity bits vector
	signal s : std_logic_vector(1 to 4); -- vector of 3 syndrome bits plus the overall parity bit s(4)
	signal e : std_logic_vector(1 to 3); -- error vector
	
begin
		
	x(1 to 3) <= word_in(6 downto 4); -- message bits
	p(1 to 4) <= word_in(3 downto 0); -- parity bits
	
	-- syndrome calculation
	s(1) <= x(1) XOR x(2) XOR p(1);	
	s(2) <= x(1) XOR x(3) XOR p(2);
	s(3) <= x(2) XOR x(3) XOR p(3);
	
	-- overall parity bit calculation
	s(4) <= x(1) XOR x(2) XOR x(3) XOR p(1) XOR p(2) XOR p(3) XOR p(4);
	
	-- error vector calculation
	e(1) <= '1' when s(1 to 3) = "110" else '0';
	e(2) <= '1' when s(1 to 3) = "101" else '0';
	e(3) <= '1' when s(1 to 3) = "011" else '0';
    
    s_err <= e(1) OR e(2) OR e(3);
		
	-- correction logic
	word_dec(2 downto 0) <=  x(1 to 3) XOR e(1 to 3);
	d_err <= '1' when (s(1 to 3) /= "000") AND (s(4) = '0') else '0';

end arch1;
