-- reg_N
--
-- N-bit register with triple redundancy (loop to recover correct value from the output) 

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_misc.all;

entity reg_N_RHSEU is
	generic (N : integer );
	port (
		data_in : in std_logic_vector(N-1 downto 0);
		clk : in std_logic;
		rst_L : in std_logic; -- active low reset
		write : in std_logic; -- write command
		data_out : out std_logic_vector(N-1 downto 0);
		seu: out std_logic
	);
end entity;

architecture reg1 of reg_N_RHSEU is
	type array_nx3 is array (n-1 downto 0) of std_logic_vector(0 to 2);
	signal reg : array_nx3;
	signal maj_out, seu_arr : std_logic_vector(n-1 downto 0);  

	component maj_voter 
		port (  a : in std_logic;
				b : in std_logic;
				c : in std_logic;
				q : out std_logic;
				seu: out std_logic
				);  
	end component;

begin 
	
    process (clk)
	begin
   
		for i in 0 to n-1 loop
			if rising_edge(clk) then
				if (rst_L = '0') then
					reg(i) <= (others => '0');
				elsif (write = '1') then
					reg(i) <= (others => data_in(i));
				else
					reg(i) <= (others => maj_out(i));
				end if;
			end if;
		end loop;      
	end process;
	      
	maj_vot_inst: for i in 0 to n-1 generate  
			maj_vot: maj_voter PORT MAP (reg(i)(0), reg(i)(1), reg(i)(2), maj_out(i), seu_arr(i));		
	end generate; 	  
	
  seu <= or_reduce(seu_arr);
	
	data_out <= maj_out;					 
end architecture;
