
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_misc.all;
use IEEE.numeric_std.all;
use work.interfaces_pkg.all;



entity sel_THS_gen_RX_RHSEU is
  generic(
       N_CYC : integer; -- number of clk_tx cycles in each clk40 cycle
       N_CYC_b : integer -- = must be equal or greater than base 2 log of N_CYC
  );
  port(
       clk40_RX : in STD_LOGIC;
       clk_tx : in STD_LOGIC;
       THS_valid : out STD_LOGIC;
       sel_THS : out STD_LOGIC;
       sel_THS_del : out STD_LOGIC;
       seu : out STD_LOGIC
  );
end sel_THS_gen_RX_RHSEU;

architecture sel_THS_gen_RX1 of sel_THS_gen_RX_RHSEU is

---- Component declarations -----

component Counter_N_RHSEU
  generic(
       N : INTEGER;
       modul : INTEGER
  );
  port (
       clk : in STD_LOGIC;
       en : in STD_LOGIC;
       rst_L : in STD_LOGIC;
       count : out STD_LOGIC_VECTOR(N-1 downto 0);
       seu : out STD_LOGIC
  );
end component;
component ffd_RHSEU
  port (
       CLR : in STD_LOGIC;
       D : in STD_LOGIC;
       clk : in STD_LOGIC;
       en : in STD_LOGIC;
       Q : out STD_LOGIC;
       seu : out STD_LOGIC
  );
end component;

----     Constants     -----
constant VCC_CONSTANT   : STD_LOGIC := '1';

---- Signal declarations used on the diagram ----

signal sel_THS_del_int : STD_LOGIC;
signal sel_THS_int : STD_LOGIC;
signal VCC : STD_LOGIC;
signal count : STD_LOGIC_VECTOR (N_CYC_b-1 downto 0);
signal seu_CNT, seu_FF : STD_LOGIC;

begin

---- User Signal Assignments ----
sel_THS_int <= '1' when (count = CONV_STD_LOGIC_VECTOR(1, N_CYC_b)) OR
			(count = CONV_STD_LOGIC_VECTOR(2, N_CYC_b))
			else '0';

----  Component instantiations  ----

CNT : Counter_N_RHSEU
  generic map (
       N => N_CYC_b,
       modul => N_CYC
  )
  port map(
       clk => clk_tx,
       count => count( N_CYC_b-1 downto 0 ),
       en => VCC,
       rst_L => clk40_RX,
       seu => seu_CNT
  );

FF : ffd_RHSEU
  port map(
       CLR => VCC,
       D => sel_THS_int,
       Q => sel_THS_del_int,
       clk => clk_tx,
       en => VCC,
       seu => seu_FF
  );

THS_valid <= not(sel_THS_int) and sel_THS_del_int;


---- Power , ground assignment ----

VCC <= VCC_CONSTANT;

---- Terminal assignment ----

    -- Output\buffer terminals
	sel_THS <= sel_THS_int;
	sel_THS_del <= sel_THS_del_int;

    seu <= seu_CNT or seu_FF;

end sel_THS_gen_RX1;
