-------------------------------------------------------------------------------
-- ser_timing_4b_6b_8x_RHSEU
--

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_misc.all;


entity ser_timing_4b_6b_8x_RHSEU is
  port(
       clk : in STD_LOGIC; -- 8x clk
       en : in STD_LOGIC;
       HDR : in STD_LOGIC; -- HDR command
       stop : in STD_LOGIC; -- stop command
       rst_L : in STD_LOGIC;
       sel_THS : in STD_LOGIC;
       BCNT : in STD_LOGIC_VECTOR(3 downto 0); -- bit counter
       pause_BCNT : out STD_LOGIC; -- pause signal for the Bit Counter
       shift_en_SHR : out STD_LOGIC; -- shift enable for SHR0 and SHR1
	   seu_out : out STD_LOGIC
  );
end ser_timing_4b_6b_8x_RHSEU;

architecture ser_timing_4b_6b_8x_RHSEU1 of ser_timing_4b_6b_8x_RHSEU is

component Counter_N_RHSEU
  generic(
       N : INTEGER;
       modul : INTEGER
  );
  port (
       clk : in STD_LOGIC;
       en : in STD_LOGIC;
       rst_L : in STD_LOGIC;
       count : out STD_LOGIC_VECTOR(N-1 downto 0);
	   seu : out STD_LOGIC
  );
end component;

component ffd_RHSEU
  port (
       CLR : in STD_LOGIC;
       D : in STD_LOGIC;
       clk : in STD_LOGIC;
       en : in STD_LOGIC;
       Q : out STD_LOGIC;
       QN : out STD_LOGIC;
	   seu : out STD_LOGIC
  );
end component;

signal FF_rst_L : STD_LOGIC;
signal CNT_rst_L : STD_LOGIC;
signal count : STD_LOGIC_VECTOR (2 downto 0);

signal seu_out_CNT, seu_out_FF : STD_LOGIC;
       
       
begin
  
  FF : ffd_RHSEU
    port map(
         CLR => FF_rst_L,
         D => '1',
         Q => CNT_rst_L,
         clk => clk,
         en => HDR,
         seu => seu_out_FF
    );
    
  FF_rst_L <= '0' when (rst_L = '0') OR ((stop = '1') AND (BCNT = "1111") AND (HDR = '0')) else '1';
    
  CNT : Counter_N_RHSEU
    generic map (
         N => 3,
         modul => 0
    )
    port map(
         clk => clk,
         count => count,
         en => '1',
         rst_L => CNT_rst_L,
         seu => seu_out_CNT
    );
  
  pause_BCNT <= '1' when ((count = "010") OR (count = "011")) else '0';
  shift_en_SHR <= '1' when (count = "111") else '0';
  
  seu_out <= seu_out_CNT OR seu_out_FF;

end ser_timing_4b_6b_8x_RHSEU1;
