
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_misc.all;
use IEEE.math_real.all;
use IEEE.numeric_std.all;
use work.interfaces_pkg.all;


entity serializer_4b_6b_8x_RHSEU is
  generic(
       N_FRM : integer; -- number of FRMbits in each clk40 cycle
       N_CYC : integer; -- number of clk_tx cycles in each clk40 cycle
       N_CYC_b : integer -- = must be equal or greater than base 2 log of N_CYC
  );
  port(
       FLF_going : in STD_LOGIC;
       HDR_cmd : in STD_LOGIC;
       SYN_cmd : in STD_LOGIC;
       TRG_cmd : in STD_LOGIC;
       clk : in STD_LOGIC;
       read_TX_Buf : in STD_LOGIC;
       rst_L : in STD_LOGIC;
       sel_THS : in STD_LOGIC;
       stop_in : in STD_LOGIC;
       FDC_in : in STD_LOGIC_VECTOR(N_FDC-1 downto 0);
       FLF_data_in : in STD_LOGIC_VECTOR(N_FRM-1 downto 0);
       word_in : in STD_LOGIC_VECTOR(N_WORD-1 downto 0);
       DAT_out : out STD_LOGIC;
       delay_HDR : out STD_LOGIC;
       sel_shr : out STD_LOGIC;
	   seu_out : out STD_LOGIC
  );
end serializer_4b_6b_8x_RHSEU;

architecture serializer1 of serializer_4b_6b_8x_RHSEU is

---- Component declarations -----

component pulse_shrinker_RHSEU
  generic(
       N_wakeup : INTEGER;
       N_wakeup_b : INTEGER
  );
  port (
       clk : in STD_LOGIC;
       pulse_in : in STD_LOGIC;
       rst_L : in STD_LOGIC;
       shrk_pulse : out STD_LOGIC;
	   seu_out : out STD_LOGIC
  );
end component;

component Counter_N_RHSEU
  generic(
       N : INTEGER;
       modul : INTEGER
  );
  port (
       clk : in STD_LOGIC;
       en : in STD_LOGIC;
       rst_L : in STD_LOGIC;
       count : out STD_LOGIC_VECTOR(N-1 downto 0);
	   seu : out STD_LOGIC
  );
end component;

component ffd_RHSEU
  port (
       CLR : in STD_LOGIC;
       D : in STD_LOGIC;
       clk : in STD_LOGIC;
       en : in STD_LOGIC;
       Q : out STD_LOGIC;
       QN : out STD_LOGIC;
	   seu : out STD_LOGIC
  );
end component;

component mux2_Nbit
  generic(
       N : INTEGER
  );
  port (
       sel : in STD_LOGIC;
       x0 : in STD_LOGIC_VECTOR(N-1 downto 0);
       x1 : in STD_LOGIC_VECTOR(N-1 downto 0);
       z : out STD_LOGIC_VECTOR(N-1 downto 0)
  );
end component;

component PISO_reg_n_RHSEU
  generic(
       N : INTEGER
  );
  port (
       P_in : in STD_LOGIC_VECTOR(N-1 downto 0);
       clk : in STD_LOGIC;
       load : in STD_LOGIC;
       rst_L : in STD_LOGIC;
       shift_en : in STD_LOGIC;
       S_out : out STD_LOGIC;
	   seu : out STD_LOGIC
  );
end component;

component ser_control_RHSEU
  port (
       HDR_cmd : in STD_LOGIC;
       clk : in STD_LOGIC;
       countB : in STD_LOGIC_VECTOR(N_WORD_b-1 downto 0);
       rst_L : in STD_LOGIC;
       stop : in STD_LOGIC;
       rst_BCNT_L : out STD_LOGIC;
       sel_shr : out STD_LOGIC;
	   seu : out STD_LOGIC
  );
end component;

component SIPISO_reg_N_RHSEU
  generic(
       N : INTEGER
  );
  port (
       P_in : in STD_LOGIC_VECTOR(N-1 downto 0);
       S_in : in STD_LOGIC;
       clk : in STD_LOGIC;
       load : in STD_LOGIC;
       rst_L : in STD_LOGIC;
       shift_en : in STD_LOGIC;
       S_out : out STD_LOGIC;
	   seu : out STD_LOGIC
  );
end component;

component THS_cmd_selector
  port (
       HDR : in STD_LOGIC_VECTOR(N_THS-1 downto 0);
       HDR_cmd : in STD_LOGIC;
       SYN : in STD_LOGIC_VECTOR(N_THS-1 downto 0);
       SYN_cmd : in STD_LOGIC;
       TRG : in STD_LOGIC_VECTOR(N_THS-1 downto 0);
       TRG_cmd : in STD_LOGIC;
       THS_seq : out STD_LOGIC_VECTOR(N_THS-1 downto 0)
  );
end component;

component THS_load_ctrl_RHSEU
  port (
       THS_cmd : in STD_LOGIC;
       clk : in STD_LOGIC;
       rst_L : in STD_LOGIC;
       sel_THS : in STD_LOGIC;
       THS_busy : out STD_LOGIC;
       load_THS : out STD_LOGIC;
	   seu_out : out STD_LOGIC
  );
end component;

component enc_4b_6b
  port (
       input : in STD_LOGIC_VECTOR(3 downto 0);
       output : out STD_LOGIC_VECTOR(5 downto 0)
  );
end component;

component ser_timing_4b_6b_8x_RHSEU
  port (
       BCNT : in STD_LOGIC_VECTOR(3 downto 0);
       HDR : in STD_LOGIC;
       clk : in STD_LOGIC;
       en : in STD_LOGIC;
       rst_L : in STD_LOGIC;
       sel_THS : in STD_LOGIC;
       stop : in STD_LOGIC;
       pause_BCNT : out STD_LOGIC;
       shift_en_SHR : out STD_LOGIC;
	   seu_out : out STD_LOGIC
  );
end component;

component shr_4x4_RHSEU
  port (
    clk : in STD_LOGIC;
    in_16b : in STD_LOGIC_VECTOR(15 downto 0);
    in_4b : in STD_LOGIC_VECTOR(3 downto 0);
    load : in STD_LOGIC;
    rst_L : in STD_LOGIC;
    shift_en : in STD_LOGIC;
    out_16b : out STD_LOGIC_VECTOR(15 downto 0);
    out_4b : out STD_LOGIC_VECTOR(3 downto 0);
	seu : out std_logic
  );
end component;

----     Constants     -----
constant VCC_CONSTANT   : STD_LOGIC := '1';
constant GND_CONSTANT   : STD_LOGIC := '0';

---- Signal declarations used on the diagram ----

signal BCNT_en : STD_LOGIC;
signal delay_HDR_0 : STD_LOGIC;
signal FLF_going_del : STD_LOGIC;
signal FLF_stream : STD_LOGIC;
signal FRM_stream : STD_LOGIC;
signal GND : STD_LOGIC;
signal HDR_cmd_shrk : STD_LOGIC;
signal load_FLF : STD_LOGIC;
signal load_shr : STD_LOGIC;
signal load_shr0 : STD_LOGIC;
signal load_shr0_pre : STD_LOGIC;
signal load_shr1 : STD_LOGIC;
signal load_shr1_pre : STD_LOGIC;
signal load_THS : STD_LOGIC;
signal NET8058 : STD_LOGIC;
signal NET8574 : STD_LOGIC;
signal NET9871 : STD_LOGIC;
signal pause_BCNT : STD_LOGIC;
signal rst_BCNT_L : STD_LOGIC;
signal rst_DH_FF_L : STD_LOGIC;
signal sel_shr_int : STD_LOGIC;
signal shift_en_4b_6b : STD_LOGIC;
signal shift_en_shr0 : STD_LOGIC;
signal shift_en_shr1 : STD_LOGIC;
signal shift_FLF : STD_LOGIC;
signal shift_VLF : STD_LOGIC;
signal stop : STD_LOGIC;
signal STP_FF_rst_L : STD_LOGIC;
signal THS_busy : STD_LOGIC;
signal THS_cmd : STD_LOGIC;
signal THS_stream : STD_LOGIC;
signal VCC : STD_LOGIC;
signal VLF_stream : STD_LOGIC;
signal BUS10105 : STD_LOGIC_VECTOR (3 downto 0);
signal BUS8706 : STD_LOGIC_VECTOR (3 downto 0);
signal countB : STD_LOGIC_VECTOR (N_WORD_b-1 downto 0);
signal HDR : STD_LOGIC_VECTOR (N_THS-1 downto 0);
signal M1_input0 : STD_LOGIC_VECTOR (N_WORD-1 downto 0);
signal nibble : STD_LOGIC_VECTOR (3 downto 0);
signal nibble_enc : STD_LOGIC_VECTOR (5 downto 0);
signal shr0_input : STD_LOGIC_VECTOR (N_WORD-1 downto 0);
signal SHR0_out : STD_LOGIC_VECTOR (3 downto 0);
signal SHR1_out : STD_LOGIC_VECTOR (3 downto 0);
signal SYN : STD_LOGIC_VECTOR (N_THS-1 downto 0);
signal THS_seq : STD_LOGIC_VECTOR (N_THS-1 downto 0);
signal TRG : STD_LOGIC_VECTOR (N_THS-1 downto 0);

signal seu_out_BIT_CNT , seu_out_DH_FF , seu_out_NOP_FF , seu_out_PLS_SHRK , seu_out_SER_CTRL , seu_out_SHR0 , seu_out_SHR1 ,
	   seu_out_SHR_FLF , seu_out_SHR_VLF, seu_out_STP_FF , seu_out_THS_SHR , seu_out_TLC , seu_out_U5 , seu_out_TIMING_4b_6b : STD_LOGIC;

begin

---- User Signal Assignments ----
M1_input0(N_WORD-1 downto N_FDC) <= (others => '0'); 
M1_input0(N_FDC-1 downto 0) <= FDC_in;
rst_DH_FF_L <= '0' when (((stop = '1' OR stop_in = '1') AND countB = CONV_STD_LOGIC_VECTOR(N_WORD-3, N_WORD_b)) OR rst_L = '0') else '1';
-- THS sequences written in reverse order
TRG <= "110001"; 
HDR <= "001101";
SYN <= "001111";

----  Component instantiations  ----

BIT_CNT : Counter_N_RHSEU
  generic map (
       N => N_WORD_b,
       modul => N_WORD
  )
  port map(
       clk => clk,
       count => countB( N_WORD_b-1 downto 0 ),
       en => BCNT_en,
       rst_L => rst_BCNT_L,
	   seu => seu_out_BIT_CNT
  );

DH_FF : ffd_RHSEU
  port map(
       CLR => rst_DH_FF_L,
       D => VCC,
       Q => delay_HDR_0,
       clk => clk,
       en => HDR_cmd,
	   seu => seu_out_DH_FF
  );
  
ENC : enc_4b_6b
  port map(
       input => nibble,
       output => nibble_enc
  );

M1 : mux2_Nbit
  generic map (
       N => N_WORD
  )
  port map(
       sel => HDR_cmd,
       x0 => word_in( N_WORD-1 downto 0 ),
       x1 => M1_input0( N_WORD-1 downto 0 ),
       z => shr0_input( N_WORD-1 downto 0 )
  );

M2 : mux2_Nbit
  generic map (
       N => 4
  )
  port map(
       sel => sel_shr_int,
       x0 => SHR0_out( 3 downto 0 ),
       x1 => SHR1_out( 3 downto 0 ),
       z => nibble( 3 downto 0 )
  );

NOP_FF : ffd_RHSEU
  port map(
       CLR => sel_THS,
       D => VCC,
       Q => NET8058,
       clk => clk,
       en => VCC,
	   seu => seu_out_NOP_FF
  );

PLS_SHRK : pulse_shrinker_RHSEU
  generic map (
       N_wakeup => N_CYC,
       N_wakeup_b => N_CYC_b
  )
  port map(
       clk => clk,
       pulse_in => read_TX_Buf,
       rst_L => rst_L,
       shrk_pulse => load_shr,
	   seu_out => seu_out_PLS_SHRK
  );

SER_CTRL : ser_control_RHSEU
  port map(
       HDR_cmd => HDR_cmd_shrk,
       clk => clk,
       countB => countB( N_WORD_b-1 downto 0 ),
       rst_BCNT_L => rst_BCNT_L,
       rst_L => rst_L,
       sel_shr => sel_shr_int,
       stop => stop,
	   seu => seu_out_SER_CTRL
  );
  
SHR0 : shr_4x4_RHSEU
  port map(
       clk => clk,
       in_16b => shr0_input( N_WORD-1 downto 0 ),
       in_4b => BUS8706,
       load => load_shr0,
       out_4b => SHR0_out,
       rst_L => rst_L,
       shift_en => NET8574,
	   seu => seu_out_SHR0
  );

SHR1 : shr_4x4_RHSEU
  port map(
       clk => clk,
       in_16b => word_in( N_WORD-1 downto 0 ),
       in_4b => BUS10105,
       load => load_shr1,
       out_4b => SHR1_out,
       rst_L => rst_L,
       shift_en => NET9871,
	   seu => seu_out_SHR1
  );

SHR_FLF : PISO_reg_n_RHSEU
  generic map (
       N => N_FRM
  )
  port map(
       P_in => FLF_data_in( N_FRM-1 downto 0 ),
       S_out => FLF_stream,
       clk => clk,
       load => load_FLF,
       rst_L => rst_L,
       shift_en => shift_FLF,  
	   seu => seu_out_SHR_FLF
  );

SHR_VLF : PISO_reg_n_RHSEU
  generic map (
       N => N_FRM
  )
  port map(
       P_in => nibble_enc( 5 downto 0 ),
       S_out => VLF_stream,
       clk => clk,
       load => sel_THS,
       rst_L => rst_L,
       shift_en => shift_VLF,  
	   seu => seu_out_SHR_VLF
  );

STP_FF : ffd_RHSEU
  port map(
       CLR => STP_FF_rst_L,
       D => VCC,
       Q => stop,
       clk => clk,
       en => stop_in,
	   seu => seu_out_STP_FF
  );

TCS : THS_cmd_selector
  port map(
       HDR => HDR( N_THS-1 downto 0 ),
       HDR_cmd => HDR_cmd,
       SYN => SYN( N_THS-1 downto 0 ),
       SYN_cmd => SYN_cmd,
       THS_seq => THS_seq( N_THS-1 downto 0 ),
       TRG => TRG( N_THS-1 downto 0 ),
       TRG_cmd => TRG_cmd
  );

THS_SHR : SIPISO_reg_N_RHSEU
  generic map (
       N => N_THS
  )
  port map(
       P_in => THS_seq( N_THS-1 downto 0 ),
       S_in => NET8058,
       S_out => THS_stream,
       clk => clk,
       load => load_THS,
       rst_L => rst_L,
       shift_en => sel_THS, 
	   seu => seu_out_THS_SHR
  );

TIMING_4b_6b : ser_timing_4b_6b_8x_RHSEU
  port map(
       BCNT => countB( N_WORD_b-1 downto 0 ),
       HDR => HDR_cmd_shrk,
       clk => clk,
       en => VCC,
       pause_BCNT => pause_BCNT,
       rst_L => rst_L,
       sel_THS => sel_THS,
       shift_en_SHR => shift_en_4b_6b,
       stop => stop,
       seu_out => seu_out_TIMING_4b_6b
  );

TLC : THS_load_ctrl_RHSEU
  port map(
       THS_busy => THS_busy,
       THS_cmd => THS_cmd,
       clk => clk,
       load_THS => load_THS,
       rst_L => rst_L,
       sel_THS => sel_THS,
	   seu_out => seu_out_TLC
  );

BCNT_en <= not(pause_BCNT) and shift_VLF;

shift_VLF <= not(sel_THS) and not(FLF_going_del);

load_shr0 <= HDR_cmd_shrk or load_shr0_pre;

load_shr1_pre <= not(sel_shr_int) and load_shr;

load_shr0_pre <= sel_shr_int and load_shr;

HDR_cmd_shrk <= not(THS_busy) and HDR_cmd;

NET8574 <= shift_en_shr0 and shift_en_4b_6b;

delay_HDR <= delay_HDR_0 or THS_busy;

load_shr1 <= HDR_cmd_shrk or load_shr1_pre;

shift_FLF <= not(sel_THS) and FLF_going_del;

NET9871 <= shift_en_shr1 and shift_en_4b_6b;

DAT_out <= (FRM_stream and not sel_THS) or (THS_stream and sel_THS);

FRM_stream <= (VLF_stream and not FLF_going_del) or (FLF_stream and FLF_going_del);

load_FLF <= FLF_going_del and sel_THS;

U5 : ffd_RHSEU
  port map(
       CLR => rst_L,
       D => FLF_going,
       Q => FLF_going_del,
       clk => clk,
       en => VCC,
	   seu => seu_out_U5
  );

shift_en_shr0 <= not(sel_shr_int) and shift_VLF;

shift_en_shr1 <= sel_shr_int and shift_VLF;

THS_cmd <= SYN_cmd or HDR_cmd or TRG_cmd;

STP_FF_rst_L <= not(HDR_cmd) and rst_L;


---- Power , ground assignment ----

VCC <= VCC_CONSTANT;
GND <= GND_CONSTANT;
BUS8706(3) <= GND;
BUS8706(2) <= GND;
BUS8706(1) <= GND;
BUS8706(0) <= GND;
BUS10105(3) <= GND;
BUS10105(2) <= GND;
BUS10105(1) <= GND;
BUS10105(0) <= GND;

---- Terminal assignment ----

    -- Output\buffer terminals
	sel_shr <= sel_shr_int;

	seu_out <= seu_out_BIT_CNT OR seu_out_DH_FF OR seu_out_NOP_FF OR seu_out_PLS_SHRK OR seu_out_SER_CTRL OR seu_out_SHR0 OR seu_out_SHR1 
	   OR seu_out_SHR_FLF OR seu_out_SHR_VLF OR seu_out_STP_FF OR seu_out_THS_SHR OR seu_out_TLC OR seu_out_U5 OR seu_out_TIMING_4b_6b;

end serializer1;
