-- shr_4x4_RHSEU
--
-- 4 by 4-bit shift register (right shift), 4-bit/16-bit Input, 4-bit/16-bit Output

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_misc.all;

entity shr_4x4_RHSEU is
	port (
		clk : in std_logic;
		rst_L : in std_logic; -- active low reset
		load : in std_logic; -- load command
		shift_en : in std_logic; -- enables shifting	(right direction)
		in_4b  : in std_logic_vector(3 downto 0);
		in_16b : in std_logic_vector(15 downto 0);
		out_4b  : out std_logic_vector(3 downto 0);
		out_16b : out std_logic_vector(15 downto 0);
		seu : out std_logic   -- seu flag
	);
end entity;

architecture shr_4x4_RHSEU1 of shr_4x4_RHSEU is

	type array_nx3 is array (15 downto 0) of std_logic_vector(0 to 2);
	signal reg_in  : array_nx3;
	signal reg_out : array_nx3;
	signal en_ffe  : std_logic;
	signal maj_out : std_logic_vector(15 downto 0); 
	signal mux_out : std_logic_vector(15 downto 0);
	signal seu_arr : std_logic_vector(15 downto 0);

	component ffe
      port (
        d, rst_l, en, clk : in std_logic;
        q : out std_logic
      );	  
	end component;	 
	
	component maj_voter 
      port (
        a : in std_logic;
        b : in std_logic;
        c : in std_logic;
        q   : out std_logic;
        seu : out std_logic
      );  
	end component;
    
begin

	sr_rh: for i in 0 to 15 generate  
      ffe_a: ffe PORT MAP (reg_in(i)(0), rst_L, en_ffe, clk, reg_out(i)(0));
      ffe_b: ffe PORT MAP (reg_in(i)(1), rst_L, en_ffe, clk, reg_out(i)(1));
      ffe_c: ffe PORT MAP (reg_in(i)(2), rst_L, en_ffe, clk, reg_out(i)(2));
      maj_vot: maj_voter PORT MAP (reg_out(i)(0), reg_out(i)(1), reg_out(i)(2), maj_out(i), seu_arr(i));		
	end generate;
    
	process(load, shift_en, in_16b, in_4b, maj_out)
	begin
      if load = '1' then
        mux_out <= in_16b;
      elsif shift_en = '1' then
        mux_out(15 downto 12) <= in_4b;
        mux_out(11 downto 8)  <= maj_out(15 downto 12);
        mux_out(7 downto 4)   <= maj_out(11 downto 8);
        mux_out(3 downto 0)   <= maj_out(7 downto 4);
      else
        mux_out <= maj_out;
      end if;	 
	end process;
	
    seu <= or_reduce(seu_arr);
	
	process (mux_out)
	begin
	  for i in 0 to 15 loop
        reg_in(i)(0) <= mux_out(i);	
        reg_in(i)(1) <= mux_out(i);
        reg_in(i)(2) <= mux_out(i);	 
      end loop;	
	end process;  
	
	en_ffe <= load or shift_en;

	out_4b  <= maj_out(3 downto 0);
	out_16b <= maj_out(15 downto 0);
	
end architecture;
