-- SIPO_reg_N_ls
--
-- N-bit SIPO shift register (left shifting) 

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_misc.all;
use work.interfaces_pkg.all;

entity SIPO_reg_N_ls_RHSEU is
	generic (N : integer);
	port (
		clk : in std_logic;
		shift_en : in std_logic; -- enables shifting	(right direction)
		rst_L : in std_logic; -- active low reset
		S_in : in std_logic; -- serial input
		P_out : out std_logic_vector(N-1 downto 0); -- parallel output
		seu : out std_logic
	);
end entity;

architecture SIPO_reg_N1 of SIPO_reg_N_ls_RHSEU is
	
    type array_nx3 is array (n-1 downto 0) of std_logic_vector(0 to 2);
	signal reg_in : array_nx3;
	signal reg_out : array_nx3;
	signal en_ffe : std_logic;
	signal maj_out : std_logic_vector(n-1 downto 0); 
	signal mux_out : std_logic_vector(n-1 downto 0);
	signal seu_arr : std_logic_vector(n-1 downto 0);
	
	component ffe
		port (	d, rst_l, en, clk : in std_logic;
				q : out std_logic);	  
	end component;	 
	
	component maj_voter 
		port (  a : in std_logic;
				b : in std_logic;
				c : in std_logic;
				q : out std_logic;
				seu: out std_logic);  
	end component;

begin

	sr_rh: for i in 0 to n-1 generate  
			ffe_a: ffe PORT MAP (reg_in(i)(0), rst_L, en_ffe, clk, reg_out(i)(0));
			ffe_b: ffe PORT MAP (reg_in(i)(1), rst_L, en_ffe, clk, reg_out(i)(1));
			ffe_c: ffe PORT MAP (reg_in(i)(2), rst_L, en_ffe, clk, reg_out(i)(2));
			maj_vot: maj_voter PORT MAP (reg_out(i)(0), reg_out(i)(1), reg_out(i)(2), maj_out(i), seu_arr(i));		
	end generate; 	  
	
	process(shift_en, maj_out, S_in)
	begin
		if shift_en = '1' then
			for j in 0 to n-1 loop
				if j = 0 then 
					mux_out(j) <= S_in;
				else 
					mux_out(j) <= maj_out(j-1);
				end if;
			end loop;
        else
            mux_out <= maj_out;
		end if;	 
	end process;
	
  seu <= or_reduce(seu_arr);
	
	process (mux_out)
	begin
		for i in 0 to n-1 loop
		reg_in(i)(0) <= mux_out(i);	
		reg_in(i)(1) <= mux_out(i);
		reg_in(i)(2) <= mux_out(i);	 
		end loop;	
	end process;  

	P_out <= maj_out;	 
	
	en_ffe <= shift_en;
	

end architecture;
