--Token Receiver Code

--Import Libraries
LIBRARY ieee;
USE IEEE.STD_LOGIC_UNSIGNED.all;
USE ieee.std_logic_1164.all;
USE ieee.numeric_std.all;

--Entity Declaration
entity flf_rx is 
port
	(clk: IN STD_LOGIC;
   	 reset: IN STD_LOGIC;
   	 enable: IN STD_LOGIC;
   	 error: OUT STD_LOGIC;
   	 p1_in : IN STD_LOGIC;
   	 p2_in : IN STD_LOGIC;
   	 e1_in : IN STD_LOGIC;
   	 e2_in : IN STD_LOGIC);
end flf_rx;

ARCHITECTURE flf_rx_arch of flf_rx is

type p_state_type is (IDLE, PULSE1, PULSE0, PULSE_ERROR);
signal p1_current_state, p1_next_state : p_state_type;
signal p2_current_state, p2_next_state : p_state_type;
signal p1_t_counter_en, p1_t_counter_reset : STD_LOGIC;
signal p1_t_counter_out : STD_LOGIC_VECTOR(15 downto 0);
signal p2_t_counter_en, p2_t_counter_reset : STD_LOGIC;
signal p2_t_counter_out : STD_LOGIC_VECTOR(15 downto 0);
signal p1_t_error, p1_t_error_en, p1_t_error_reset : STD_LOGIC;
signal p2_t_error, p2_t_error_en, p2_t_error_reset : STD_LOGIC;
signal p1_w_error, p1_w_error_en, p1_w_error_reset : STD_LOGIC;
signal p2_w_error, p2_w_error_en, p2_w_error_reset : STD_LOGIC;
signal p1_error, p2_error : STD_LOGIC;

type e_state_type is (IDLE, EDGE1, EDGE0, EDGE_ERROR);
signal e1_current_state, e1_next_state : e_state_type;
signal e2_current_state, e2_next_state : e_state_type;
signal e1_t_counter_en, e1_t_counter_reset : STD_LOGIC;
signal e1_t_counter_out : STD_LOGIC_VECTOR(15 downto 0);
signal e2_t_counter_en, e2_t_counter_reset : STD_LOGIC;
signal e2_t_counter_out : STD_LOGIC_VECTOR(15 downto 0);
signal e1_w_counter_en, e1_w_counter_reset : STD_LOGIC;
signal e1_w_counter_out : STD_LOGIC_VECTOR(15 downto 0);
signal e2_w_counter_en, e2_w_counter_reset : STD_LOGIC;
signal e2_w_counter_out : STD_LOGIC_VECTOR(15 downto 0);
signal e1_t_error, e1_t_error_en, e1_t_error_reset : STD_LOGIC;
signal e2_t_error, e2_t_error_en, e2_t_error_reset : STD_LOGIC;
signal e1_w_error, e1_w_error_en, e1_w_error_reset : STD_LOGIC;
signal e2_w_error, e2_w_error_en, e2_w_error_reset : STD_LOGIC;
signal e1_error, e2_error : STD_LOGIC;

constant p1_period: STD_LOGIC_VECTOR(15 downto 0) := "0000000100000000";

constant p2_period: STD_LOGIC_VECTOR(15 downto 0) := "0000000100000000";

constant e1_width: STD_LOGIC_VECTOR(15 downto 0) := "0000000000010000";
constant e1_period: STD_LOGIC_VECTOR(15 downto 0) := "0000000100000000";

constant e2_width: STD_LOGIC_VECTOR(15 downto 0) := "0000000000010000";
constant e2_period: STD_LOGIC_VECTOR(15 downto 0) := "0000000100000000";
  
  
BEGIN

-- P1
 
p1_error_regs : process(clk, reset, p1_t_error_en, p1_t_error_reset, p1_w_error_en, p1_w_error_reset)

begin
      
if (reset = '1') then
	p1_t_error <= '0';
  	elsif rising_edge(clk) then
		if (p1_t_error_reset = '1') then
       		p1_t_error <= '0';
		elsif (p1_t_error_en = '1') then
       		p1_t_error <= '1';
		end if;
	end if;

	if (reset = '1') then
       	p1_w_error <= '0';
  	elsif rising_edge(clk) then
		if (p1_w_error_reset = '1') then
       		p1_w_error <= '0';
		elsif (p1_w_error_en = '1') then
       		p1_w_error <= '1';
		end if;
	end if;

	p1_error <= p1_t_error or p1_w_error;

end process;

p1_fsm_regs : process(clk, p1_next_state)

begin
      
	if (reset = '1') then
       	p1_current_state <= IDLE;
  	elsif rising_edge(clk) then
       	p1_current_state <= p1_next_state;
	end if;

end process;

p1_t_counter: process (clk, p1_t_counter_en, reset, p1_t_counter_reset)

variable p1_t_counter_data : std_logic_vector(15 downto 0);

begin

	if (reset = '1') then
		p1_t_counter_data := "0000000000000000";
	elsif (rising_edge(clk)) then
		if (p1_t_counter_reset = '1') then
			p1_t_counter_data := "0000000000000000";
		elsif (p1_t_counter_en = '1') then
			p1_t_counter_data := p1_t_counter_data + 1;
		end if;              
	end if; 
	p1_t_counter_out <= p1_t_counter_data;

end process;

p1_fsm_logic : process(clk, reset, p1_t_counter_out, p1_current_state, enable, p1_in)

begin

case p1_current_state is

	when IDLE =>

		if (enable = '1') then
			p1_w_error_en <= '0';
			p1_w_error_reset <= '0';
 			if ((p1_t_counter_out = p1_period) and (p1_in = '0')) then
				p1_t_error_en <= '1';
				p1_t_error_reset <= '0';
				p1_t_counter_en <= '0';
				p1_t_counter_reset <= '1';
				p1_next_state <= PULSE_ERROR;
			elsif (p1_in = '1') then
				p1_t_error_en <= '0';
				p1_t_error_reset <= '0';
				p1_t_counter_en <= '0';
				p1_t_counter_reset <= '1';
	    			p1_next_state <= PULSE1;
			else
				p1_t_error_en <= '0';
				p1_t_error_reset <= '0';
				p1_t_counter_en <= '1';
				p1_t_counter_reset <= '0';
				p1_next_state <= IDLE;
			end if;
		else
			p1_w_error_en <= '0';
			p1_w_error_reset <= '1';
			p1_t_error_en <= '0';
			p1_t_error_reset <= '1';
			p1_t_counter_en <= '0';
			p1_t_counter_reset <= '1';
			p1_next_state <= IDLE;
		end if;

	when PULSE1 =>

    		p1_t_error_en <= '0';
		p1_t_error_reset <= '0';
		if (p1_in = '0') then
		    	p1_w_error_en <= '0';
		    	p1_w_error_reset <= '0';
 			p1_t_counter_en <= '1';
			p1_t_counter_reset <= '0';
	      	p1_next_state <= PULSE0;
		else
			p1_w_error_en <= '1';
		    	p1_w_error_reset <= '0';
 			p1_t_counter_en <= '0';
			p1_t_counter_reset <= '1';
	      	p1_next_state <= PULSE_ERROR;
		end if;

	when PULSE0 => 

		p1_w_error_en <= '0';
		p1_w_error_reset <= '0';
		if (p1_in = '1') then
      		if (p1_t_counter_out = p1_period) then
 			  	p1_t_error_en <= '0';
				p1_t_error_reset <= '0';
	      		p1_next_state <= PULSE1;
			else
				p1_t_error_en <= '1';
				p1_t_error_reset <= '0';
	      		p1_next_state <= PULSE_ERROR;
			end if;
		  	p1_t_counter_en <= '0';
		  	p1_t_counter_reset <= '1';
    		else
      		if (p1_t_counter_out = p1_period) then
 			  	p1_t_error_en <= '1';
				p1_t_error_reset <= '0';
 				p1_t_counter_en <= '0';
				p1_t_counter_reset <= '1';
	      		p1_next_state <= PULSE_ERROR;
			else
 			  	p1_t_error_en <= '0';
 				p1_t_error_reset <= '0';
				p1_t_counter_en <= '1';
				p1_t_counter_reset <= '0';
      			p1_next_state <= PULSE0;
			end if;
    		end if;

	when PULSE_ERROR => 

		p1_t_counter_en <= '0';
		p1_t_counter_reset <= '1';
		if (reset = '1') or (enable = '0') then
			p1_t_error_en <= '0';
			p1_t_error_reset <= '1';
			p1_w_error_en <= '0';
			p1_w_error_reset <= '1';
	       	p1_next_state <= IDLE;
   		else
			p1_t_error_en <= '0';
			p1_t_error_reset <= '0';
			p1_w_error_en <= '0';
			p1_w_error_reset <= '0';
      		p1_next_state <= PULSE_ERROR;
   		end if;

	when others =>

		p1_t_error_en <= '0';
		p1_t_error_reset <= '1';
		p1_w_error_en <= '0';
		p1_w_error_reset <= '1';
 		p1_t_counter_en <= '0';
		p1_t_counter_reset <= '1';
		p1_next_state <= IDLE;

end case;
 
end process;

-- P2
 
p2_error_regs : process(clk, reset, p2_t_error_en, p2_t_error_reset, p2_w_error_en, p2_w_error_reset)

begin
      
if (reset = '1') then
	p2_t_error <= '0';
  	elsif rising_edge(clk) then
		if (p2_t_error_reset = '1') then
       		p2_t_error <= '0';
		elsif (p2_t_error_en = '1') then
       		p2_t_error <= '1';
		end if;
	end if;

	if (reset = '1') then
       	p2_w_error <= '0';
  	elsif rising_edge(clk) then
		if (p2_w_error_reset = '1') then
       		p2_w_error <= '0';
		elsif (p2_w_error_en = '1') then
       		p2_w_error <= '1';
		end if;
	end if;

	p2_error <= p2_t_error or p2_w_error;

end process;

p2_fsm_regs : process(clk, p2_next_state)

begin
      
	if (reset = '1') then
       	p2_current_state <= IDLE;
  	elsif rising_edge(clk) then
       	p2_current_state <= p2_next_state;
	end if;

end process;

p2_t_counter: process (clk, p2_t_counter_en, reset, p2_t_counter_reset)

variable p2_t_counter_data : std_logic_vector(15 downto 0);

begin

	if (reset = '1') then
		p2_t_counter_data := "0000000000000000";
	elsif (rising_edge(clk)) then
		if (p2_t_counter_reset = '1') then
			p2_t_counter_data := "0000000000000000";
		elsif (p2_t_counter_en = '1') then
			p2_t_counter_data := p2_t_counter_data + 1;
		end if;              
	end if; 
	p2_t_counter_out <= p2_t_counter_data;

end process;

p2_fsm_logic : process(clk, reset, p2_t_counter_out, p1_current_state, enable, p2_in)

begin

case p2_current_state is

	when IDLE =>

		if (enable = '1') then
			p2_w_error_en <= '0';
			p2_w_error_reset <= '0';
 			if ((p2_t_counter_out = p2_period) and (p2_in = '0')) then
				p2_t_error_en <= '1';
				p2_t_error_reset <= '0';
				p2_t_counter_en <= '0';
				p2_t_counter_reset <= '1';
				p2_next_state <= PULSE_ERROR;
			elsif (p2_in = '1') then
				p2_t_error_en <= '0';
				p2_t_error_reset <= '0';
				p2_t_counter_en <= '0';
				p2_t_counter_reset <= '1';
	    			p2_next_state <= PULSE1;
			else
				p2_t_error_en <= '0';
				p2_t_error_reset <= '0';
				p2_t_counter_en <= '1';
				p2_t_counter_reset <= '0';
				p2_next_state <= IDLE;
			end if;
		else
			p2_w_error_en <= '0';
			p2_w_error_reset <= '1';
			p2_t_error_en <= '0';
			p2_t_error_reset <= '1';
			p2_t_counter_en <= '0';
			p2_t_counter_reset <= '1';
			p2_next_state <= IDLE;
		end if;

	when PULSE1 =>

    		p2_t_error_en <= '0';
		p2_t_error_reset <= '0';
		if (p2_in = '0') then
		    	p2_w_error_en <= '0';
		    	p2_w_error_reset <= '0';
 			p2_t_counter_en <= '1';
			p2_t_counter_reset <= '0';
	      	p2_next_state <= PULSE0;
		else
			p2_w_error_en <= '1';
		    	p2_w_error_reset <= '0';
 			p2_t_counter_en <= '0';
			p2_t_counter_reset <= '1';
	      	p2_next_state <= PULSE_ERROR;
		end if;

	when PULSE0 => 

		p2_w_error_en <= '0';
		p2_w_error_reset <= '0';
		if (p2_in = '1') then
      		if (p2_t_counter_out = p2_period) then
 			  	p2_t_error_en <= '0';
				p2_t_error_reset <= '0';
	      		p2_next_state <= PULSE1;
			else
				p2_t_error_en <= '1';
				p2_t_error_reset <= '0';
	      		p2_next_state <= PULSE_ERROR;
			end if;
		  	p2_t_counter_en <= '0';
		  	p2_t_counter_reset <= '1';
    		else
      		if (p2_t_counter_out = p2_period) then
 			  	p2_t_error_en <= '1';
				p2_t_error_reset <= '0';
 				p2_t_counter_en <= '0';
				p2_t_counter_reset <= '1';
	      		p2_next_state <= PULSE_ERROR;
			else
 			  	p2_t_error_en <= '0';
 				p2_t_error_reset <= '0';
				p2_t_counter_en <= '1';
				p2_t_counter_reset <= '0';
      			p2_next_state <= PULSE0;
			end if;
    		end if;

	when PULSE_ERROR => 

		p2_t_counter_en <= '0';
		p2_t_counter_reset <= '1';
		if (reset = '1') or (enable = '0') then
			p2_t_error_en <= '0';
			p2_t_error_reset <= '1';
			p2_w_error_en <= '0';
			p2_w_error_reset <= '1';
	       	p2_next_state <= IDLE;
   		else
			p2_t_error_en <= '0';
			p2_t_error_reset <= '0';
			p2_w_error_en <= '0';
			p2_w_error_reset <= '0';
      		p2_next_state <= PULSE_ERROR;
   		end if;

	when others =>

		p2_t_error_en <= '0';
		p2_t_error_reset <= '1';
		p2_w_error_en <= '0';
		p2_w_error_reset <= '1';
 		p2_t_counter_en <= '0';
		p2_t_counter_reset <= '1';
		p2_next_state <= IDLE;

end case;
 
end process;

-- E1
 
e1_error_regs : process(clk, reset, e1_t_error_en, e1_t_error_reset, e1_w_error_en, e1_w_error_reset)

begin
      
	if (reset = '1') then
		e1_t_error <= '0';
  	elsif rising_edge(clk) then
		if (e1_t_error_reset = '1') then
       		e1_t_error <= '0';
		elsif (e1_t_error_en = '1') then
       		e1_t_error <= '1';
		end if;
	end if;

	if (reset = '1') then
       	e1_w_error <= '0';
  	elsif rising_edge(clk) then
		if (e1_w_error_reset = '1') then
       		e1_w_error <= '0';
		elsif (e1_w_error_en = '1') then
       		e1_w_error <= '1';
		end if;
	end if;

	e1_error <= e1_t_error or e1_w_error;

end process;

e1_fsm_regs : process(clk, e1_next_state)

begin
      
	if (reset = '1') then
       	e1_current_state <= IDLE;
  	elsif rising_edge(clk) then
       	e1_current_state <= e1_next_state;
	end if;

end process;

e1_t_counter: process (clk, e1_t_counter_en, reset, e1_t_counter_reset)

variable e1_t_counter_data : std_logic_vector(15 downto 0);

begin

	if (reset = '1') then
		e1_t_counter_data := "0000000000000000";
	elsif (rising_edge(clk)) then
		if (e1_t_counter_reset = '1') then
			e1_t_counter_data := "0000000000000000";
		elsif (e1_t_counter_en = '1') then
			e1_t_counter_data := e1_t_counter_data + 1;
		end if;              
	end if; 
	e1_t_counter_out <= e1_t_counter_data;

end process;

e1_w_counter: process (clk, e1_w_counter_en, reset, e1_w_counter_reset)

variable e1_w_counter_data : std_logic_vector(15 downto 0);

begin

	if (reset = '1') then
		e1_w_counter_data := "0000000000000000";
	elsif (rising_edge(clk)) then
		if (e1_w_counter_reset = '1') then
			e1_w_counter_data := "0000000000000000";
		elsif (e1_w_counter_en = '1') then
			e1_w_counter_data := e1_w_counter_data + 1;
		end if;              
	end if; 
	e1_w_counter_out <= e1_w_counter_data;

end process;
  
  
e1_fsm_logic : process(clk, reset, e1_w_counter_out, e1_t_counter_out, e1_current_state, enable, e1_in)

begin

case e1_current_state is

	when IDLE =>

		if (enable = '1') then
 			if ((e1_t_counter_out = e1_period) and (e1_in = '0')) then
				e1_t_error_en <= '1';
				e1_t_error_reset <= '0';
				e1_w_error_en <= '0';
				e1_w_error_reset <= '0';
				e1_t_counter_en <= '0';
				e1_t_counter_reset <= '1';
				e1_w_counter_en <= '0';
				e1_w_counter_reset <= '1';
				e1_next_state <= EDGE_ERROR;
			elsif (e1_in = '1') then
				e1_t_error_en <= '0';
				e1_t_error_reset <= '0';
				e1_w_error_en <= '0';
				e1_w_error_reset <= '0';
				e1_t_counter_en <= '0';
				e1_t_counter_reset <= '1';
				e1_w_counter_en <= '0';
				e1_w_counter_reset <= '1';
	    			e1_next_state <= EDGE1;
			else
				e1_t_error_en <= '0';
				e1_t_error_reset <= '0';
				e1_w_error_en <= '0';
				e1_w_error_reset <= '0';
				e1_t_counter_en <= '1';
				e1_t_counter_reset <= '0';
				e1_w_counter_en <= '0';
				e1_w_counter_reset <= '1';
				e1_next_state <= IDLE;
			end if;
		else
			e1_t_error_en <= '0';
			e1_t_error_reset <= '1';
			e1_w_error_en <= '0';
			e1_w_error_reset <= '1';
			e1_t_counter_en <= '0';
			e1_t_counter_reset <= '1';
 			e1_w_counter_en <= '0';
			e1_w_counter_reset <= '1';
			e1_next_state <= IDLE;
		end if;

	when EDGE1 =>

		if (e1_in = '0') then
		  	if (e1_w_counter_out = e1_width) then
	    			e1_t_error_en <= '0';
				e1_t_error_reset <= '0';
	    			e1_w_error_en <= '0';
				e1_w_error_reset <= '0';
 				e1_t_counter_en <= '1';
				e1_t_counter_reset <= '0';
      			e1_w_counter_en <= '0';
     	 			e1_w_counter_reset <= '1';
	      		e1_next_state <= EDGE0;
			else
	    			e1_t_error_en <= '0';
				e1_t_error_reset <= '0';
				e1_w_error_en <= '1';
				e1_w_error_reset <= '0';
	      		e1_next_state <= EDGE_ERROR;
 				e1_t_counter_en <= '0';
				e1_t_counter_reset <= '1';
      			e1_w_counter_en <= '0';
     	 			e1_w_counter_reset <= '1';
			end if;
    		else
	    		e1_t_error_en <= '0';
			e1_t_error_reset <= '0';
 		    	e1_w_error_en <= '0';
			e1_w_error_reset <= '0';
 			e1_t_counter_en <= '1';
			e1_t_counter_reset <= '0';
			e1_w_counter_en <= '1';
			e1_w_counter_reset <= '0';
      		e1_next_state <= EDGE1;
    		end if;

	when EDGE0 => 

		if (e1_in = '1') then
      		if (e1_t_counter_out = e1_period) then
			  	e1_t_error_en <= '0';
				e1_t_error_reset <= '0';
 				e1_w_error_en <= '0';
				e1_w_error_reset <= '0';
 		  		e1_t_counter_en <= '0';
		  		e1_t_counter_reset <= '1';
 				e1_w_counter_en <= '0';
				e1_w_counter_reset <= '1';
     				e1_next_state <= EDGE1;
			else
				e1_t_error_en <= '1';
				e1_t_error_reset <= '0';
 				e1_w_error_en <= '0';
				e1_w_error_reset <= '0';
 		  		e1_t_counter_en <= '0';
		  		e1_t_counter_reset <= '1';
 				e1_w_counter_en <= '0';
				e1_w_counter_reset <= '1';
      			e1_next_state <= EDGE_ERROR;
			end if;
    		else
      		if (e1_t_counter_out = e1_period) then
 			  	e1_t_error_en <= '1';
				e1_t_error_reset <= '0';
 				e1_w_error_en <= '0';
				e1_w_error_reset <= '0';
 				e1_t_counter_en <= '0';
				e1_t_counter_reset <= '1';
 				e1_w_counter_en <= '0';
				e1_w_counter_reset <= '1';
	      		e1_next_state <= EDGE_ERROR;
			else
 			  	e1_t_error_en <= '0';
 				e1_t_error_reset <= '0';
 				e1_w_error_en <= '0';
				e1_w_error_reset <= '0';
				e1_t_counter_en <= '1';
				e1_t_counter_reset <= '0';
 				e1_w_counter_en <= '0';
				e1_w_counter_reset <= '1';
      			e1_next_state <= EDGE0;
			end if;
    		end if;

	when EDGE_ERROR => 

		if (reset = '1') or (enable = '0') then
			e1_t_error_en <= '0';
			e1_t_error_reset <= '1';
			e1_w_error_en <= '0';
			e1_w_error_reset <= '1';
 			e1_w_counter_en <= '0';
			e1_w_counter_reset <= '1';
			e1_t_counter_en <= '0';
			e1_t_counter_reset <= '1';
	       	e1_next_state <= IDLE;
   		else
			e1_t_error_en <= '0';
			e1_t_error_reset <= '0';
			e1_w_error_en <= '0';
			e1_w_error_reset <= '0';
 			e1_w_counter_en <= '0';
			e1_w_counter_reset <= '1';
			e1_t_counter_en <= '0';
			e1_t_counter_reset <= '1';
      		e1_next_state <= EDGE_ERROR;
   		end if;

	when others =>

		e1_t_error_en <= '0';
		e1_t_error_reset <= '1';
		e1_w_error_en <= '0';
		e1_w_error_reset <= '1';
 		e1_t_counter_en <= '0';
		e1_t_counter_reset <= '1';
 		e1_w_counter_en <= '0';
		e1_w_counter_reset <= '1';
		e1_next_state <= IDLE;

end case;
 
end process;

-- E2
 
e2_error_regs : process(clk, reset, e2_t_error_en, e2_t_error_reset, e2_w_error_en, e2_w_error_reset)

begin
      
	if (reset = '1') then
		e2_t_error <= '0';
  	elsif rising_edge(clk) then
		if (e2_t_error_reset = '1') then
       		e2_t_error <= '0';
		elsif (e2_t_error_en = '1') then
       		e2_t_error <= '1';
		end if;
	end if;

	if (reset = '1') then
       	e2_w_error <= '0';
  	elsif rising_edge(clk) then
		if (e2_w_error_reset = '1') then
       		e2_w_error <= '0';
		elsif (e2_w_error_en = '1') then
       		e2_w_error <= '1';
		end if;
	end if;

	e2_error <= e2_t_error or e2_w_error;

end process;

e2_fsm_regs : process(clk, e2_next_state)

begin
      
	if (reset = '1') then
       	e2_current_state <= IDLE;
  	elsif rising_edge(clk) then
       	e2_current_state <= e2_next_state;
	end if;

end process;

e2_t_counter: process (clk, e2_t_counter_en, reset, e2_t_counter_reset)

variable e2_t_counter_data : std_logic_vector(15 downto 0);

begin

	if (reset = '1') then
		e2_t_counter_data := "0000000000000000";
	elsif (rising_edge(clk)) then
		if (e2_t_counter_reset = '1') then
			e2_t_counter_data := "0000000000000000";
		elsif (e2_t_counter_en = '1') then
			e2_t_counter_data := e2_t_counter_data + 1;
		end if;              
	end if; 
	e2_t_counter_out <= e2_t_counter_data;

end process;

e2_w_counter: process (clk, e2_w_counter_en, reset, e2_w_counter_reset)

variable e2_w_counter_data : std_logic_vector(15 downto 0);

begin

	if (reset = '1') then
		e2_w_counter_data := "0000000000000000";
	elsif (rising_edge(clk)) then
		if (e2_w_counter_reset = '1') then
			e2_w_counter_data := "0000000000000000";
		elsif (e2_w_counter_en = '1') then
			e2_w_counter_data := e2_w_counter_data + 1;
		end if;              
	end if; 
	e2_w_counter_out <= e2_w_counter_data;

end process;
  
  
e2_fsm_logic : process(clk, reset, e2_w_counter_out, e2_t_counter_out, e2_current_state, enable, e2_in)

begin

case e2_current_state is

	when IDLE =>

		if (enable = '1') then
 			if ((e2_t_counter_out = e2_period) and (e2_in = '0')) then
				e2_t_error_en <= '1';
				e2_t_error_reset <= '0';
				e2_w_error_en <= '0';
				e2_w_error_reset <= '0';
				e2_t_counter_en <= '0';
				e2_t_counter_reset <= '1';
				e2_w_counter_en <= '0';
				e2_w_counter_reset <= '1';
				e2_next_state <= EDGE_ERROR;
			elsif (e2_in = '1') then
				e2_t_error_en <= '0';
				e2_t_error_reset <= '0';
				e2_w_error_en <= '0';
				e2_w_error_reset <= '0';
				e2_t_counter_en <= '0';
				e2_t_counter_reset <= '1';
				e2_w_counter_en <= '0';
				e2_w_counter_reset <= '1';
	    			e2_next_state <= EDGE1;
			else
				e2_t_error_en <= '0';
				e2_t_error_reset <= '0';
				e2_w_error_en <= '0';
				e2_w_error_reset <= '0';
				e2_t_counter_en <= '1';
				e2_t_counter_reset <= '0';
				e2_w_counter_en <= '0';
				e2_w_counter_reset <= '1';
				e2_next_state <= IDLE;
			end if;
		else
			e2_t_error_en <= '0';
			e2_t_error_reset <= '1';
			e2_w_error_en <= '0';
			e2_w_error_reset <= '1';
			e2_t_counter_en <= '0';
			e2_t_counter_reset <= '1';
 			e2_w_counter_en <= '0';
			e2_w_counter_reset <= '1';
			e2_next_state <= IDLE;
		end if;

	when EDGE1 =>

		if (e2_in = '0') then
		  	if (e2_w_counter_out = e1_width) then
	    			e2_t_error_en <= '0';
				e2_t_error_reset <= '0';
	    			e2_w_error_en <= '0';
				e2_w_error_reset <= '0';
 				e2_t_counter_en <= '1';
				e2_t_counter_reset <= '0';
      			e2_w_counter_en <= '0';
     	 			e2_w_counter_reset <= '1';
	      		e2_next_state <= EDGE0;
			else
	    			e2_t_error_en <= '0';
				e2_t_error_reset <= '0';
				e2_w_error_en <= '1';
				e2_w_error_reset <= '0';
	      		e2_next_state <= EDGE_ERROR;
 				e2_t_counter_en <= '0';
				e2_t_counter_reset <= '1';
      			e2_w_counter_en <= '0';
     	 			e2_w_counter_reset <= '1';
			end if;
    		else
	    		e2_t_error_en <= '0';
			e2_t_error_reset <= '0';
 		    	e2_w_error_en <= '0';
			e2_w_error_reset <= '0';
 			e2_t_counter_en <= '1';
			e2_t_counter_reset <= '0';
			e2_w_counter_en <= '1';
			e2_w_counter_reset <= '0';
      		e2_next_state <= EDGE1;
    		end if;

	when EDGE0 => 

		if (e2_in = '1') then
      		if (e2_t_counter_out = e2_period) then
			  	e2_t_error_en <= '0';
				e2_t_error_reset <= '0';
 				e2_w_error_en <= '0';
				e2_w_error_reset <= '0';
 		  		e2_t_counter_en <= '0';
		  		e2_t_counter_reset <= '1';
 				e2_w_counter_en <= '0';
				e2_w_counter_reset <= '1';
     				e2_next_state <= EDGE1;
			else
				e2_t_error_en <= '1';
				e2_t_error_reset <= '0';
 				e2_w_error_en <= '0';
				e2_w_error_reset <= '0';
 		  		e2_t_counter_en <= '0';
		  		e2_t_counter_reset <= '1';
 				e2_w_counter_en <= '0';
				e2_w_counter_reset <= '1';
      			e2_next_state <= EDGE_ERROR;
			end if;
    		else
      		if (e2_t_counter_out = e2_period) then
 			  	e2_t_error_en <= '1';
				e2_t_error_reset <= '0';
 				e2_w_error_en <= '0';
				e2_w_error_reset <= '0';
 				e2_t_counter_en <= '0';
				e2_t_counter_reset <= '1';
 				e2_w_counter_en <= '0';
				e2_w_counter_reset <= '1';
	      		e2_next_state <= EDGE_ERROR;
			else
 			  	e2_t_error_en <= '0';
 				e2_t_error_reset <= '0';
 				e2_w_error_en <= '0';
				e2_w_error_reset <= '0';
				e2_t_counter_en <= '1';
				e2_t_counter_reset <= '0';
 				e2_w_counter_en <= '0';
				e2_w_counter_reset <= '1';
      			e2_next_state <= EDGE0;
			end if;
    		end if;

	when EDGE_ERROR => 

		if (reset = '1') or (enable = '0') then
			e2_t_error_en <= '0';
			e2_t_error_reset <= '1';
			e2_w_error_en <= '0';
			e2_w_error_reset <= '1';
 			e2_w_counter_en <= '0';
			e2_w_counter_reset <= '1';
			e2_t_counter_en <= '0';
			e2_t_counter_reset <= '1';
	       	e2_next_state <= IDLE;
   		else
			e2_t_error_en <= '0';
			e2_t_error_reset <= '0';
			e2_w_error_en <= '0';
			e2_w_error_reset <= '0';
 			e2_w_counter_en <= '0';
			e2_w_counter_reset <= '1';
			e2_t_counter_en <= '0';
			e2_t_counter_reset <= '1';
      		e2_next_state <= EDGE_ERROR;
   		end if;

	when others =>

		e2_t_error_en <= '0';
		e2_t_error_reset <= '1';
		e2_w_error_en <= '0';
		e2_w_error_reset <= '1';
 		e2_t_counter_en <= '0';
		e2_t_counter_reset <= '1';
 		e2_w_counter_en <= '0';
		e2_w_counter_reset <= '1';
		e2_next_state <= IDLE;

end case;
 
end process;

error <= p1_error or p2_error or e1_error or e2_error;

end flf_rx_arch;