--Token Transmitter Code

--Import Libraries
LIBRARY ieee;
USE IEEE.STD_LOGIC_UNSIGNED.all;
USE ieee.std_logic_1164.all;
USE ieee.numeric_std.all;

--Entity Declaration
entity flf_tx is 
port
	(clk: IN STD_LOGIC;
	reset: IN STD_LOGIC;
	enable: IN STD_LOGIC;
	p1_out : OUT STD_LOGIC;
	p2_out : OUT STD_LOGIC;
	e1_out : OUT STD_LOGIC;
	e2_out : OUT STD_LOGIC);

end flf_tx;

ARCHITECTURE flf_tx_arch of flf_tx is
  
type p_state_type is (IDLE, START, PULSE1, PULSE0);
signal p1_current_state, p1_next_state : p_state_type;
signal p2_current_state, p2_next_state : p_state_type;
signal p1_t_counter_en, p1_t_counter_reset : STD_LOGIC;
signal p1_t_counter_out : STD_LOGIC_VECTOR(15 downto 0);
signal p2_t_counter_en, p2_t_counter_reset : STD_LOGIC;
signal p2_t_counter_out : STD_LOGIC_VECTOR(15 downto 0);
signal p1_o_counter_en, p1_o_counter_reset : STD_LOGIC;
signal p1_o_counter_out : STD_LOGIC_VECTOR(15 downto 0);
signal p2_o_counter_en, p2_o_counter_reset : STD_LOGIC;
signal p2_o_counter_out : STD_LOGIC_VECTOR(15 downto 0);

type e_state_type is (IDLE, START, EDGE1, EDGE0);
signal e1_current_state, e1_next_state : e_state_type;
signal e2_current_state, e2_next_state : e_state_type;
signal e1_t_counter_en, e1_t_counter_reset : STD_LOGIC;
signal e1_t_counter_out : STD_LOGIC_VECTOR(15 downto 0);
signal e2_t_counter_en, e2_t_counter_reset : STD_LOGIC;
signal e2_t_counter_out : STD_LOGIC_VECTOR(15 downto 0);
signal e1_w_counter_en, e1_w_counter_reset : STD_LOGIC;
signal e1_w_counter_out : STD_LOGIC_VECTOR(15 downto 0);
signal e2_w_counter_en, e2_w_counter_reset : STD_LOGIC;
signal e2_w_counter_out : STD_LOGIC_VECTOR(15 downto 0);
signal e1_o_counter_en, e1_o_counter_reset : STD_LOGIC;
signal e1_o_counter_out : STD_LOGIC_VECTOR(15 downto 0);
signal e2_o_counter_en, e2_o_counter_reset : STD_LOGIC;
signal e2_o_counter_out : STD_LOGIC_VECTOR(15 downto 0);

constant p1_offset: STD_LOGIC_VECTOR(15 downto 0) := "0000000000010000";
constant p1_period: STD_LOGIC_VECTOR(15 downto 0) := "0000000100000000";

constant p2_offset: STD_LOGIC_VECTOR(15 downto 0) := "0000000000100000";
constant p2_period: STD_LOGIC_VECTOR(15 downto 0) := "0000000100000000";

constant e1_offset: STD_LOGIC_VECTOR(15 downto 0) := "0000000001000000";
constant e1_width: STD_LOGIC_VECTOR(15 downto 0) := "0000000000010000";
constant e1_period: STD_LOGIC_VECTOR(15 downto 0) := "0000000100000000";

constant e2_offset: STD_LOGIC_VECTOR(15 downto 0) := "0000000010000000";
constant e2_width: STD_LOGIC_VECTOR(15 downto 0) := "0000000000010000";
constant e2_period: STD_LOGIC_VECTOR(15 downto 0) := "0000000100000000";
  
BEGIN

-- P1
  
p1_fsm_regs : process(clk)

begin
      
	if (reset = '1') then
       	p1_current_state <= IDLE;
  	elsif rising_edge(clk) then
        	p1_current_state <= p1_next_state;
	end if;

end process;

p1_o_counter: process (clk, p1_o_counter_en, reset, p1_o_counter_reset)

variable p1_o_counter_data : std_logic_vector(15 downto 0);

begin

	if (reset = '1') then
		p1_o_counter_data := "0000000000000000";
	elsif (rising_edge(clk)) then
		if (p1_o_counter_reset = '1') then
			p1_o_counter_data := "0000000000000000";
    		elsif (p1_o_counter_en = '1') then
			p1_o_counter_data := p1_o_counter_data + 1;
		end if;              
	end if; 
	p1_o_counter_out <= p1_o_counter_data;

end process;

p1_t_counter: process (clk, p1_t_counter_en, reset, e1_t_counter_reset)

variable p1_t_counter_data : std_logic_vector(15 downto 0);

begin

	if (reset = '1') then
		p1_t_counter_data := "0000000000000000";
	elsif (rising_edge(clk)) then
		if (p1_t_counter_reset = '1') then
			p1_t_counter_data := "0000000000000000";
		elsif (p1_t_counter_en = '1') then
			p1_t_counter_data := p1_t_counter_data + 1;
		end if;              
	end if; 
	p1_t_counter_out <= p1_t_counter_data;

end process;

p1_fsm_logic : process(clk, reset, p1_o_counter_out, p1_t_counter_out, p1_current_state, enable)

begin

	case p1_current_state is

	when IDLE =>

    		p1_out <= '0';
 		p1_t_counter_en <= '0';
		p1_t_counter_reset <= '1';
      	p1_o_counter_en <= '0';
      	p1_o_counter_reset <= '1';
    		if (enable = '1') then
      		p1_next_state <= START;
    		else
      		p1_next_state <= IDLE;
		end if;
        
	when START =>

    		p1_out <= '0';
 		p1_t_counter_en <= '0';
		p1_t_counter_reset <= '1';
    		if (p1_o_counter_out = p1_offset) then
 	    		p1_o_counter_en <= '0';
 	    		p1_o_counter_reset <= '1';
   	  		p1_next_state <= PULSE1;
    		else
 	    		p1_o_counter_en <= '1';
  	    		p1_o_counter_reset <= '0';
  	  		p1_next_state <= START;
		end if;
        
  	when PULSE1 => 

		p1_out <= '1';
 		p1_t_counter_en <= '1';
		p1_t_counter_reset <= '0';
 		p1_o_counter_en <= '0';
		p1_o_counter_reset <= '1';
      	p1_next_state <= PULSE0;

	when PULSE0 => 

		p1_out <= '0';
 		p1_o_counter_en <= '0';
		p1_o_counter_reset <= '1';
		if (p1_t_counter_out = p1_period) then
 			p1_t_counter_en <= '0';
			p1_t_counter_reset <= '1';
     			p1_next_state <= PULSE1;
   		else
 			p1_t_counter_en <= '1';
			p1_t_counter_reset <= '0';
      		p1_next_state <= PULSE0;
   		end if;

	when others =>

    		p1_out <= '0';
    		p1_t_counter_en <= '0';
    		p1_t_counter_reset <= '1';
     		p1_o_counter_en <= '0';
    		p1_o_counter_reset <= '1';
   		p1_next_state <= IDLE;

	end case;

end process;

-- P2
  
p2_fsm_regs : process(clk)

begin
      
	if (reset = '1') then
       	p2_current_state <= IDLE;
  	elsif rising_edge(clk) then
        	p2_current_state <= p2_next_state;
	end if;

end process;

p2_o_counter: process (clk, p2_o_counter_en, reset, p2_o_counter_reset)

variable p2_o_counter_data : std_logic_vector(15 downto 0);

begin

	if (reset = '1') then
		p2_o_counter_data := "0000000000000000";
	elsif (rising_edge(clk)) then
		if (p2_o_counter_reset = '1') then
			p2_o_counter_data := "0000000000000000";
    		elsif (p2_o_counter_en = '1') then
			p2_o_counter_data := p2_o_counter_data + 1;
		end if;              
	end if; 
	p2_o_counter_out <= p2_o_counter_data;

end process;

p2_t_counter: process (clk, p2_t_counter_en, reset, e1_t_counter_reset)

variable p2_t_counter_data : std_logic_vector(15 downto 0);

begin

	if (reset = '1') then
		p2_t_counter_data := "0000000000000000";
	elsif (rising_edge(clk)) then
		if (p2_t_counter_reset = '1') then
			p2_t_counter_data := "0000000000000000";
		elsif (p2_t_counter_en = '1') then
			p2_t_counter_data := p2_t_counter_data + 1;
		end if;              
	end if; 
	p2_t_counter_out <= p2_t_counter_data;

end process;

p2_fsm_logic : process(clk, reset, p2_o_counter_out, p2_t_counter_out, p2_current_state, enable)

begin

	case p2_current_state is

	when IDLE =>

    		p2_out <= '0';
 		p2_t_counter_en <= '0';
		p2_t_counter_reset <= '1';
      	p2_o_counter_en <= '0';
      	p2_o_counter_reset <= '1';
    		if (enable = '1') then
      		p2_next_state <= START;
    		else
      		p2_next_state <= IDLE;
		end if;
        
	when START =>

    		p2_out <= '0';
 		p2_t_counter_en <= '0';
		p2_t_counter_reset <= '1';
    		if (p2_o_counter_out = p2_offset) then
 	    		p2_o_counter_en <= '0';
 	    		p2_o_counter_reset <= '1';
   	  		p2_next_state <= PULSE1;
    		else
 	    		p2_o_counter_en <= '1';
  	    		p2_o_counter_reset <= '0';
  	  		p2_next_state <= START;
		end if;
        
  	when PULSE1 => 

		p2_out <= '1';
 		p2_t_counter_en <= '1';
		p2_t_counter_reset <= '0';
 		p2_o_counter_en <= '0';
		p2_o_counter_reset <= '1';
      	p2_next_state <= PULSE0;

	when PULSE0 => 

		p2_out <= '0';
 		p2_o_counter_en <= '0';
		p2_o_counter_reset <= '1';
		if (p2_t_counter_out = p2_period) then
 			p2_t_counter_en <= '0';
			p2_t_counter_reset <= '1';
     			p2_next_state <= PULSE1;
   		else
 			p2_t_counter_en <= '1';
			p2_t_counter_reset <= '0';
      		p2_next_state <= PULSE0;
   		end if;

	when others =>

    		p2_out <= '0';
    		p2_t_counter_en <= '0';
    		p2_t_counter_reset <= '1';
     		p2_o_counter_en <= '0';
    		p2_o_counter_reset <= '1';
   		p2_next_state <= IDLE;

	end case;

end process;

-- E1
  
e1_fsm_regs : process(clk)

begin
      
	if (reset = '1') then
       	e1_current_state <= IDLE;
  	elsif rising_edge(clk) then
        	e1_current_state <= e1_next_state;
	end if;

end process;

e1_o_counter: process (clk, e1_o_counter_en, reset, e1_o_counter_reset)

variable e1_o_counter_data : std_logic_vector(15 downto 0);

begin

	if (reset = '1') then
		e1_o_counter_data := "0000000000000000";
	elsif (rising_edge(clk)) then
		if (e1_o_counter_reset = '1') then
			e1_o_counter_data := "0000000000000000";
    		elsif (e1_o_counter_en = '1') then
			e1_o_counter_data := e1_o_counter_data + 1;
		end if;              
	end if; 
	e1_o_counter_out <= e1_o_counter_data;

end process;

e1_t_counter: process (clk, e1_t_counter_en, reset, e1_t_counter_reset)

variable e1_t_counter_data : std_logic_vector(15 downto 0);

begin

	if (reset = '1') then
		e1_t_counter_data := "0000000000000000";
	elsif (rising_edge(clk)) then
		if (e1_t_counter_reset = '1') then
			e1_t_counter_data := "0000000000000000";
		elsif (e1_t_counter_en = '1') then
			e1_t_counter_data := e1_t_counter_data + 1;
		end if;              
	end if; 
	e1_t_counter_out <= e1_t_counter_data;

end process;

e1_w_counter: process (clk, e1_w_counter_en, reset, e1_w_counter_reset)

variable e1_w_counter_data : std_logic_vector(15 downto 0);

begin

	if (reset = '1') then
		e1_w_counter_data := "0000000000000000";
	elsif (rising_edge(clk)) then
		if (e1_w_counter_reset = '1') then
			e1_w_counter_data := "0000000000000000";
		elsif (e1_w_counter_en = '1') then
			e1_w_counter_data := e1_w_counter_data + 1;
		end if;              
	end if; 
	e1_w_counter_out <= e1_w_counter_data;

end process;
 
e1_fsm_logic : process(clk, reset, e1_o_counter_out, e1_t_counter_out, e1_w_counter_out, e1_current_state, enable)

begin

	case e1_current_state is

	when IDLE =>

    		e1_out <= '0';
 		e1_t_counter_en <= '0';
		e1_t_counter_reset <= '1';
 		e1_w_counter_en <= '0';
		e1_w_counter_reset <= '1';
      	e1_o_counter_en <= '0';
      	e1_o_counter_reset <= '1';
    		if (enable = '1') then
      		e1_next_state <= START;
    		else
      		e1_next_state <= IDLE;
		end if;
        
	when START =>

    		e1_out <= '0';
 		e1_t_counter_en <= '0';
		e1_t_counter_reset <= '1';
 		e1_w_counter_en <= '0';
		e1_w_counter_reset <= '1';
    		if (e1_o_counter_out = e1_offset) then
 	    		e1_o_counter_en <= '0';
 	    		e1_o_counter_reset <= '1';
   	  		e1_next_state <= EDGE1;
    		else
 	    		e1_o_counter_en <= '1';
  	    		e1_o_counter_reset <= '0';
  	  		e1_next_state <= START;
		end if;
        
  when EDGE1 => 

		e1_out <= '1';
 		e1_t_counter_en <= '1';
		e1_t_counter_reset <= '0';
 		e1_o_counter_en <= '0';
		e1_o_counter_reset <= '1';
 	  	if (e1_w_counter_out = e1_width) then
		  	e1_w_counter_en <= '0';
			e1_w_counter_reset <= '1';
      		e1_next_state <= EDGE0;
    		else
			e1_w_counter_en <= '1';
			e1_w_counter_reset <= '0';
     			e1_next_state <= EDGE1;
   		end if;

	when EDGE0 => 

		e1_out <= '0';
    		e1_w_counter_en <= '0';
		e1_w_counter_reset <= '1';
 		e1_o_counter_en <= '0';
		e1_o_counter_reset <= '1';
		if (e1_t_counter_out = e1_period) then
 			e1_t_counter_en <= '0';
			e1_t_counter_reset <= '1';
     			e1_next_state <= EDGE1;
   		else
 			e1_t_counter_en <= '1';
			e1_t_counter_reset <= '0';
      		e1_next_state <=EDGE0;
   		end if;

	when others =>

    		e1_out <= '0';
    		e1_t_counter_en <= '0';
    		e1_t_counter_reset <= '1';
    		e1_w_counter_en <= '0';
    		e1_w_counter_reset <= '1';
     		e1_o_counter_en <= '0';
    		e1_o_counter_reset <= '1';
   		e1_next_state <= IDLE;

	end case;

end process;

-- E2

e2_fsm_regs : process(clk)

begin
      
	if (reset = '1') then
       	e2_current_state <= IDLE;
  	elsif rising_edge(clk) then
        	e2_current_state <= e2_next_state;
	end if;

end process;

e2_o_counter: process (clk, e2_o_counter_en, reset, e2_o_counter_reset)

variable e2_o_counter_data : std_logic_vector(15 downto 0);

begin

	if (reset = '1') then
		e2_o_counter_data := "0000000000000000";
	elsif (rising_edge(clk)) then
		if (e2_o_counter_reset = '1') then
			e2_o_counter_data := "0000000000000000";
    		elsif (e2_o_counter_en = '1') then
			e2_o_counter_data := e2_o_counter_data + 1;
		end if;              
	end if; 
	e2_o_counter_out <= e2_o_counter_data;

end process;

e2_t_counter: process (clk, e2_t_counter_en, reset, e2_t_counter_reset)

variable e2_t_counter_data : std_logic_vector(15 downto 0);

begin

	if (reset = '1') then
		e2_t_counter_data := "0000000000000000";
	elsif (rising_edge(clk)) then
		if (e2_t_counter_reset = '1') then
			e2_t_counter_data := "0000000000000000";
		elsif (e2_t_counter_en = '1') then
			e2_t_counter_data := e2_t_counter_data + 1;
		end if;              
	end if; 
	e2_t_counter_out <= e2_t_counter_data;

end process;

e2_w_counter: process (clk, e2_w_counter_en, reset, e2_w_counter_reset)

variable e2_w_counter_data : std_logic_vector(15 downto 0);

begin

	if (reset = '1') then
		e2_w_counter_data := "0000000000000000";
	elsif (rising_edge(clk)) then
		if (e2_w_counter_reset = '1') then
			e2_w_counter_data := "0000000000000000";
		elsif (e2_w_counter_en = '1') then
			e2_w_counter_data := e2_w_counter_data + 1;
		end if;              
	end if; 
	e2_w_counter_out <= e2_w_counter_data;

end process;
 
e2_fsm_logic : process(clk, reset, e2_o_counter_out, e2_t_counter_out, e2_w_counter_out, e2_current_state, enable)

begin

	case e2_current_state is

	when IDLE =>

    		e2_out <= '0';
 		e2_t_counter_en <= '0';
		e2_t_counter_reset <= '1';
 		e2_w_counter_en <= '0';
		e2_w_counter_reset <= '1';
      	e2_o_counter_en <= '0';
      	e2_o_counter_reset <= '1';
    		if (enable = '1') then
      		e2_next_state <= START;
    		else
      		e2_next_state <= IDLE;
		end if;
        
	when START =>

    		e2_out <= '0';
 		e2_t_counter_en <= '0';
		e2_t_counter_reset <= '1';
 		e2_w_counter_en <= '0';
		e2_w_counter_reset <= '1';
    		if (e2_o_counter_out = e2_offset) then
 	    		e2_o_counter_en <= '0';
 	    		e2_o_counter_reset <= '1';
   	  		e2_next_state <= EDGE1;
    		else
 	    		e2_o_counter_en <= '1';
  	    		e2_o_counter_reset <= '0';
  	  		e2_next_state <= START;
		end if;
        
  when EDGE1 => 

		e2_out <= '1';
 		e2_t_counter_en <= '1';
		e2_t_counter_reset <= '0';
 		e2_o_counter_en <= '0';
		e2_o_counter_reset <= '1';
 	  	if (e2_w_counter_out = e2_width) then
		  	e2_w_counter_en <= '0';
			e2_w_counter_reset <= '1';
      		e2_next_state <= EDGE0;
    		else
			e2_w_counter_en <= '1';
			e2_w_counter_reset <= '0';
     			e2_next_state <= EDGE1;
   		end if;

	when EDGE0 => 

		e2_out <= '0';
    		e2_w_counter_en <= '0';
		e2_w_counter_reset <= '1';
 		e2_o_counter_en <= '0';
		e2_o_counter_reset <= '1';
		if (e2_t_counter_out = e2_period) then
 			e2_t_counter_en <= '0';
			e2_t_counter_reset <= '1';
     			e2_next_state <= EDGE1;
   		else
 			e2_t_counter_en <= '1';
			e2_t_counter_reset <= '0';
      		e2_next_state <=EDGE0;
   		end if;

	when others =>

    		e2_out <= '0';
    		e2_t_counter_en <= '0';
    		e2_t_counter_reset <= '1';
    		e2_w_counter_en <= '0';
    		e2_w_counter_reset <= '1';
     		e2_o_counter_en <= '0';
    		e2_o_counter_reset <= '1';
   		e2_next_state <= IDLE;

	end case;

end process;

end flf_tx_arch;