--Token Receiver Code

--Import Libraries
LIBRARY ieee;
USE IEEE.STD_LOGIC_UNSIGNED.all;
USE ieee.std_logic_1164.all;
USE ieee.numeric_std.all;

--Entity Declaration
entity tkn_rx is 
port
	(clk: IN STD_LOGIC;
   	 reset: IN STD_LOGIC;
   	 enable: IN STD_LOGIC;
   	 error: OUT STD_LOGIC;
   	 token_in : IN STD_LOGIC);
end tkn_rx;

ARCHITECTURE tkn_rx_arch of tkn_rx is

type state_type is (IDLE, TKN1, TKN0, TKN_ERROR);
signal current_state, next_state : state_type;
signal t_counter_en, t_counter_reset : STD_LOGIC;
signal t_counter_out : STD_LOGIC_VECTOR(15 downto 0);
signal t_error, t_error_en, t_error_reset : STD_LOGIC;
signal w_counter_en, w_counter_reset : STD_LOGIC;
signal w_counter_out : STD_LOGIC_VECTOR(15 downto 0);
signal w_error, w_error_en, w_error_reset : STD_LOGIC;

constant tkn_width_max: STD_LOGIC_VECTOR(15 downto 0) := "0000001000000000";
constant tkn_period_max: STD_LOGIC_VECTOR(15 downto 0) := "0001000000000000";
  
BEGIN
  
error_regs : process(clk, reset, t_error_en, t_error_reset, w_error_en, w_error_reset)

begin
      
	if (reset = '1') then
       	t_error <= '0';
  	elsif rising_edge(clk) then
		if (t_error_reset = '1') then
       		t_error <= '0';
		elsif (t_error_en = '1') then
       		t_error <= '1';
		end if;
	end if;

	if (reset = '1') then
       	w_error <= '0';
  	elsif rising_edge(clk) then
		if (w_error_reset = '1') then
       		w_error <= '0';
		elsif (w_error_en = '1') then
       		w_error <= '1';
		end if;
	end if;

	error <= t_error or w_error;

end process;

fsm_regs : process(clk)

begin
      
	if (reset = '1') then
       	current_state <= IDLE;
  	elsif rising_edge(clk) then
        	current_state <= next_state;
	end if;

end process;

t_counter: process (clk, t_counter_en, reset, t_counter_reset)

variable t_counter_data : std_logic_vector(15 downto 0);

begin

	if (reset = '1') then
		t_counter_data := "0000000000000000";
	elsif (rising_edge(clk)) then
		if (t_counter_reset = '1') then
			t_counter_data := "0000000000000000";
		elsif (t_counter_en = '1') then
			t_counter_data := t_counter_data + 1;
		end if;              
	end if; 
	t_counter_out <= t_counter_data;

end process;

w_counter: process (clk, w_counter_en, reset, w_counter_reset)

variable w_counter_data : std_logic_vector(15 downto 0);

begin

	if (reset = '1') then
		w_counter_data := "0000000000000000";
	elsif (rising_edge(clk)) then
		if (w_counter_reset = '1') then
			w_counter_data := "0000000000000000";
		elsif (w_counter_en = '1') then
			w_counter_data := w_counter_data + 1;
		end if;              
	end if; 
	w_counter_out <= w_counter_data;

end process;
  
fsm_logic : process(clk, reset, t_counter_out, w_counter_out, current_state, enable, token_in)

begin

case current_state is

	when IDLE =>

		t_error_en <= '0';
		t_error_reset <= '1';
		w_error_en <= '0';
		w_error_reset <= '1';
 		w_counter_en <= '0';
		w_counter_reset <= '1';
		if (enable = '1') then
 			if (t_counter_out = tkn_period_max) then
				t_error_en <= '1';
				t_counter_en <= '0';
				t_counter_reset <= '1';
	    			next_state <= TKN_ERROR;
			elsif (token_in = '1') then
				t_error_en <= '0';
				t_counter_en <= '1';
				t_counter_reset <= '0';
				next_state <= TKN1;
			else
				t_error_en <= '0';
				t_counter_en <= '1';
				t_counter_reset <= '0';
				next_state <= IDLE;
			end if;
		else
			t_error_en <= '0';
			t_counter_en <= '0';
			t_counter_reset <= '1';
			next_state <= IDLE;
		end if;

	when TKN1 =>

		t_error_en <= '0';
		t_error_reset <= '0';
		w_error_reset <= '0';
 		t_counter_en <= '1';
		t_counter_reset <= '0';
		if (w_counter_out = tkn_width_max) then
			w_error_en <= '1';
			w_counter_en <= '0';
			w_counter_reset <= '1';
	         	next_state <= TKN_ERROR;
		elsif (token_in = '0') then
			w_error_en <= '0';
			w_counter_en <= '0';
			w_counter_reset <= '1';
          		next_state <= TKN0;
 	  	else
 			w_error_en <= '0';
			w_counter_en <= '1';
			w_counter_reset <= '0';
         		next_state <= TKN1;
        	end if;

	when TKN0 => 

		t_error_reset <= '0';
		w_error_en <= '0';
		w_error_reset <= '0';
 		w_counter_en <= '0';
		w_counter_reset <= '1';
		if (t_counter_out = tkn_period_max) then
			t_error_en <= '1';
			t_counter_en <= '0';
			t_counter_reset <= '1';
	    next_state <= TKN_ERROR;
		elsif (token_in = '1') then
 			t_error_en <= '0';
			t_counter_en <= '0';
			t_counter_reset <= '1';
      next_state <= TKN1;
   	else
 			t_error_en <= '0';
			t_counter_en <= '1';
			t_counter_reset <= '0';
      next_state <= TKN0;
   	end if;

	when TKN_ERROR => 

		t_error_en <= '0';
		t_error_reset <= '0';
		w_error_en <= '0';
		w_error_reset <= '0';
 		w_counter_en <= '0';
		w_counter_reset <= '1';
		t_counter_en <= '0';
		t_counter_reset <= '1';
		  if (reset = '1') then
	       next_state <= IDLE;
   		else
      		next_state <= TKN_ERROR;
   		end if;

	when others =>

		t_error_en <= '0';
		t_error_reset <= '0';
		w_error_en <= '0';
		w_error_reset <= '0';
 		t_counter_en <= '0';
		t_counter_reset <= '1';
 		w_counter_en <= '0';
		w_counter_reset <= '1';
		next_state <= IDLE;
 
end case;

end process;

end tkn_rx_arch;