--Token Transmitter Code

--Import Libraries
LIBRARY ieee;
USE IEEE.STD_LOGIC_UNSIGNED.all;
USE ieee.std_logic_1164.all;
USE ieee.numeric_std.all;

--Entity Declaration
entity tkn_tx is 
port
	(clk: IN STD_LOGIC;
   	 reset: IN STD_LOGIC;
   	 enable: IN STD_LOGIC;
   	 token_out : OUT STD_LOGIC);
end tkn_tx;

ARCHITECTURE tkn_tx_arch of tkn_tx is
  
type state_type is (IDLE, TKN1, TKN0);
signal current_state, next_state : state_type;
signal t_counter_en, t_counter_reset : STD_LOGIC;
signal t_counter_out : STD_LOGIC_VECTOR(15 downto 0);
signal w_counter_en, w_counter_reset : STD_LOGIC;
signal w_counter_out : STD_LOGIC_VECTOR(15 downto 0);

constant tkn_width: STD_LOGIC_VECTOR(15 downto 0) := "0000000000100000";
constant tkn_period: STD_LOGIC_VECTOR(15 downto 0) := "0000010000000000";
  
BEGIN
  
fsm_regs : process(clk)

begin
      
	if (reset = '1') then
       	current_state <= IDLE;
  	elsif rising_edge(clk) then
        	current_state <= next_state;
	end if;

end process;

t_counter: process (clk, t_counter_en, reset, t_counter_reset)

variable t_counter_data : std_logic_vector(15 downto 0);

begin

	if (reset = '1') then
		t_counter_data := "0000000000000000";
	elsif (rising_edge(clk)) then
		if (t_counter_reset = '1') then
			t_counter_data := "0000000000000000";
		elsif (t_counter_en = '1') then
			t_counter_data := t_counter_data + 1;
		end if;              
	end if; 
	t_counter_out <= t_counter_data;

end process;

w_counter: process (clk, w_counter_en, reset, w_counter_reset)

variable w_counter_data : std_logic_vector(15 downto 0);

begin

	if (reset = '1') then
		w_counter_data := "0000000000000000";
	elsif (rising_edge(clk)) then
		if (w_counter_reset = '1') then
			w_counter_data := "0000000000000000";
		elsif (w_counter_en = '1') then
			w_counter_data := w_counter_data + 1;
		end if;              
	end if; 
	w_counter_out <= w_counter_data;

end process;
 
fsm_logic : process(clk, reset, t_counter_out, w_counter_out, 				    current_state, enable)

begin

	case current_state is

	when IDLE =>

        	token_out <= '0';
 		t_counter_en <= '0';
		t_counter_reset <= '1';
 		w_counter_en <= '0';
		w_counter_reset <= '1';
        	if (enable = '1') then
         		next_state <= TKN1;
        	else
         		next_state <= IDLE;
		end if;
        
      when TKN1 => 

		token_out <= '1';
 		t_counter_en <= '1';
		t_counter_reset <= '0';
        	if (w_counter_out = tkn_width) then
			w_counter_en <= '0';
			w_counter_reset <= '1';
                	next_state <= TKN0;
        	else
			w_counter_en <= '1';
			w_counter_reset <= '0';
                	next_state <= TKN1;
        	end if;

      when TKN0 => 

		token_out <= '0';
        	w_counter_en <= '0';
		w_counter_reset <= '1';
		if (t_counter_out = tkn_period) then
 			t_counter_en <= '0';
			t_counter_reset <= '1';
                	next_state <= TKN1;
        	else
 			t_counter_en <= '1';
			t_counter_reset <= '0';
                	next_state <= TKN0;
        	end if;

	when others =>

        	token_out <= '0';
 		t_counter_en <= '0';
		t_counter_reset <= '1';
 		w_counter_en <= '0';
		w_counter_reset <= '1';
         	next_state <= IDLE;

	end case;

end process;

end tkn_tx_arch;